--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2021 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
title = "Layout4 APB Master"
require([[defines]]);
require([[functions]]);
--[[
When using this module, all variables (including functions!) must be declared through 
a regular assignment (even assigning nil will do) in a strict scope before being used 
anywhere or assigned to inside a nested scope.
https://github.com/lua-stdlib/strict
]]
local _ENV = require 'std.strict' (_G)	-- strict.lua will detect undeclared global variables
--
--[[
---------------------------------------------------------------------------
Based Program for APB example defined in August 2016 RMC by Bruce Chubb
Version Revision 07/17/2016

Initialize signals to red to cover possibile single track
occupancies at startup resulting in false setting of
traffic sticks, i.e. sticks set for both directions
when shuld be no traffic sticks set
---------------------------------------------------------------------------]]
local Count = 0;
-- These labels appear on each of the tabs for the Function Buttons
local Tab01Label = "Sim Movement";
local Tab02Label = "Not Used";
local Tab03Label = "Not Used";

local Tags = {};		-- table to hold all of the tags detected in sequence
local TagCount = 0;

local FSR = {};	-- table to hold all of the traffic sticks
local Sig = {};	-- table to hold all of the Next Signal Aspects
local SavedSig = {};	-- table to hold the last known Signal Aspects
local BK = {};	-- table to hold all of the Block Occupancy Status
local TUMCNT = {};	-- table to hold all of the Tumbledown Counters
--
-- Traffic Stick Directions
local WEST = 1;
local EAST = 2;
local NDT = 0;		-- No direction-of-traffic
-- state machine states
local IDLE = 0;
local SM = IDLE;
local ErrCount = 0;
local BeepOn = false;
local SoundOff = false;
local EngineSoundSave = {};	-- Engine sound value indexed by EngineNo
local Layout4;
MASTER_TIU = nil;	-- TIU number used by Layout4 (global)
-- clear all tag handling functions
local Func30 = nil;
local Func51 = nil;
local Func10 = nil;
local Func41 = nil;
local Func21 = nil;
local Func20 = nil;
local Func40 = nil;
local Func50 = nil;
local Func60 = nil;
local Func61 = nil;
--
local MyTIUNo, Enum;

--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
	print("setup() Running");
	MASTER_TIU = TIU;
	-- Instrumented Layout 4
	Layout4 = require([[Layout4]]);		-- All of the details about the Layout
	-- an undirected weighted double vertex graph
	MyTIUNo = MASTER_TIU;
	
	ShowSignals();		-- show the Signal Window
	DeleteAllSignals();	-- clear the window of any old signals
	print("Creating Signals");
	-- Layout using the bypass block - Lionel-Ives Block
	-- 10 signals in each direction with 1 head on each signal
	-- Most signals are Permissive
	-- Aignals controlling entrance into head blocks from sidings are Absolute
	-- Route runs EAST-WEST
--[[	
	Initialize signals to red to cover possibile single track
    occupancies at startup resulting in false setting of
    traffic sticks, i.e. sticks set for both directions
    when shuld be no traffic sticks set
]]   

	print("All siding switches to NORMAL");
	Switch(NORMAL, Switches[ALIQUIPPA_WEST][CHAN_NUMBER], Switches[ALIQUIPPA_WEST][AIU_NUMBER], Switches[ALIQUIPPA_WEST][TIU_NUMBER], 0)	
	Switch(NORMAL, Switches[ALIQUIPPA_EAST][CHAN_NUMBER], Switches[ALIQUIPPA_EAST][AIU_NUMBER], Switches[ALIQUIPPA_EAST][TIU_NUMBER], 0)
	Switch(NORMAL, Switches[COLLEGE_WEST][CHAN_NUMBER], Switches[COLLEGE_WEST][AIU_NUMBER], Switches[COLLEGE_WEST][TIU_NUMBER], 0)
	Switch(NORMAL, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0)
	Switch(NORMAL, Switches[STRUTHERS_WEST][CHAN_NUMBER], Switches[STRUTHERS_WEST][AIU_NUMBER], Switches[STRUTHERS_WEST][TIU_NUMBER], 0)
	Switch(NORMAL, Switches[STRUTHERS_EAST][CHAN_NUMBER], Switches[STRUTHERS_EAST][AIU_NUMBER], Switches[STRUTHERS_EAST][TIU_NUMBER], 0)
	Sleep(3);	-- time for all of the switches to align
-----------------------------------------------------------------------------------------------

-- Westbound
	CreateSignal(1, WB, PERMISSIVE_IN_OPEN_COUNTRY, 370,280, "LI", AsStopandProceed, 2, 1);		-- Permissive signal in open country
	FSR[1] = NDT;
	Sig[1] = AsStopandProceed;

	CreateSignal(5, WB, ABSOLUTE_HEAD_BLOCK, 800,260, "YG", AsStopSignal, 1, 0);		-- Absolute Head Block Signal
    FSR[5] = nil;
	Sig[5] = AsStopSignal;

	CreateSignal(9, WB, PERMISSIVE_HEAD_BLOCK, 1190,240, "ST", AsStopandProceed, 3, 9);	-- Permissive head block signal
	FSR[9] = NDT;
	Sig[9] = AsStopandProceed;

	CreateSignal(13, WB, ABSOLUTE_HEAD_BLOCK, 1450,240, "NC", AsStopSignal, 1, 0);		-- Absolute Head Block Signal
	FSR[13] = NDT;
	Sig[13] = AsStopSignal;

	CreateSignal(17, WB, PERMISSIVE_HEAD_BLOCK, 1750,240, "C0", AsStopandProceed, 3, 17);	-- Permissive head block signal
	FSR[17] = NDT;
	Sig[17] = AsStopandProceed;

	CreateSignal(21, WB, APPROACH_TO_HEAD_BLOCK, 1900,240, "BF", AsStopandProceed, 2, 21);	-- Signal in approach to head block signal
	FSR[21] = NDT;
	Sig[21] = AsStopandProceed;

	CreateSignal(25, WB, PERMISSIVE_IN_OPEN_COUNTRY, 2100,240, "MO", AsStopandProceed, 2, 25);	-- Permissive signal in open country
	FSR[25] = NDT;
	Sig[25] = AsStopandProceed;

	CreateSignal(29, WB, ABSOLUTE_HEAD_BLOCK, 2675,240, "AL", AsStopSignal, 1, 0);		-- Absolute Head Block Signal
	FSR[29] = NDT;
	Sig[29] = AsStopSignal;

	CreateSignal(33, WB, PERMISSIVE_HEAD_BLOCK, 3280,240, "JL", AsStopandProceed, 3, 33);	-- Permissive head block signal
	FSR[33] = NDT;
	Sig[33] = AsStopandProceed;

	CreateSignal(37, WB, APPROACH_TO_HEAD_BLOCK, 3430,240, "MK", AsStopandProceed, 2, 37);	-- Signal in approach to head block signal
	FSR[37] = NDT;
	Sig[37] = AsStopandProceed;
-----------------------------------------------------------------------------------------------

-- Eastbound
	
	CreateSignal(2, EB, APPROACH_TO_HEAD_BLOCK, 370,440, "YG", AsStopandProceed, 2, 2);		-- Signal in approach to head block signal
	FSR[2] = NDT;
	Sig[2] = AsStopandProceed;

	CreateSignal(6, EB, PERMISSIVE_HEAD_BLOCK, 800,430, "ST", AsStopandProceed, 3, 6);		-- Permissive head block signal
	FSR[6] = NDT;
	Sig[6] = AsStopandProceed;

	CreateSignal(10, EB, ABSOLUTE_HEAD_BLOCK, 1190,410, "NC", AsStopSignal, 1, 0);		-- Absolute Head Block Signal
	FSR[10] = NDT;
	Sig[10] = AsStopSignal;

	CreateSignal(14, EB, PERMISSIVE_HEAD_BLOCK, 1450,410, "CO", AsStopandProceed, 3, 14);	-- Permissive head block signal
	FSR[14] = NDT;
	Sig[14] = AsStopandProceed;

	CreateSignal(18, EB, ABSOLUTE_HEAD_BLOCK, 1750,410, "BF", AsStopSignal, 1, 0);		-- Absolute Head Block Signal
	FSR[18] = NDT;
	Sig[18] = AsStopSignal;

	CreateSignal(22, EB, PERMISSIVE_IN_OPEN_COUNTRY, 1900,410, "MO", AsStopandProceed, 2, 22);	-- Permissive signal in open country
	FSR[22] = NDT;
	Sig[22] = AsStopandProceed;

	CreateSignal(26, EB, APPROACH_TO_HEAD_BLOCK, 2100,410, "AL", AsStopandProceed, 2, 26);	-- Signal in approach to head block signal
	FSR[26] = NDT;
	Sig[26] = AsStopandProceed;

	CreateSignal(30, EB, PERMISSIVE_HEAD_BLOCK, 2675,410, "JL", AsStopandProceed, 3, 30);	-- Permissive head block signal
	FSR[30] = NDT;
	Sig[30] = AsStopandProceed;

	CreateSignal(34, EB, ABSOLUTE_HEAD_BLOCK, 3280,410, "MK", AsStopSignal, 1, 0);		-- Absolute Head Block Signal
	FSR[34] = NDT;
	Sig[34] = AsStopSignal;

	CreateSignal(38, EB, PERMISSIVE_IN_OPEN_COUNTRY, 3430,410, "LI", AsStopandProceed, 2, 38);	-- Permissive signal in open country
	FSR[38] = NDT;
	Sig[38] = AsStopandProceed;

	print("Signals Created");

-- save the current values
local orig_key, orig_value;
for orig_key, orig_value in pairs(Sig) do
	--print("Signal copy [", orig_key, "] = " , orig_value);
    SavedSig[orig_key] = orig_value;	
	end;
-- Initialize Tumbledown counters
for Signal=1,38 do
	TUMCNT[Signal] = 0;
	end;

-- clear all tag handling functions
Func30 = nil;
Func51 = nil;
Func10 = nil;
Func41 = nil;
Func21 = nil;
Func20 = nil;
Func40 = nil;
Func50 = nil;
Func60 = nil;
Func61 = nil;

-- setup which blocks are occupied
for Block = FIRST_BLOCK, LAST_BLOCK do
	BK[Block] = CLR;
	end;
---[[
-- must ask the dispatcher
print("Select Starting Block Number");
for k, v in pairs(BlockName) do
	if (k <= 100) then
		print(v .. "(" .. k .. ")");
		end;
	end;
--
-- read Starting Block Number
--
local response = InputQuery(title, "Enter 1st Engine Starting Block# (0 for none)", ALIQUIPPA_BLOCK);
if (response == false) then
	return Stop("Canceled");
	end;
local Enum = tonumber(response);
if (Enum ~= nil and Enum ~= 0) then	-- if the result is a block number, then use it, otherwise, ignore it
	if (Enum == STRUTHERS_STATION_TRACK) then Enum = STRUTHERS_BLOCK; end;	-- correct for passing and station tracks
	if (Enum == COLLEGE_STATION_TRACK) then Enum = COLLEGE_BLOCK; end;
	if (Enum == J_L_PASSING_TRACK) then Enum = J_L_BLOCK; end;
	BK[Enum] = BK[Enum] + 1;
	print("1st Starting Block = " .. BlockName[Enum]);
	response = InputQuery(title, "Enter 2nd Engine Starting Block# (0 for none)", YOUNGSTOWN_BLOCK);
	if (response == false) then
		return Stop("Canceled");
		end;
	Enum = tonumber(response);
	if (Enum ~= nil and Enum ~= 0) then	-- if the result is a block number, then use it, otherwise, ignore it
		if (Enum == STRUTHERS_STATION_TRACK) then Enum = STRUTHERS_BLOCK; end;	-- correct for passing and station tracks
		if (Enum == COLLEGE_STATION_TRACK) then Enum = COLLEGE_BLOCK; end;
		if (Enum == J_L_PASSING_TRACK) then Enum = J_L_BLOCK; end;
		BK[Enum] = BK[Enum] + 1;
		print("2nd Starting Block = " .. BlockName[Enum]);
		end;
	end;
	
	
-- now as soon as the first engine passes over a reader, it will change the BK[] Occupancy flags and cause signals to change
	
	return true;		-- false=setup failed, true=setup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
-- SET TRAFFIC STICK UPON SEQUENTIAL TRAIN MOVEMENT	
-- use (western side block, eastern side block, westbound signal #, eastbound signal #)
function SetTrafficStick(Block1, Block2, Sig1, Sig2)
if (((BK[Block1] ~= CLR) and (BK[Block2] ~= CLR)) and ((FSR[Sig1] == NDT) and (FSR[Sig2] == NDT))) then
	if (Sig[Sig1] ~= AsStopSignal and Sig[Sig1] ~= AsStopandProceed) then
		--print("Traffic Stick at signal #", Sig1, " set to WEST");
		FSR[Sig1] = WEST;
		end;
	if (Sig[Sig2] ~= AsStopSignal and Sig[Sig2] ~= AsStopandProceed) then
		--print("Traffic Stick at signal #", Sig2, " set to EAST");
		FSR[Sig2] = EAST;
		end;
	end;	
end;
--
local TUMMAX = 15;  --maximum value for tumbledown delay counter
--
function PermissiveinOpenCountry(NextBlock, ThisSig, OppositeSig, NextSig, MyDir, OppositeDir)	-- Uses Traffic Sticks
	if (FSR[OppositeSig] == nil) then print("Traffic Stick nil in PermissiveinOpenCountry - Opposite Side #", OppositeSig); end;
	if (FSR[NextSig] == nil) then print("Traffic Stick nil in PermissiveinOpenCountry  - Next Signal This Side # ", NextSig); end;
-- Calculate Signal Permissive in open country using Traffic stick on opposite and next signal
    if (BK[NextBlock] ~= CLR) then goto POCAsStopandProceed; end;
    if (FSR[OppositeSig] == OppositeDir) then goto POCAsStopandProceed; end;
    if (Sig[NextSig] ~= AsStopSignal and Sig[NextSig] ~= AsStopandProceed) then Sig[ThisSig] = AsClear; TUMCNT[ThisSig] = 0; goto POCDone; end;
    if (FSR[NextSig] == MyDir) then Sig[ThisSig] = AsApproach; TUMCNT[ThisSig] = 0; goto POCDone; end;
::POCAsStopandProceed::
    if (Sig[ThisSig] == AsStopandProceed) then goto POCDone; end;
    if (TUMCNT[ThisSig] == TUMMAX) then Sig[ThisSig] = AsStopandProceed; TUMCNT[ThisSig] = 0; goto POCDone; end;
    TUMCNT[ThisSig] = TUMCNT[ThisSig] + 1;
::POCDone::	  
	  return;
end;

function HeadBlockPermissive(NextBlock, ThisSig, OppositeSig, NextSig, Switch1, Switch2, MyDir, OppositeDir)
--Calculate Signal Head Block Permissive - no Traffic Stick involved
    if (BK[NextBlock] ~= CLR) then goto HBPAsStopandProceed; end;
    --if (GetSwitchStatus(Switches[Switch1][CHAN_NUMBER], Switches[Switch1][AIU_NUMBER], Switches[Switch1][TIU_NUMBER]) == REVERSE or 
	--	GetSwitchStatus(Switches[Switch2][CHAN_NUMBER], Switches[Switch2][AIU_NUMBER], Switches[Switch2][TIU_NUMBER]) == REVERSE) then goto HBPAsStopandProceed; end;
    if (GetSwitchStatus(Switches[Switch1][CHAN_NUMBER], Switches[Switch1][AIU_NUMBER], Switches[Switch1][TIU_NUMBER]) == REVERSE) 
		then goto HBPAsStopandProceed; end;
    if (Sig[NextSig] ~= AsStopSignal  and Sig[NextSig] ~= AsStopandProceed) then Sig[ThisSig] = AsClear; else Sig[ThisSig] = AsApproach; end
    TUMCNT[ThisSig] = 0;
	goto HBPDone;	  
::HBPAsStopandProceed::
    if (Sig[ThisSig] == AsStopandProceed) then goto HBPDone; end;
    if (TUMCNT[ThisSig] == TUMMAX) then Sig[ThisSig] = AsStopandProceed; TUMCNT[ThisSig] = 0; goto HBPDone; end;
    TUMCNT[ThisSig] = TUMCNT[ThisSig] + 1;
::HBPDone::
	return;
end;

function HeadBlockAbsolute(NextBlock, ThisSig, OppositeSig, NextSig, MyDir, OppositeDir)	-- Uses Traffic Sticks
-- THIS FUNCTION FAILS if less than 3 blocks between sidings
	if (FSR[NextSig] == nil) then print("Traffic Stick nil in HeadBlockAbsolute - Next Signal This Side #", NextSig); end;
--Calculate Signal Head Block Absolute using Traffic Stick on next signal
    if (BK[NextBlock] ~= CLR) then goto HBAAsStop; end; -- Stop - next block is occupied by virtue of a train being in the block
	-- Otherwise the next block is unoccupied
	--
	-- if next signal (signal in advance of ThisSig) is not Stop, then set signal to clear indicating we are clear two blocks ahead with no opposing traffic
    if (Sig[NextSig] ~= AsStopSignal and Sig[NextSig] ~= AsStopandProceed) then Sig[ThisSig] = AsClear; TUMCNT[ThisSig] = 0; goto HBADone; end;
	
	-- if traffic stick ahead indicates I'm following traffic then set signal to Approach indicating we are clear one block ahead and must prepare to stop.
    if (FSR[NextSig] == MyDir) then Sig[ThisSig] = AsApproach; TUMCNT[ThisSig] = 0; goto HBADone; end;
	-- if next signal is Stop and traffic stick indicates opposing traffic, then set signal to Stop by falling through to next code
::HBAAsStop::
    if (Sig[ThisSig] == AsStopSignal) then goto HBADone; end;
    if (TUMCNT[ThisSig] == TUMMAX) then Sig[ThisSig] = AsStopSignal; TUMCNT[ThisSig] = 0; goto HBADone; end;
    TUMCNT[ThisSig] = TUMCNT[ThisSig] + 1;
::HBADone::
	return;
end;

function HeadBlockAbsolute1(NextBlock, ThisSig, OppositeSig, NextSig, Switch1, Switch2, MyDir, OppositeDir)
--Calculate Signal Head Block Absolute when there is 1 block between sidings - no Traffic Stick involved
    if (BK[NextBlock] ~= CLR) then goto HBA1AsStop; end; -- Stop - next block is occupied by virtue of a train being in the block
	-- No train in the through track, check the passing siding
    --if (GetSwitchStatus(Switches[Switch1][CHAN_NUMBER], Switches[Switch1][AIU_NUMBER], Switches[Switch1][TIU_NUMBER]) == REVERSE or 
	--	GetSwitchStatus(Switches[Switch2][CHAN_NUMBER], Switches[Switch2][AIU_NUMBER], Switches[Switch2][TIU_NUMBER]) == REVERSE) then goto HBA1AsStop; end;
    if (GetSwitchStatus(Switches[Switch1][CHAN_NUMBER], Switches[Switch1][AIU_NUMBER], Switches[Switch1][TIU_NUMBER]) == REVERSE)
		then goto HBA1AsStop; end;
		-- Stop - next block is occupied by virtue of a train being in the next siding and the crew has aligned one of the switches for the siding
	-- Otherwise the next block is unoccupied
	--
	-- if next signal (signal in advance of ThisSig) is not Stop, then set signal to clear indicating we are clear two blocks ahead with no opposing traffic
	-- else next signal (signal in advance of ThisSig) is Stop so set signal to Approach indicating we are clear one block ahead and must prepare to stop.
    if (Sig[NextSig] ~= AsStopSignal and Sig[NextSig] ~= AsStopandProceed) then Sig[ThisSig] = AsClear; else Sig[ThisSig] = AsApproach; end
    TUMCNT[ThisSig] = 0; 
	goto HBA1Done;
::HBA1AsStop::
    if (Sig[ThisSig] == AsStopSignal) then goto HBA1Done; end;
    if (TUMCNT[ThisSig] == TUMMAX) then Sig[ThisSig] = AsStopSignal; TUMCNT[ThisSig] = 0; goto HBA1Done; end;
    TUMCNT[ThisSig] = TUMCNT[ThisSig] + 1;
::HBA1Done::
	return;
end;

function ApproachtoHeadBlock(NextBlock, ThisSig, OppositeSig, NextSig, MyDir, OppositeDir)	-- Uses Traffic Sticks
	if (FSR[OppositeSig] == nil) then print("Traffic Stick nil in ApproachtoHeadBlock - Opposite Side #", OppositeSig); end;
--Calculate Signal Approach to Head Block using Traffic Stick on opposite signal
    if (BK[NextBlock] ~= CLR) then goto AHBAsStopandProceed; end;
    if (FSR[OppositeSig] == OppositeDir) then goto AHBAsStopandProceed; end;
    if (Sig[NextSig] ~= AsStopSignal and Sig[NextSig] ~= AsStopandProceed) then Sig[ThisSig] = AsClear; else Sig[ThisSig] = AsApproach; end
    TUMCNT[ThisSig] = 0; 
	goto AHBDone;
::AHBAsStopandProceed::
    if (Sig[ThisSig] == AsStopandProceed) then goto AHBDone; end;
    if (TUMCNT[ThisSig] == TUMMAX) then Sig[ThisSig] = AsStopandProceed; TUMCNT[ThisSig] = 0; goto AHBDone; end;
    TUMCNT[ThisSig] = TUMCNT[ThisSig] + 1;
::AHBDone::	  
	return;
end;

--[[---------------------------------------------------------------------------------------]]

function loop(Engine, TIU)
-- Always call Sleep(X) at least once in this function if it returns true;
--print("loop() Running ", RunTime(), " seconds");

-- Read Occupancy flags
-- This is all done in the tag() function starting from initial occupancy conditions

-- SET TRAFFIC STICK UPON SEQUENTIAL TRAIN MOVEMENT
-- use (western side block, eastern side block, westbound signal #, eastbound signal #)
SetTrafficStick(LIONEL_IVES_BLOCK, YOUNGSTOWN_BLOCK, 1, 2);
--SetTrafficStick(YOUNGSTOWN_BLOCK, STRUTHERS_BLOCK, 5, 6);
--SetTrafficStick(STRUTHERS_BLOCK, NEW_CASTLE_BLOCK, 9, 10);
--SetTrafficStick(NEW_CASTLE_BLOCK, COLLEGE_BLOCK, 13, 14);
--SetTrafficStick(COLLEGE_BLOCK, BEAVER_FALLS_BLOCK , 17, 18);
SetTrafficStick(BEAVER_FALLS_BLOCK, MONACA_BLOCK, 21, 22);
SetTrafficStick(MONACA_BLOCK, ALIQUIPPA_BLOCK, 25, 26);
--SetTrafficStick(ALIQUIPPA_BLOCK, J_L_BLOCK, 29, 30);
--SetTrafficStick(J_L_BLOCK, MCKEES_ROCKS_BLOCK , 33, 34);
SetTrafficStick(MCKEES_ROCKS_BLOCK, LIONEL_IVES_BLOCK, 37, 38);

---[[
-- Original set of Signals with three sidings
-- CALCULATE WESTBOUND SIGNAL ASPECTS
ApproachtoHeadBlock(MCKEES_ROCKS_BLOCK, 37, 38, 33, WEST, EAST);
HeadBlockPermissive(J_L_BLOCK, 33, 34, 29, ALIQUIPPA_WEST, ALIQUIPPA_EAST, WEST, EAST);
HeadBlockAbsolute(ALIQUIPPA_BLOCK, 29, 30, 25, WEST, EAST);
PermissiveinOpenCountry(MONACA_BLOCK, 25, 26, 21, WEST, EAST);
ApproachtoHeadBlock(BEAVER_FALLS_BLOCK, 21, 22, 17, WEST, EAST);
HeadBlockPermissive(COLLEGE_BLOCK, 17, 18, 13, COLLEGE_WEST, COLLEGE_EAST, WEST, EAST); 
  HeadBlockAbsolute1(NEW_CASTLE_BLOCK, 13, 14, 9, WEST, EAST);	-- 1 block between sidings
HeadBlockPermissive(STRUTHERS_BLOCK, 9, 10, 5, STRUTHERS_WEST, STRUTHERS_EAST, WEST, EAST);
HeadBlockAbsolute(YOUNGSTOWN_BLOCK, 5, 6, 1, WEST, EAST);
PermissiveinOpenCountry(LIONEL_IVES_BLOCK, 1, 2, 37, WEST, EAST);

-- CALCULATE EASTBOUND SIGNAL ASPECTS
ApproachtoHeadBlock(YOUNGSTOWN_BLOCK, 2, 1, 6, EAST, WEST);
HeadBlockPermissive(STRUTHERS_BLOCK, 6, 5, 10, STRUTHERS_EAST, STRUTHERS_WEST, EAST, WEST);
  HeadBlockAbsolute1(NEW_CASTLE_BLOCK, 10, 9, 14, EAST, WEST);	-- 1 block between sidings
HeadBlockPermissive(COLLEGE_BLOCK, 14, 13, 18, COLLEGE_EAST, COLLEGE_WEST,	EAST, WEST);
HeadBlockAbsolute(BEAVER_FALLS_BLOCK, 18, 17, 22, EAST, WEST);
PermissiveinOpenCountry(MONACA_BLOCK, 22, 21, 26, EAST, WEST);
ApproachtoHeadBlock(ALIQUIPPA_BLOCK, 26, 25, 30, EAST, WEST);
HeadBlockPermissive(J_L_BLOCK, 30, 29, 34, ALIQUIPPA_EAST, ALIQUIPPA_WEST, EAST, WEST);
HeadBlockAbsolute(MCKEES_ROCKS_BLOCK, 34, 33, 38, EAST, WEST);
PermissiveinOpenCountry(LIONEL_IVES_BLOCK, 38, 37, 2, EAST, WEST);
--]]

-- CLEAR TRAFFIC STICK FOLLOWING SIGNAL UPGRADES
--[[
for Signal=1,38 do
	if (Sig[Signal] ~= nil) then
		if ((Sig[Signal] ~= AsStopSignal and Sig[Signal] ~= AsStopandProceed) and (TUMCNT[Signal] == 0)) then FSR[Signal] = NDT; end;
		end;
	end;
--]]
if (Sig[1] ~= AsStopSignal and Sig[1] ~= AsStopandProceed and TUMCNT[1] == 0) then FSR[1] = NDT; end;
if (Sig[2] ~= AsStopSignal and Sig[2] ~= AsStopandProceed and TUMCNT[2] == 0) then FSR[2] = NDT; end;

if (Sig[21] ~= AsStopSignal and Sig[21] ~= AsStopandProceed and TUMCNT[21] == 0) then FSR[21] = NDT; end;
if (Sig[22] ~= AsStopSignal and Sig[22] ~= AsStopandProceed and TUMCNT[22] == 0) then FSR[22] = NDT;	end;
	
if (Sig[25] ~= AsStopSignal and Sig[25] ~= AsStopandProceed and TUMCNT[25] == 0) then FSR[25] = NDT;	end;
if (Sig[26] ~= AsStopSignal and Sig[26] ~= AsStopandProceed and TUMCNT[26] == 0) then FSR[26] = NDT;	end;
	
if (Sig[37] ~= AsStopSignal and Sig[37] ~= AsStopandProceed and TUMCNT[37] == 0) then FSR[37] = NDT;	end;
if (Sig[38] ~= AsStopSignal and Sig[38] ~= AsStopandProceed and TUMCNT[38] == 0) then FSR[38] = NDT;	end;
	
-- Display signal aspects
for Signal=1,38 do
	if (Sig[Signal] ~= nil) then	-- nil means no change
		if (SavedSig[Signal] ~= Sig[Signal]) then	-- if no change, don't need to SetAspect
			SetAspect(Signal, Sig[Signal]);
			--print("SetAspect on signal # ", Signal, " to ", AspectNames[Sig[Signal]]);
			print("(", RunTime(),") SetAspect on signal # ", Signal, " to ", AspectNames[Sig[Signal]], " was ", AspectNames[SavedSig[Signal]]);
			SavedSig[Signal] = Sig[Signal];	-- save current aspect
			end;
		end;
	end;
-- consider approach lighting but that is hard to do since
-- we don't have a connection between block and the signals at each end
-- of the block (on opposite sides)
	
Sleep(0.1);

return true;
end
--[[---------------------------------------------------------------------------------------]]

function tag(Detector, Reader, EngineNo, TagLocation, CarModel, Railroad, CarNumber, TagPacket)
-- This function is called each time a tag programmed with 32 bytes of data is detected (TAGDATABLOCK4)
-- TagPacket contains 8 characters (4 hex digits) of UID and 32 characters
-- of block 4 data from the programmed tag.

BumpCounter(COUNTER01);	
local PacketLen = #TagPacket;

if (Debug() >= 8) then print("Packet Length = ", PacketLen); end;
if (Debug() >= 7) then print(TagPacket); end;

	-- index the table Tags[] by tag ID, value is the tag packet itself
	Tags[string.sub(TagPacket, 1, 8)] = string.sub(TagPacket, 9, 40);	-- remove checksum and EOP
	TagCount = TagCount + 1;	-- count the number of tags
	if (EngineNo > 0) then	-- its an engine
		if (BeepOn) then Beep(ASTERISK); end;
		-- next line shows a different way to quote a string using brackets
		printc(clGreen, string.format([[tag(%.2f) : Detector %d Reader %d Eng#%d %s %s #%s %s]],
			RunTime(), Detector, Reader, EngineNo, GetRailroadName(Railroad), GetEngineName(CarModel), CarNumber, GetTagLocation(TagLocation)));
		-- follow on to process the detected engine
		else	-- it's a car
		if (BeepOn) then Beep(EXCLAMATION); end;
		printc(clBlue, string.format([[tag(%.2f) : Detector %d Reader %d Car %s %s #%s]],
			RunTime(), Detector, Reader, GetRailroadName(Railroad), GetCarName(CarModel), CarNumber));
		-- follow on to process non-engine tags
		end

	-- The complete packet received is in TagPacket 
	-- (all of the fields have already been extracted into the first parameters to tag()
	--   First 8 digits are the 4 byte Tag UID in hexidecimal
	--   Next 32 digits are the 16 bytes of Block 4 Information read from the Tag in hexidecimal
	if (Debug() >= 6) then print(string.format("Packet  %s  %s", string.sub(TagPacket, 1, 8),
		string.sub(TagPacket,9,40) )); end


-- Tag 3-0
if (Detector == 3 and Reader == 0) then
	if (Func30 == nil) then
		Func30 = createMainLine(false);
		end;
	Func30("@ 3-0 ", EngineNo, TagLocation, CarModel, LIONEL_IVES_BLOCK, MCKEES_ROCKS_BLOCK);

-- Tag 5-1
elseif (Detector == 5 and Reader == 1) then
	if (Func51 == nil) then
		Func51 = createMainLine(false);
		end;
	Func51("@ 5-1 ", EngineNo, TagLocation, CarModel, MCKEES_ROCKS_BLOCK, J_L_BLOCK);

-- Tag 1-0
elseif (Detector == 1 and Reader == 0) then
	if (Func10 == nil) then
		Func10 = createMainLine(false); 
		end;	
	Func10("@ 1-0 ", EngineNo, TagLocation, CarModel, J_L_BLOCK, ALIQUIPPA_BLOCK); 

-- Tag 6-0
elseif (Detector == 6 and Reader == 0) then
	if (Func60 == nil) then
		Func60 = createMainLine(false);
		end;
	Func60("@ 6-0 ", EngineNo, TagLocation, CarModel, ALIQUIPPA_BLOCK, MONACA_BLOCK);
	
-- Tag 6-1
elseif (Detector == 6 and Reader == 1) then
	if (Func61 == nil) then
		Func61 = createMainLine(false);	
		end;
	Func61("@ 6-1 ", EngineNo, TagLocation, CarModel, MONACA_BLOCK, BEAVER_FALLS_BLOCK);

-- Tag 4-1
elseif (Detector == 4 and Reader == 1) then
	if (Func41 == nil) then
		Func41 = createMainLine(false);
		end;
	Func41("@ 4-1 ", EngineNo, TagLocation, CarModel, BEAVER_FALLS_BLOCK, COLLEGE_BLOCK);

-- Tag 2-1
elseif (Detector == 2 and Reader == 1) then
	if (Func21 == nil) then
		Func21 = createMainLine(false);
		end;	
	Func21("@ 2-1 ", EngineNo, TagLocation, CarModel, COLLEGE_BLOCK, NEW_CASTLE_BLOCK);

-- Tag 2-0
elseif (Detector == 2 and Reader == 0) then
	if (Func20 == nil) then
		Func20 = createMainLine(false);
		end;
	Func20("@ 2-0 ", EngineNo, TagLocation, CarModel, NEW_CASTLE_BLOCK, STRUTHERS_BLOCK);

-- Tag 4-0
elseif (Detector == 4 and Reader == 0) then
	if (Func40 == nil) then
		Func40 = createMainLine(false);
		end;
	Func40("@ 4-0 ", EngineNo, TagLocation, CarModel, STRUTHERS_BLOCK, YOUNGSTOWN_BLOCK);

-- Tag 5-0
elseif (Detector == 5 and Reader == 0) then
	if (Func50 == nil) then
		Func50 = createMainLine(false);
		end;
	Func50("@ 5-0 ", EngineNo, TagLocation, CarModel, YOUNGSTOWN_BLOCK, LIONEL_IVES_BLOCK);
	
	end;

return true;	-- true=continue to process tags, false=stop processing tags
end			
--[[---------------------------------------------------------------------------------------]]

function createMainLine(flag)
	local WaitforCaboose = flag;
	local MyLastBlockNo;
	local function MainLine(DetectorID, EngineNo, TagLocation, CarModel, Block1, Block2)
		---- uses globals: BK[]
		if (isEngine(EngineNo) and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if its an engine
			print("MainLine : " .. DetectorID .. "Engine " .. EngineNo .. " detected");
			MyLastBlockNo = nil;	-- reset last block number
			if (BK[Block1] == CLR) then
				print("MainLine : occupying " .. BlockName[Block1] .. "(" .. Block1 .. ")");
				BK[Block1] = BK[Block1] + 1;
				MyLastBlockNo = Block2;-- save current state for the Caboose detection call of this function
				-- save current state for the Caboose detection call of this function
				WaitforCaboose = true;
				return true;
				end;
			if (BK[Block2] == CLR) then
				print("MainLine : occupying " .. BlockName[Block2] .. "(" .. Block2 .. ")");
				BK[Block2] = BK[Block2] + 1;
				MyLastBlockNo = Block1;-- save current state for the Caboose detection call of this function
				-- save current state for the Caboose detection call of this function
				WaitforCaboose = true;
				return true;
				end;
			---[[
			if (BK[Block1] ~= CLR and BK[Block2] ~= CLR) then
				-- special case of engine entering a siding block with the other track occupied by another train
				-- occupy the siding and clear the head block when a caboose comes along
				if (Block1 == ALIQUIPPA_BLOCK or
					Block1 == MCKEES_ROCKS_BLOCK or
					Block1 == BEAVER_FALLS_BLOCK or
					Block1 == NEW_CASTLE_BLOCK or
					Block1 == YOUNGSTOWN_BLOCK) then
					print("MainLine : occupying " .. BlockName[Block2] .. "(" .. Block2 .. ")");
					BK[Block2] = BK[Block2] + 1;	-- bump the siding occupancy count					
					MyLastBlockNo = Block1;			-- setup to decrement the head block occupancy count
					end;
				if (Block2 == ALIQUIPPA_BLOCK or
					Block2 == MCKEES_ROCKS_BLOCK or
					Block2 == BEAVER_FALLS_BLOCK or
					Block2 == NEW_CASTLE_BLOCK or
					Block2 == YOUNGSTOWN_BLOCK) then
					print("MainLine : occupying " .. BlockName[Block1] .. "(" .. Block1 .. ")");
					BK[Block1] = BK[Block1] + 1;	-- bump the siding occupancy count					
					MyLastBlockNo = Block2;			-- setup to decrement the head block occupancy count
					end;
				end;
			--]]
			-- save current state for the Caboose detection call of this function
			WaitforCaboose = true;
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			print("MainLine : " .. DetectorID .. "Caboose detected");
			BK[MyLastBlockNo] = BK[MyLastBlockNo] - 1;	-- decrement the occupancy count
			print("MainLine : clearing " .. BlockName[MyLastBlockNo] .. "(" .. MyLastBlockNo .. ")");
			MyLastBlockNo = nil;	-- once MyLastBlockNo is used, set it to nil to catch possible future errors
			WaitforCaboose = false;
			end;
	return true;
	end;

return MainLine;
end;
--[[---------------------------------------------------------------------------------------]]

function cleanup(Engine, TIU)
	-- this function is called once when the user presses the [STOP] button or the Stop() function is called
	print("cleanup() Running");
	if (ErrCount == 0) then
		print("No errors");
		else
		print(ErrCount .. " Error(s)");
		end;	
	return true;		-- false=cleanup failed, true=cleanup succeeded
	end
--[[---------------------------------------------------------------------------------------]]

--[==[
function F01()
	print("College East switch to reverse");
	Switch(REVERSE, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0)
	return true;
end
Function01Name, Function01Label = F01, "College E REV";
--[[---------------------------------------------------------------------------------------]]
function F02()
	print("Engine into Beaver Falls Block");
	BK[BEAVER_FALLS_BLOCK] = OCC;
	return true;
end
Function02Name, Function02Label = F02, "Eng to BF";
--[[---------------------------------------------------------------------------------------]]
function F03()
	print("Caboose into Beaver Falls Block");
	BK[COLLEGE_BLOCK] = CLR;
	return true;
end
Function03Name, Function03Label = F03, "Cab to BF";
--[[---------------------------------------------------------------------------------------]]
function F04()
	print("College East switch to normal");
	Switch(NORMAL, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0)
	return true;
end
Function04Name, Function04Label = F04, "College E NOR";
--[[---------------------------------------------------------------------------------------]]
function F05()
	print("Engine moves from BF to MO");
	BK[MONACA_BLOCK] = OCC;
	return true;
end
Function05Name, Function05Label = F05, "Eng to MO";
--[[---------------------------------------------------------------------------------------]]
function F06()
	print("Caboose moves from BF to MO");
	BK[BEAVER_FALLS_BLOCK] = CLR;
	return true;
end
Function06Name, Function06Label = F06, "Cab to MO";
--]==]
--[[---------------------------------------------------------------------------------------]]
function ReloadButton()
	print("Reloading layout4.lua");
	package.loaded["layout4"] = nil;	-- unload
	Layout4 = require([[layout4]]);	-- All of the details about the Layout
	return true;
end
Function11Name, Function11Label = ReloadButton, "Reload";
--[[---------------------------------------------------------------------------------------]]

function BeepOffButton()
	print("Beep Off");
	BeepOn = false;
	return true;
end
Function14Name, Function14Label = BeepOffButton, "Beep Off";
--[[---------------------------------------------------------------------------------------]]

function BeepOnButton()
	print("Beep On");
	BeepOn = true;
	return true;
end
Function15Name, Function15Label = BeepOnButton, "Beep On";
--[[---------------------------------------------------------------------------------------]]

function OccButton(Eng, TIU)
printc(clGreen, "Detection Status -------------------------");
local x, FColor;
for x = FIRST_BLOCK, LAST_BLOCK do
	local flag = BK[x];
	if (flag == CLR) then
		FColor = clBlack;
		else
		FColor = clRed;
		end;
	printc(FColor, BlockName[x] .. "(" .. x .. ") = ", flag)
	end;	
return true;
end
Function01Name, Function01Label = OccButton, "Detection Status";
--[[---------------------------------------------------------------------------------------]]

function F02()
local response = InputBox(title, "Occupy Block Number?", ALIQUIPPA_BLOCK);
local Enum = tonumber(response);
if (Enum ~= nil) then	-- if the result is a number, then use it, otherwise, ignore it
	BK[Enum] = BK[Enum] + 1;
	print("Occupy " .. BlockName[Enum] .. "(" .. Enum .. ")");
	end
return true;
end
Function02Name, Function02Label = F02, "Occupy Block";
--[[---------------------------------------------------------------------------------------]]
function F03()
local response = InputBox(title, "Clear Block Number?", ALIQUIPPA_BLOCK);
local Enum = tonumber(response);
if (Enum ~= nil) then	-- if the result is a number, then use it, otherwise, ignore it
	BK[Enum] = CLR;
	print("Clear " .. BlockName[Enum] .. "(" .. Enum .. ")");
	end
return true;
end
Function03Name, Function03Label = F03, "Clear Block";
--[[---------------------------------------------------------------------------------------]]
function F04()
for Block = FIRST_BLOCK, LAST_BLOCK do
	BK[Block] = CLR;
	end;
print("All blocks manually cleared");
return true;
end
Function04Name, Function04Label = F04, "Clear All Blocks";
--[[---------------------------------------------------------------------------------------]]
--
-- Test routine to accept events from "Layout 4 Block Occupancy.lua"
--
function event(FromPC, Type, P1, P2, P3)
	print("event() - From PC# ", FromPC, " EventType = ", Type, " Parameters = ", P1, " ", P2, " ", P3);
	if (Type == BLOCK_OCCUPANCY_EVENT) then
		if (P1 ~= 0) then
			if (P2 >= CLR) then
				BK[P1] = P2;	-- set occupancy flag
				end;
			end;
		end;
	if (Type == TAG_EVENT) then
		-- Detector = P1
		-- Reader	= P2
		-- P3 ~= 0 means engine, P3 == 0 means CABOOSE
		-- Engine Number	1561				NOT_AN_ENGINE
		-- CarModel			UNKNOWN_CARMODEL	CABOOSE
		-- Car Number		1561				123
		if (P3 ~= NOT_AN_ENGINE) then	-- engine
			tag(P1, P2, 1561, TAG_ON_FRONT_TRUCK, UNKNOWN_CARMODEL, P_AND_LE, "1561", "NONE")	;
			else			-- caboose
			tag(P1, P2, NOT_AN_ENGINE, TAG_ON_FRONT_TRUCK, CABOOSE, P_AND_LE, "123", "NONE")	;
			end;
		end;
	return true;		-- false=event failed, true=event succeeded	
	end

--[[---------------------------------------------------------------------------------------]]