--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2020 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
title = "Occupancy Display"
require([[defines]]);
require([[functions]]);

--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
print("setup() Running");
ASTER_TIU = TIU;
-- Instrumented Layout 4
Layout4 = require([[Layout4]]);		-- All of the details about the Layout
-- an undirected weighted double vertex graph
MyTIUNo = MASTER_TIU;
MyEngineNo = Engine;
return true;		-- false=setup failed, true=setup succeeded
end
--[[---------------------------------------------------------------------------------------]]
function loop(Engine, TIU)
-- Always call Sleep(X) at least once in this function if it returns true;
printc(clGreen, "Occupancy Flags -------------------------");
for x = FIRST_BLOCK, LAST_BLOCK do
	flag = GetGlobalInt(x);
	if (flag == 0) then
		FColor = clBlack;
		else
		FColor = clRed;
		end;
	printc(FColor, BlockName[x] .. " = ", flag)
	end;
Sleep(4);
return true;
end
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
-- this function is called once when the user presses the [STOP] button or the Stop() function is called
print("Done");
return true;		-- false=cleanup failed, true=cleanup succeeded
end
--[[---------------------------------------------------------------------------------------]]
function InitButton(Eng, TIU)
print("Clear Occupancy Flags");
-- clear occupancy flags
local x;
for x = FIRST_BLOCK, LAST_BLOCK do
	OverrideOccupancy(x, 0);
	end;
print("Occupancy Flags cleared");
return true;
end
Function06Name, Function06Label = InitButton, "Clear Flags";
--[[---------------------------------------------------------------------------------------]]
--[[---------------------------------------------------------------------------------------]]
-- clear a block occupancy flag to zero if we have ownership
function ClearOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return Stop("Invalid Block Number in ClearOccupancy()");
	end;
if (value ~= nil and GetGlobalInt(block) == value) then
	print("Clear occupancy flag on " .. BlockName[block]);
	SetGlobalInt(block, 0);
	end;
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- set a block occupancy flag without waiting for lock or checking ownership
function OverrideOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return Stop("Invalid Block Number in ClearOccupancy()");
	end;
if (value == nil) then
	value = 0;
	end;
print("Override occupancy flag on " .. BlockName[block]);
SetGlobalInt(block, value);
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- fetch a block occupany flag
function GetOccupancy(block)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return Stop("Invalid Block Number in GetOccupancy()");
	end;
return GetGlobalInt(block);
end;
--[[---------------------------------------------------------------------------------------]]

--[[---------------------------------------------------------------------------------------]]

function ABButton()
local value = 99;
local block = ALIQUIPPA_BLOCK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;	
end
Function01Name, Function01Label = ABButton, "Aliquippa Block";
--[[---------------------------------------------------------------------------------------]]
function YButton()
local value = 99;
local block = YOUNGSTOWN_BLOCK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;
end
Function02Name, Function02Label = YButton, "Youngstown Block";
--[[---------------------------------------------------------------------------------------]]
function NCButton()
local value = 99;
local block = NEW_CASTLE_BLOCK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;
end
Function03Name, Function03Label = NCButton, "New Castle Block";
--[[---------------------------------------------------------------------------------------]]

function JPassButton()
local value = 99;
local block = J_L_PASSING_TRACK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;
end
Function04Name, Function04Label = JPassButton, "J&L Passing Track";
--[[---------------------------------------------------------------------------------------]]
function JThruButton()
local value = 99;
local block = J_L_THRU_TRACK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;
end
Function05Name, Function05Label = JThruButton, "J&L Thru Track";
--[[---------------------------------------------------------------------------------------]]

function SPassButton()
local value = 99;
local block = STRUTHERS_PASSING_TRACK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;
end
Function07Name, Function07Label = SPassButton, "Struthers Passing Track";
--[[---------------------------------------------------------------------------------------]]

function SThruButton()
local value = 99;
local block = STRUTHERS_THRU_TRACK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;
end
Function08Name, Function08Label = SThruButton, "Struthers Thru Track";
--[[---------------------------------------------------------------------------------------]]


function CPassButton()
local value = 99;
local block = COLLEGE_PASSING_TRACK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;
end
Function09Name, Function09Label = CPassButton, "College Passing Track";
--[[---------------------------------------------------------------------------------------]]

function CThruButton()
local value = 99;
local block = COLLEGE_THRU_TRACK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;
end
Function10Name, Function10Label = CThruButton, "College Thru Track";
--[[---------------------------------------------------------------------------------------]]

function McKRButton()
local value = 99;
local block = MCKEES_ROCKS_BLOCK;
if (GetGlobalInt(block) == 0) then
	print("Set " .. BlockName[block] .. " Occupancy");
	if (not TrySetGlobalInt(block, value)) then
		print("Can't occupy " .. BlockName[block]);
		return true;
		end;
	print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
	else
	SetGlobalInt(block, 0);
	print("Clear occupancy flag on " .. BlockName[block]);
	end;
return true;
end
Function11Name, Function11Label = McKRButton, "McKees Rocks Block";
--[[---------------------------------------------------------------------------------------]]