--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2019 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
title = "DCH Accuracy Test"
require("defines");
--[[
Compares the DCH timer in the engine with the PC's clock
and displays the results
--]]
Counter01Label = "Loop Number";
Counter02Label = "Test Number";

LOOPCOUNTER = COUNTER01;
TESTCOUNTER = COUNTER02;

StartDCH = 0;
NowDCH = 0;
StartTime = 0;
NowTime = 0;
CalcTime = 0;
LoopCount = 50;
--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
	print(os.date("setup() %d %b %Y   %X"));
	MyTIU = TIU;
	MyEngine = Engine;
	StartTime = os.time();
	StartDCH = DCH(MyEngine, MyTIU);
	return true;		-- false=setup failed, true=setup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
----[==[
-- NOTE: using loop() call as part of the timing shows that it takes
-- about 1 second for C code to execute the loop back
-- When running this code, the os.time() value and the DCH() value kept
-- pretty close but the the CalcTime started to fall behind.
-- but that is OK as the important check is between the differences of os.time()
-- and DCH()
function loop()	-- parameters not used
	-- Always call Sleep() at least once in this function since it returns true;
	--print(os.date("loop() %d %b %Y   %X"));
	Sleep(LoopCount);
	CalcTime = CalcTime + LoopCount;
	NowTime = os.time();
	NowDCH = DCH(MyEngine, MyTIU);
	T1 = NowTime - StartTime;
	D1 = NowDCH - StartDCH
	print(CalcTime, "  " , T1, " ", D1, " Error = ", 100*(D1-T1)/D1, "%");
	BumpCounter(LOOPCOUNTER);		-- bump loop counter
	return true;	-- false=do not loop, true=continue to loop
	end
--]==]
--[[---------------------------------------------------------------------------------------]]
--[==[
-- NOTE: this version of loop() runs the test entirely in the loop() function
-- never using the C code that calls loop()
-- The code still has an overhead of about a second so the elapsed time slowly
-- increases.
-- but that is OK as the important check is between the differences of os.time()
-- and DCH()
function loop()	-- parameters not used 
	-- Always call Sleep() at least once in this function since it returns true;
	print(os.date("loop() %d %b %Y   %X"));

	StartTime = os.time();
	StartDCH = DCH(MyEngine, MyTIU);
	while(true) do
		Sleep(LoopCount);
		CalcTime = CalcTime + LoopCount;
		NowTime = os.time();
		NowDCH = DCH(MyEngine, MyTIU);
		T1 = NowTime - StartTime;
		D1 = NowDCH - StartDCH
		print(CalcTime, "  " , T1, " ", D1, " Error = ", 100*(D1-T1)/D1, "%");
		BumpCounter(LOOPCOUNTER);		-- bump loop counter	
		end;
	
	return true;	-- false=do not loop, true=continue to loop
	end	
--]==]
--[[---------------------------------------------------------------------------------------]]
function cleanup()	-- parameters not used
	-- this function is called once when the user presses the [STOP] button
	print(os.date("cleanup()  %d %b %Y   %X"));
	return true;		-- false=cleanup failed, true=cleanup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
function GetDCH()	-- parameters not used
	local MyDCH;
	-- Get the Chronometer
	MyDCH = DCH(MyEngine, MyTIU);
	print("DCH : " .. MyDCH .. " seconds");
	Myhours = MyDCH//(60 * 60);	-- use floor divide to get an integer
	Myminutes = (MyDCH//60) % 60;
	Myseconds = (MyDCH//1) % 60;	-- converts MyDCH to integer for modulo operation
	print(string.format("DCH : %02dh:%02dm:%02ds", Myhours, Myminutes, Myseconds)); 
	BumpCounter(TESTCOUNTER);		-- bump test counter
	return true;		-- false=function failed, true=function succeeded
end
Function03Name, Function03Label = GetDCH, "DCH";
--[[---------------------------------------------------------------------------------------]]
--[[
Dropped file D:\trains\MTH\PC Control\Scripts\DCH Accuracy Test.lua
Success compiling D:\trains\MTH\PC Control\Scripts\DCH Accuracy Test.lua
setup() 28 Feb 2022   20:01:28
50  50 50.00047893179 Error = 0.00095785440503136%
100  101 100.00095786358 Error = -0.99903256705092%
150  151 151.02315794851 Error = 0.015334038054977%
200  202 202.01342924741 Error = 0.0066477003337925%
250  252 252.0139081792 Error = 0.005518814140401%
300  303 303.00417947809 Error = 0.0013793466806389%
350  353 353.00465840988 Error = 0.001319645441167%
400  404 404.02685849481 Error = 0.0066477003319918%
450  454 454.0273374266 Error = 0.0060210970463645%
500  505 505.0176087255 Error = 0.0034867547582455%
550  555 555.01808765729 Error = 0.0032589311401627%
600  606 606.00835895618 Error = 0.0013793466806389%
650  656 656.00883788797 Error = 0.0013472208700869%
700  707 706.00931681976 Error = -0.14032154486274%
750  757 756.99958811866 Error = -5.4409717535572e-05%
800  808 808.02178820359 Error = 0.0026964871383582%
850  858 858.02226713538 Error = 0.002595169873431%
900  909 909.01253843428 Error = 0.0013793466806389%
950  959 959.01301736607 Error = 0.0013573711544835%
1000  1010 1010.003288665 Error = 0.00032560933227745%
1050  1060 1061.0254887499 Error = 0.096650717703692%
1100  1112 1111.0259676817 Error = -0.087669626691677%
1150  1162 1162.0162389806 Error = 0.0013974831015764%
1200  1212 1212.0167179124 Error = 0.0013793466806389%
1250  1263 1263.0069892113 Error = 0.00055337866898188%
1300  1313 1313.0074681431 Error = 0.00056878146067153%
1350  1364 1364.029668228 Error = 0.0021750427192874%
1400  1414 1414.0301471598 Error = 0.002132002619481%
1450  1465 1465.0204184587 Error = 0.0013937320198334%
1500  1515 1515.0208973905 Error = 0.0013793466806389%
1550  1566 1566.0111686894 Error = 0.00071319346765713%
1600  1616 1617.0014399882 Error = 0.061931916910143%
1650  1667 1667.00191892 Error = 0.00011511204741497%
1700  1718 1718.024119005 Error = 0.0014038804638533%
1750  1768 1769.0143903039 Error = 0.057342117137639%
1800  1819 1819.0148692357 Error = 0.0008174334312047%
1850  1869 1870.0051405346 Error = 0.053750682966778%
1900  1920 1920.0056194663 Error = 0.00029267968198555%
1950  1970 1971.0278195513 Error = 0.05214637465172%
2000  2021 2021.0282984831 Error = 0.0014002022180524%
2050  2071 2072.018569782 Error = 0.049158332691427%
2100  2122 2122.0190487138 Error = 0.00089766930994757%
2150  2172 2173.0093200126 Error = 0.046448029621757%
2200  2223 2223.0097989444 Error = 0.00044079627708523%
2250  2273 2274.0000702433 Error = 0.043978461408961%
2300  2324 2324.0005491751 Error = 2.3630593456614e-05%
2350  2374 2375.0227492601 Error = 0.043062714257033%
2400  2425 2425.0232281918 Error = 0.00095785440623149%
2450  2475 2476.0134994907 Error = 0.040932712642451%
2500  2526 2526.0139784225 Error = 0.00055337866869384%
2550  2576 2577.0042497214 Error = 0.03896965717197%
2600  2627 2628.0264498064 Error = 0.039057818707679%
2650  2677 2678.0269287381 Error = 0.038346467958375%
2700  2728 2729.017200037 Error = 0.037273493073729%
2750  2779 2779.0176789688 Error = 0.00063615891904718%
2800  2829 2830.0079502677 Error = 0.035616517177276%
2850  2880 2880.0084291995 Error = 0.00029267968211187%
2900  2930 2930.9987004984 Error = 0.034073727096399%
2950  2981 2980.9991794302 Error = -2.7526669877796e-05%
3000  3031 3032.0213795151 Error = 0.033686421937164%
3050  3082 3083.011650814 Error = 0.032813720109041%
3100  3132 3133.0121297458 Error = 0.032305324840837%
3150  3183 3184.0024010447 Error = 0.031482421130706%
3200  3233 3234.0028799765 Error = 0.031010484953741%
3250  3284 3285.0250800614 Error = 0.03120463425537%
3300  3334 3335.0255589932 Error = 0.030751158426802%
3350  3385 3385.026037925 Error = 0.00076920900219353%
3400  3435 3436.0163092239 Error = 0.029578125726027%
3450  3486 3486.0167881557 Error = 0.00048158562391532%
3500  3536 3537.0070594546 Error = 0.028472079293666%
3550  3587 3588.0292595395 Error = 0.028685929379914%
3600  3637 3638.0297384713 Error = 0.028304839304168%
3650  3688 3688.0302174031 Error = 0.00081933718871261%
3700  3738 3739.020488702 Error = 0.027292942231379%
3750  3789 3789.0209676338 Error = 0.00055337866878986%
3800  3839 3840.0112389327 Error = 0.026334270129985%
3850  3889 3891.0015102316 Error = 0.051439461699462%
3900  3940 3941.0019891634 Error = 0.025424731226379%
3950  3991 3992.0241892483 Error = 0.025655887834076%
4000  4041 4043.0144605472 Error = 0.049825707200699%
4050  4092 4093.014939479 Error = 0.024796867199155%
4100  4142 4143.0154184108 Error = 0.024509163211458%
4150  4193 4194.0056897097 Error = 0.02397921662673%
4200  4243 4244.0061686415 Error = 0.023707991965171%
4250  4294 4295.0283687264 Error = 0.023943234784721%
4300  4344 4345.0288476582 Error = 0.023678730205417%
4350  4395 4396.0191189571 Error = 0.023182768989362%
4400  4445 4446.0195978889 Error = 0.022932824888073%
4450  4496 4497.0098691878 Error = 0.022456459228254%
4500  4546 4548.0001404867 Error = 0.043978461408802%
4550  4597 4598.0006194185 Error = 0.021762054885934%
4600  4647 4649.0228195034 Error = 0.043510638297833%
4650  4698 4699.0232984352 Error = 0.021776832549619%
4700  4748 4749.023777367 Error = 0.021557638263346%
4750  4798 4800.0140486659 Error = 0.04195922439886%
4800  4849 4850.0145275976 Error = 0.02091803213936%
4850  4899 4901.0047988965 Error = 0.040905874995198%
4900  4950 4952.0269989815 Error = 0.040932712642451%
4950  5001 5002.0274779133 Error = 0.020541228887762%
5000  5051 5053.0177492122 Error = 0.03993156787294%
5050  5102 5103.0182281439 Error = 0.019953449084949%
5100  5152 5153.0187070757 Error = 0.01976913210774%
5150  5202 5204.0089783746 Error = 0.038604437136551%
5200  5253 5254.0094573064 Error = 0.019213085066363%
5250  5303 5304.9997286053 Error = 0.037695168852426%
5300  5354 5356.0219286903 Error = 0.037750567812642%
5350  5404 5406.022407622 Error = 0.037410270797042%
5400  5455 5457.0126789209 Error = 0.03688243072466%
5450  5506 5507.0131578527 Error = 0.018397592736531%
5500  5556 5558.0034291516 Error = 0.036045842309352%
5550  5607 5608.0039080834 Error = 0.017901344219167%
5600  5657 5659.0261081683 Error = 0.035803124594487%
5650  5708 5709.0265871001 Error = 0.017981823774506%
5700  5758 5760.016858399 Error = 0.035014800279444%
5750  5809 5810.0173373308 Error = 0.017510056713327%
5800  5859 5861.0076086297 Error = 0.034253643123688%
5850  5909 5911.0080875615 Error = 0.033971998206664%
5900  5960 5962.0302876464 Error = 0.034053628520484%
5950  6010 6011.9988377922 Error = 0.033247474693873%
6000  6061 6063.0210378771 Error = 0.033333842394685%
6050  6111 6113.0215168089 Error = 0.033069028194225%
6100  6162 6164.0117881078 Error = 0.032637642122706%
6150  6212 6214.0122670396 Error = 0.032382733620715%
6200  6263 6265.0025383385 Error = 0.031963887105205%
6250  6313 6315.0030172703 Error = 0.031718389758562%
6300  6364 6366.0252173552 Error = 0.031812901866823%
6350  6414 6416.025696287 Error = 0.031572446603147%
6400  6465 6467.0159675859 Error = 0.031173072650513%
6450  6515 6517.0164465177 Error = 0.03094125255374%
6500  6565 6568.0067178166 Error = 0.045778239057345%
6550  6616 6618.0071967484 Error = 0.03032932253928%
6600  6667 6669.0293968333 Error = 0.030430167758293%
6650  6717 6719.0298757651 Error = 0.030210845949916%
[STOP]
cleanup()  28 Feb 2022   21:53:56

]]