--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2019 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
title = "Double Vertex Graph Test"

--
-- Original luagraph library Downloaded from
-- https://github.com/chen0040/lua-graph
--
-- Original Dijkstra algorithm described in
-- https://en.wikipedia.org/wiki/Dijkstra%27s_algorithm


require("defines");
require("functions");
	-- an undirected weighted double vertex graph with requiredVertices nodes starting at 0
local requiredVertices
-- create the graph and related tables
local g = require('rtcgraphs.data.graph').create(0)

function setup(Engine, TIU)
	-- this function is called once when the [Run Script] button is pushed
	-- Engine : Engine number selected on the RTC Main window
	-- TIU : TIU number selected on the RTC Main window
	print("setup()");
--[[	-- Layout 0	
	-- an undirected weighted double vertex graph
	switchtrackweight = 1;
	requiredVertices = 30
	g:addDoubleVertex(27, 0) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set	
	g:addUndirectedEdge(0, 1, 3)
	g:addDoubleVertex(1, 2) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(2, 3, switchtrackweight)
	g:addDoubleVertex(3, 4) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(4, 5, 4)
	g:addDoubleVertex(5, 6) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(6, 7, switchtrackweight)
	g:addDoubleVertex(7, 8) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(8, 14, 5)	----------  was 8 to 14
	g:addUndirectedEdge(6, 9, switchtrackweight)
	g:addDoubleVertex(9, 10) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(10, 11, 10)
	g:addDoubleVertex(11, 12) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(13, 12, switchtrackweight)
	g:addDoubleVertex(13, 14) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set	
	g:addUndirectedEdge(12, 15, switchtrackweight)
	g:addDoubleVertex(15, 16) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(16, 17, 2)
	g:addDoubleVertex(17, 18) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(18, 21, switchtrackweight)
	g:addDoubleVertex(21, 22) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(22, 25, 11)
	g:addUndirectedEdge(3, 23, switchtrackweight)
	g:addDoubleVertex(23, 24) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(24, 19, 6)
	g:addDoubleVertex(19, 20) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(20, 21, switchtrackweight)
	g:addDoubleVertex(25, 26) -- double vertex	-- double vertex entered as an edge between the two vertices with a flag set
	g:addUndirectedEdge(28, 29, 7)		-- unreachable vertices
--]]

--[[	-- Layouts 1 & 2
	-- an undirected weighted double vertex graph
	local switchtrackweight = 1;
	local requiredVertices = 74
	
	g:addDoubleVertex(0, 1) -- double vertex
	g:addUndirectedEdge(1, 2, 5)
	g:addDoubleVertex(2, 3) -- double vertex	
	g:addUndirectedEdge(3, 4, switchtrackweight)
	g:addDoubleVertex(4, 5) -- double vertex	
	g:addUndirectedEdge(5, 6, 18)
	g:addDoubleVertex(6, 7) -- double vertex	
	g:addUndirectedEdge(7, 8, switchtrackweight)
	g:addDoubleVertex(8, 9) -- double vertex
	g:addUndirectedEdge(9, 10, 5)
	g:addDoubleVertex(10, 11) -- double vertex	
	g:addUndirectedEdge(11, 12, switchtrackweight)
	g:addDoubleVertex(12, 13) -- double vertex
	g:addUndirectedEdge(13, 14, 5)	
	g:addDoubleVertex(14, 15) -- double vertex
	
	g:addUndirectedEdge(3, 25, switchtrackweight)
	g:addDoubleVertex(25, 24) -- double vertex
	g:addUndirectedEdge(24, 23, 8)
	g:addDoubleVertex(23, 22) -- double vertex
	g:addUndirectedEdge(22, 21, 9)	
	g:addDoubleVertex(21, 20) -- double vertex
	g:addUndirectedEdge(20, 8, switchtrackweight)

	g:addUndirectedEdge(11, 19, switchtrackweight)
	g:addDoubleVertex(19, 18) -- double vertex
	g:addUndirectedEdge(18, 17, 4)	
	g:addDoubleVertex(17, 16) -- double vertex

	g:addUndirectedEdge(15, 26, 7)	
	g:addDoubleVertex(26, 57) -- double vertex
	g:addUndirectedEdge(57, 27, switchtrackweight)	
	g:addDoubleVertex(27, 28) -- double vertex
	g:addUndirectedEdge(28, 29, switchtrackweight)
	g:addDoubleVertex(29, 30) -- double vertex
	g:addUndirectedEdge(30, 31, 22)	
	g:addDoubleVertex(31, 32) -- double vertex
	g:addUndirectedEdge(32, 33, switchtrackweight)	
	g:addDoubleVertex(33, 34) -- double vertex
	g:addUndirectedEdge(34, 35, 8)	
	g:addDoubleVertex(35, 36) -- double vertex
	g:addUndirectedEdge(36, 37, switchtrackweight)	
	g:addDoubleVertex(37, 38) -- double vertex
	g:addUndirectedEdge(38, 39, 3)	
	g:addDoubleVertex(39, 40) -- double vertex

	g:addUndirectedEdge(16, 41, 6)	
	g:addDoubleVertex(41, 42) -- double vertex
	g:addUndirectedEdge(42, 43, switchtrackweight)
	g:addDoubleVertex(43, 44) -- double vertex
	g:addUndirectedEdge(44, 58, 14)	
	g:addDoubleVertex(58, 45) -- double vertex
	g:addUndirectedEdge(45, 46, switchtrackweight)
	g:addDoubleVertex(46, 47) -- double vertex
	g:addUndirectedEdge(47, 48, 19)	
	g:addDoubleVertex(48, 49) -- double vertex

	g:addUndirectedEdge(43, 51, switchtrackweight)
	g:addDoubleVertex(51, 50) -- double vertex
	g:addUndirectedEdge(50, 57, switchtrackweight)
	g:addUndirectedEdge(45, 52, switchtrackweight)
	g:addDoubleVertex(52, 53) -- double vertex
	g:addUndirectedEdge(53, 33, switchtrackweight)

	g:addUndirectedEdge(29, 54, switchtrackweight)
	g:addDoubleVertex(54, 55) -- double vertex
	g:addUndirectedEdge(55, 56, 6)	
	g:addDoubleVertex(56, 59) -- double vertex
--[=[	-- Layout 1	A reversing loop
	g:addUndirectedEdge(40, 60, switchtrackweight)
	g:addDoubleVertex(60, 61) -- double vertex	
	g:addUndirectedEdge(61, 62, 4)
	g:addDoubleVertex(62, 63) -- double vertex
	g:addUndirectedEdge(63, 64, 5)
	g:addDoubleVertex(64, 65) -- double vertex
	g:addUndirectedEdge(65, 66, 6)
	g:addDoubleVertex(66, 67) -- double vertex
	g:addUndirectedEdge(67, 40, switchtrackweight)
--]=]	
---[=[	-- Layout 2 A reversing loop with a siding
	g:addUndirectedEdge(40, 60, switchtrackweight)
	g:addDoubleVertex(60, 61) -- double vertex	
	g:addUndirectedEdge(61, 62, 4)
	g:addDoubleVertex(62, 63) -- double vertex
	g:addUndirectedEdge(63, 64, 5)
	g:addDoubleVertex(65, 68) -- double vertex
	g:addUndirectedEdge(68, 69, switchtrackweight)
	g:addUndirectedEdge(68, 70, switchtrackweight)
	g:addDoubleVertex(70, 71) -- double vertex
	g:addUndirectedEdge(71, 72, 16)
	g:addDoubleVertex(72, 73) -- double vertex
	g:addDoubleVertex(69, 64) -- double vertex
	g:addUndirectedEdge(65, 66, 16)
	g:addDoubleVertex(66, 67) -- double vertex
	g:addUndirectedEdge(67, 40, switchtrackweight)
	--
	-- next edge makes the layout a loop
	g:addUndirectedEdge(0, 73, 7)
	--
--]=]
--]]

--[[
	-- Layout 3
	-- an undirected weighted double vertex graph
	local switchtrackweight = 1;
	local requiredVertices = 26
	
	g:addDoubleVertex(0, 1) -- double vertex
	g:addUndirectedEdge(1, 2, 5)
	g:addDoubleVertex(2, 3) -- double vertex	
	g:addUndirectedEdge(3, 4, switchtrackweight)
	g:addDoubleVertex(4, 5) -- double vertex	
	g:addUndirectedEdge(5, 6, 2)
	g:addDoubleVertex(6, 7) -- double vertex
	g:addUndirectedEdge(7, 8, 2)
	g:addDoubleVertex(8, 9) -- double vertex
	g:addUndirectedEdge(9, 10, 2)
	g:addDoubleVertex(10, 11) -- double vertex
	g:addUndirectedEdge(11, 12, 2)
	g:addDoubleVertex(12, 13) -- double vertex

	g:addUndirectedEdge(3, 14, switchtrackweight)
	g:addDoubleVertex(14, 15) -- double vertex
	g:addUndirectedEdge(15, 16, 6)
	g:addDoubleVertex(16, 17) -- double vertex
	g:addUndirectedEdge(17, 18, 6)
	g:addDoubleVertex(18, 19) -- double vertex	
	
	g:addUndirectedEdge(3, 20, switchtrackweight)
	g:addDoubleVertex(20, 21) -- double vertex
	g:addUndirectedEdge(21, 22, 3)
	g:addDoubleVertex(22, 23) -- double vertex
	g:addUndirectedEdge(23, 24, 5)
	g:addDoubleVertex(24, 25) -- double vertex	
--]]

---[[	-- Layouts 4
	-- an undirected weighted double vertex graph
	local requiredVertices = 56
	local TIU = 2;
	local AIU = 1;
	local Channel = 1;
-- Youngstown Yard	
	-- Track 1
	g:addDoubleVertex(0, 1) -- double vertex
	g:addUndirectedEdge(1, 2, 5)
	g:addDoubleVertex(2, 3) -- double vertex
	-- Track 2
	g:addDoubleVertex(36, 37) -- double vertex
	g:addUndirectedEdge(37, 38, 5)
	g:addDoubleVertex(38, 39) -- double vertex
	-- switch track
	g:addSwitchTrack(4, 3, 39, TIU, 1, 3);	-- common vertex, normal vertex, reverse vertex, TIU, AIU, Channel
	
-- Main Line	
	g:addDoubleVertex(4, 5) -- double vertex
	g:addUndirectedEdge(5, 6, 5)
	g:addDoubleVertex(6, 7) -- double vertex
	g:addSwitchTrack(7, 8, 40, TIU, 2, 5);	-- common vertex, normal vertex, reverse vertex, TIU, AIU, Channel
	g:addDoubleVertex(8, 9) -- double vertex
	g:addDirectedEdge(9, 10, 15)	-- Struthers Through Track Eastward
	g:addDirectedEdge(10, 9, 5)		-- Struthers Through Track Westward

	g:addDoubleVertex(10, 11) -- double vertex
	g:addSwitchTrack(12, 11, 43, TIU, 2, 6);	-- common vertex, normal vertex, reverse vertex, TIU, AIU, Channel	
	
	g:addDoubleVertex(12, 13) -- double vertex
	g:addUndirectedEdge(13, 14, 5)
	g:addDoubleVertex(14, 15) -- double vertex
	g:addSwitchTrack(15, 16, 44, TIU, 1, 1);	-- common vertex, normal vertex, reverse vertex, TIU, AIU, Channel	
	
	g:addDoubleVertex(16, 17) -- double vertex
	g:addDirectedEdge(17, 18, 15)	-- College Through Track Eastward
	g:addDirectedEdge(18, 17, 5)	-- College Through Track Westward
	
	g:addDoubleVertex(18, 19) -- double vertex	
	g:addSwitchTrack(20, 19, 47, TIU, 1, 2);	-- common vertex, normal vertex, reverse vertex, TIU, AIU, Channel	

	g:addDoubleVertex(20, 21) -- double vertex
	g:addUndirectedEdge(21, 22, 5)
	g:addDoubleVertex(22, 23) -- double vertex	
	g:addSwitchTrack(23, 24, 48, TIU, 1, 9);	-- common vertex, normal vertex, reverse vertex, TIU, AIU, Channel	
	
	g:addDoubleVertex(24, 25) -- double vertex
	g:addDirectedEdge(25, 26, 15)	-- Aliquippa Through Track Eastward
	g:addDirectedEdge(26, 25, 5)	-- Aliquippa Through Track Westward	
	
	g:addDoubleVertex(26, 27) -- double vertex	
	g:addSwitchTrack(28, 27, 51, TIU, 1, 5);	-- common vertex, normal vertex, reverse vertex, TIU, AIU, Channel	
	
	g:addDoubleVertex(28, 29) -- double vertex	
	g:addUndirectedEdge(29, 30, 5)
	g:addDoubleVertex(30, 31) -- double vertex

-- McKees Rocks Yard
	-- Track 2
	g:addDoubleVertex(52, 53) -- double vertex	
	g:addUndirectedEdge(53, 54, 5)
	g:addDoubleVertex(54, 55) -- double vertex
	-- Track 1
	g:addDoubleVertex(32, 33) -- double vertex
	g:addUndirectedEdge(33, 34, 5)
	g:addDoubleVertex(34, 35) -- double vertex	
	g:addSwitchTrack(31, 32, 52, TIU, 1, 4);	-- common vertex, normal vertex, reverse vertex, TIU, AIU, Channel	
--
--	
-- Struthers Passing Track
	g:addDoubleVertex(40, 41) -- double vertex
	g:addDirectedEdge(41, 42, 7)	-- Eastward
	g:addDirectedEdge(42, 41, 20)	-- Westward
	g:addDoubleVertex(42, 43) -- double vertex
	
-- College Passing Track
	g:addDoubleVertex(44, 45) -- double vertex
	g:addDirectedEdge(45, 46, 7)	-- Eastward
	g:addDirectedEdge(46, 45, 20)	-- Westward
	g:addDoubleVertex(46, 47) -- double vertex
	
-- Aliquippa Passing Track
	g:addDoubleVertex(48, 49) -- double vertex
	g:addDirectedEdge(49, 50, 7)	-- Eastward
	g:addDirectedEdge(50, 49, 20)	-- Westward
	g:addDoubleVertex(50, 51) -- double vertex
--]]
--	
-- templates:	
--	g:addUndirectedEdge(000, 000, switchtrackweight)
--	g:addUndirectedEdge(000, 000, 000)	
--	g:addDoubleVertex(000, 000) -- double vertex
--	g:addSwitchTrack(000, 000, 000);	-- common vertex, normal vertex, reverse vertex	
--
	actualVertices = g:vertexCount() 
	print("Vertices added : ", actualVertices) 
	print("Edges added : ", g:edgeCount())
	if (actualVertices ~= requiredVertices) then
		return Stop("Graph Double Vertex Test : Missing Vertices");		-- end the script
		end;
	
if (Debug() > 4) then	
	dumpadjacencylist()
	end

print("Shortest Paths (Based on Dijkstra) - Weighted");
-- https://github.com/chen0040/lua-graph#shortest-paths-dijkstra
-- https://en.wikipedia.org/wiki/Dijkstra%27s_algorithm
local source = 33	-- beginging vertex
-- this call to create() sets up the data structures used
local double_vertex = require('rtcgraphs.shortest_paths.Double_Vertex').create()

----------
-- re-weight an edge
-- internally its a directed graph so we can re-weight each direction separately
-- valid for Layout 3 only
--[[
local ef = g:findEdge(23,24)
if (ef) then
	ef:changeWeight(9)
	end
ef = g:findEdge(24,23)
if (ef) then
	ef:changeWeight(15)
	end	
--]]

if (Debug() > 4) then	
	dumpadjacencylist()
	end

----------
-- this call to run() calculates the lowest cost to all vertices
double_vertex:run(g, source) -- "source" is the id of the source vertex in the path search

--[=[
-- this chunk of code displays the shortest paths from the source vertex to all other vertices
for k = 0,g:vertexCount() - 1 do
    local v = g:vertexAt(k)
    if v ~= source and double_vertex:hasPathTo(v) then
        print('path from ' .. source .. ' -> ' .. v .. ' ( cost: '  .. double_vertex:getPathLength(v) .. ' )')
        local path = double_vertex:getPathTo(v)
        for i = 0,path:size()-1 do
            print('     # from ' .. path:get(i):from() .. ' -> ' .. path:get(i):to() .. ' ( cost: ' .. path:get(i).weight .. ' )')
        end
    else
		print("no path from " .. source .. " -> " .. v .. " (hasPathTo(v) == false)")
    end
end
--]=]
---[[
-- this chunk of code displays the shortest path from the souce vertex to a single destination vertex (k)
	k = 1	--destination vertex
    local v = k
    if v ~= source and double_vertex:hasPathTo(v) then
		if (double_vertex:getPathLength(v) ~= math.huge) then -- mcd
			print('path from ' .. source .. ' -> ' .. v .. ' ( cost: '  .. double_vertex:getPathLength(v) .. ' )')
			local path = double_vertex:getPathTo(v)
			--print("path : " .. dumpx(path));
			for i = 0,path:size()-1 do
				local position, TIU, AIU, Channel = path:get(i):control();
				if (position == nil) then
					print('     # from ' .. path:get(i):from() .. ' -> ' .. path:get(i):to() .. ' ( cost: ' .. path:get(i).weight .. ' )')
					else
					print('     # from ' .. path:get(i):from() .. ' -> ' .. path:get(i):to() .. ' ( cost: ' .. path:get(i).weight .. ' )' ..
						' switch:' .. (position and "normal" or "reverse")  .. ' TIU = ' .. TIU .. ' AIU = ' .. AIU .. ' Channel = ' .. Channel)
					end
			end
		else
		print("no path from " .. source .. " -> " .. v .. " (getPathLength(v) == math.huge)")	-- mcd		
		end
    else
		print("no path from " .. source .. " -> " .. v .. " (hasPathTo(v) == false)")	-- mcd
    end
--]]
	print(string.format("RunTime = %.2f seconds", RunTime()));
	return Stop("End of AIU Status Test");		-- end the script
	end
--[[---------------------------------------------------------------------------------------]]
function dumpadjacencylist()	
-- code below prints the adjacency list 
	for k = 0, g:vertexCount() - 1 do -- iterate through all vertices in g
		local v = g:vertexAt(k)
		local adj_v = g:adj(v) -- adjacency list for vertex v
		local text = ' Vertex ' .. v ;
		for i = 0, adj_v:size()-1 do
			local e = adj_v:get(i)
			text = text .. ', ' .. e:other(v) .. '(' .. e.weight .. ')' 
		end
		print(text);
		--print("adj(" .. v .. ") : " .. dumpx(adj_v));
	end
end

--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
	-- this function is called once when the user presses the [STOP] button
	print(string.format("cleanup() Complete at %.2f", RunTime()));
	return true;		-- false=cleanup failed, true=cleanup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
function dumpx(o)
   if type(o) == 'table' then
      local s = '{ '
      for k,v in pairs(o) do
         if type(k) ~= 'number' then k = '"'..k..'"' end
		 -- optional : print(k,v)
         s = s .. '['..k..'] = ' .. dumpx(v) .. ','
      end
      return s .. '} \n'
   else
      return tostring(o)
   end
end
--[[---------------------------------------------------------------------------------------]]