--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2021 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
title = "Layout4 EB Block Occupancy Test"
require([[defines]]);
require([[functions]]);
--[[
When using this module, all variables (including functions!) must be declared through 
a regular assignment (even assigning nil will do) in a strict scope before being used 
anywhere or assigned to inside a nested scope.
https://github.com/lua-stdlib/strict
]]
local _ENV = require 'std.strict' (_G)	-- strict.lua will detect undeclared global variables

local ErrCount = 0;
local Layout4;
-- OCC is any value greater than zero
local OCC = 1;  -- Occupied
local PauseOn = false;
local StepTime = 3;

--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
	print("setup() Running");	
	-- Instrumented Layout 4
	Layout4 = require([[Layout4]]);		-- All of the details about the Layout
	print([[Run "Layout4 APB Master.lua" in PC windows #1 (don't set any block occupied)]]);
	print("Run this Script in a PC window other than #1");
	ClearButton(Engine, TIU);			-- clear all program flags
	-- start in Aliquippa Block
	print("Start in the Aliquippa Block, heading toward McKees Rocks - setup occupancy flags for start");
	SetOccupancy(ALIQUIPPA_BLOCK, 1);	-- set occupancy flag
	SetEvent(1, BLOCK_OCCUPANCY_EVENT, ALIQUIPPA_BLOCK, OCC, 0);	-- set next block occupancy
	print([[SetEvent() calls simulate a tag detection (TAG_EVENT) to the Layout4 APB Master.lua" script]]);
	
	return true;		-- false=setup failed, true=setup succeeded
end;
--[[---------------------------------------------------------------------------------------]]
function loop(Engine, TIU)
	-- Always call Sleep(X) at least once in this function if it returns true;
	print("loop() Running Eastbound at ", RunTime(), " seconds");
	local res;
	-- using events, cause "Layout4 APB Master.lua" to run a simulated train around the layout
	--
	-- This script always starts in ALIQUIPPA_BLOCK going eastbound
	--
	-- this runs a train by generating a TAG_EVENT just as if a tag passed over a tag reader
	--  SetEvent(1, TAG_EVENT, detector, reader, 1 for EngineNo or 0 for Caboose)
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, ALIQUIPPA_WEST_DETECTOR, ALIQUIPPA_WEST_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(J_L_PASSING_TRACK, 1);
	Switch(REVERSE, Switches[ALIQUIPPA_WEST][CHAN_NUMBER], Switches[ALIQUIPPA_WEST][AIU_NUMBER], Switches[ALIQUIPPA_WEST][TIU_NUMBER], 0);
	if (type(res) ==  "nil") then
		print("SetEvent() returned nil : invalid destination Program Control number - Layout4 APB Master.lua not running");
		return Stop("DONE");
		end;		
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, ALIQUIPPA_WEST_DETECTOR, ALIQUIPPA_WEST_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(ALIQUIPPA_BLOCK, 1);
	Switch(NORMAL,  Switches[ALIQUIPPA_WEST][CHAN_NUMBER], Switches[ALIQUIPPA_WEST][AIU_NUMBER], Switches[ALIQUIPPA_WEST][TIU_NUMBER], 0)
	-- repeat for each block around the layout
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, ALIQUIPPA_EAST_DETECTOR, ALIQUIPPA_EAST_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(MCKEES_ROCKS_BLOCK, 1);
	Switch(REVERSE, Switches[ALIQUIPPA_EAST][CHAN_NUMBER], Switches[ALIQUIPPA_EAST][AIU_NUMBER], Switches[ALIQUIPPA_EAST][TIU_NUMBER], 0)
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, ALIQUIPPA_EAST_DETECTOR, ALIQUIPPA_EAST_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(J_L_PASSING_TRACK, 1);
	Switch(NORMAL,  Switches[ALIQUIPPA_EAST][CHAN_NUMBER], Switches[ALIQUIPPA_EAST][AIU_NUMBER], Switches[ALIQUIPPA_EAST][TIU_NUMBER], 0);
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, MCKEES_ROCKS_YARD_DETECTOR, MCKEES_ROCKS_YARD_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(LIONEL_IVES_BLOCK, 1);
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, MCKEES_ROCKS_YARD_DETECTOR, MCKEES_ROCKS_YARD_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(MCKEES_ROCKS_BLOCK, 1);
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, YOUNGSTOWN_YARD_DETECTOR, YOUNGSTOWN_YARD_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(YOUNGSTOWN_BLOCK, 1);
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, YOUNGSTOWN_YARD_DETECTOR, YOUNGSTOWN_YARD_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(LIONEL_IVES_BLOCK, 1);
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, STRUTHERS_WEST_DETECTOR, STRUTHERS_WEST_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(STRUTHERS_STATION_TRACK, 1);
	Switch(REVERSE, Switches[STRUTHERS_WEST][CHAN_NUMBER], Switches[STRUTHERS_WEST][AIU_NUMBER], Switches[STRUTHERS_WEST][TIU_NUMBER], 0)
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, STRUTHERS_WEST_DETECTOR, STRUTHERS_WEST_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(YOUNGSTOWN_BLOCK, 1);
	Switch(NORMAL,  Switches[STRUTHERS_WEST][CHAN_NUMBER], Switches[STRUTHERS_WEST][AIU_NUMBER], Switches[STRUTHERS_WEST][TIU_NUMBER], 0)
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, STRUTHERS_EAST_DETECTOR, STRUTHERS_EAST_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(LOWELLVILLE_BLOCK, 1);
	Switch(REVERSE, Switches[STRUTHERS_EAST][CHAN_NUMBER], Switches[STRUTHERS_EAST][AIU_NUMBER], Switches[STRUTHERS_EAST][TIU_NUMBER], 0)
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, STRUTHERS_EAST_DETECTOR, STRUTHERS_EAST_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(STRUTHERS_STATION_TRACK, 1);
	Switch(NORMAL,  Switches[STRUTHERS_EAST][CHAN_NUMBER], Switches[STRUTHERS_EAST][AIU_NUMBER], Switches[STRUTHERS_EAST][TIU_NUMBER], 0)
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, LOWELLVILLE_DETECTOR, LOWELLVILLE_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(NEW_CASTLE_BLOCK, 1);
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, LOWELLVILLE_DETECTOR, LOWELLVILLE_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(LOWELLVILLE_BLOCK, 1);
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, COLLEGE_WEST_DETECTOR, COLLEGE_WEST_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(COLLEGE_STATION_TRACK, 1);
	Switch(REVERSE, Switches[COLLEGE_WEST][CHAN_NUMBER], Switches[COLLEGE_WEST][AIU_NUMBER], Switches[COLLEGE_WEST][TIU_NUMBER], 0)
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, COLLEGE_WEST_DETECTOR, COLLEGE_WEST_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(NEW_CASTLE_BLOCK, 1);
	Switch(NORMAL,  Switches[COLLEGE_WEST][CHAN_NUMBER], Switches[COLLEGE_WEST][AIU_NUMBER], Switches[COLLEGE_WEST][TIU_NUMBER], 0)
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, COLLEGE_EAST_DETECTOR, COLLEGE_EAST_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(BEAVER_FALLS_BLOCK, 1);
	Switch(REVERSE, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0)
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, COLLEGE_EAST_DETECTOR, COLLEGE_EAST_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(COLLEGE_STATION_TRACK, 1);
	Switch(NORMAL,  Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0)
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, FALLSTON_DETECTOR, FALLSTON_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(FALLSTON_BLOCK, 1);
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, FALLSTON_DETECTOR, FALLSTON_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(BEAVER_FALLS_BLOCK, 1);		
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, MONACA_WEST_DETECTOR, MONACA_WEST_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(MONACA_BLOCK, 1);
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, MONACA_WEST_DETECTOR, MONACA_WEST_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(FALLSTON_BLOCK, 1);
	--	
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, MONACA_EAST_DETECTOR, MONACA_EAST_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(WEST_ALIQUIPPA_BLOCK, 1);
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, MONACA_EAST_DETECTOR, MONACA_EAST_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(MONACA_BLOCK, 1);	
	--
	Sleep(StepTime);
	while (PauseOn) do Sleep(1); end;
	res = SetEvent(1, TAG_EVENT, WEST_ALIQUIPPA_DETECTOR, WEST_ALIQUIPPA_READER, 1);	-- Engine detected - set next block occupancy
		SetOccupancy(ALIQUIPPA_BLOCK, 1);
	Sleep(StepTime);
	res = SetEvent(1, TAG_EVENT, WEST_ALIQUIPPA_DETECTOR, WEST_ALIQUIPPA_READER, 0);	-- Caboose detected - clear current block occupancy
		ClearOccupancy(WEST_ALIQUIPPA_BLOCK, 1);	
	--	
	return true;		-- all of the code runs in the loop()
end;
--[[---------------------------------------------------------------------------------------]]

function cleanup(Engine, TIU)
	-- this function is called once when the user presses the [STOP] button or the Stop() function is called
	print("cleanup() Running");
	if (ErrCount == 0) then
		print("No errors");
		else
		print(ErrCount .. " Error(s)");
		end;	
	return true;		-- false=cleanup failed, true=cleanup succeeded
	end
--[[---------------------------------------------------------------------------------------]]

function PZ()
	if (PauseOn) then
		print("...Resumed");
		PauseOn = false;
		else
		print("Paused...");
		PauseOn = true;
		end;
	return true;
end
Function01Name, Function01Label = PZ, "Pause/Resume";
--[[---------------------------------------------------------------------------------------]]
function ClearButton(Eng, TIU)
print("Clear Occupancy Flags");
-- clear occupancy flags
local x;
for x = FIRST_BLOCK, LAST_BLOCK do
	OverrideOccupancy(x, 0);
	end;
print("Occupancy Flags cleared");
return true;
end
Function02Name, Function02Label = ClearButton, "Clear Flags";
--[[---------------------------------------------------------------------------------------]]