--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2020 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
title = "Thinking Engine Eastbound"
require([[defines]]);
require([[functions]]);
--
--[[
When using this module, all variables (including functions!) must be declared through 
a regular assignment (even assigning nil will do) in a strict scope before being used 
anywhere or assigned to inside a nested scope.
https://github.com/lua-stdlib/strict
]]
local _ENV = require 'std.strict' (_G)	-- strict.lua will detect undeclared global variables
--
local TagCount = 0;
-- Counter Labels
Counter01Label = "tags()";
Counter03Label = "Block Speed Limit";
local C3 = COUNTER03;
Counter04Label = "Target Distance";
local C4 = COUNTER04;
Counter05Label = "Stopping Distance";
local C5 = COUNTER05;
Counter06Label = "Stop Delay";
local C6 = COUNTER06;

local Tags = {};		-- table to hold all of the tags detected in sequence
local SoundOn = true;

local MyDirection = 0;
local StartBlockNo, EndBlockNo;
local CurrentBlockNo, LastBlockNo, NextBlockNo;
local StartedFlag = false;
local RFIDFlag = false;
local ResetinProgress = false;
local LLLoop = false;
local Layout4;
local MyTIUNo, MyEngineNo, Enum;
local SoundOff = false;
local EngineSoundSave, S1;
local BeepOn = false;
local MyEngineSpeed;	-- Smph
MASTER_TIU = nil;	-- TIU number used by Layout4 (global)
-- clear all tag handling functions
local Func30 = nil;
local Func51 = nil;
local Func10 = nil;
local Func41 = nil;
local Func21 = nil;
local Func20 = nil;
local Func40 = nil;
local Func50 = nil;
--[[---------------------------------------------------------------------------------------]]
function CalculateStopDelay(StoppingBlockNo, EngineNo, direction, EngSpeed)
--
-- Calculate the stop delay based on parameters
--
local Target_Station;
local Target_Distance;
local StopDelay;
local StoppingDistance
Target_Station = BlockName[StoppingBlockNo];
--print("Block = " .. BlockName[StoppingBlockNo] .. ", Direction = " .. direction .. ", Type = " .. FREIGHT);
--Target_Detector = Layout4.STOPBlock(StoppingBlockNo, direction, DETECTOR);
--Target_Reader = Layout4.STOPBlock(StoppingBlockNo, direction, READER);
if (EngineNo == 7 or EngineNo == 13) then
	Target_Distance = Layout4.STOPBlock(StoppingBlockNo, direction, PASSENGER);	--Passenger StoppingDistance;
	else
	Target_Distance = Layout4.STOPBlock(StoppingBlockNo, direction, FREIGHT);	--Freight StoppingDistance;
	end
if (Target_Distance == nil) then
	Rate(8, 8, 0, EngineNo, MyTIUNo);
	SetSpeed(0, 0, EngineNo, MyTIUNo);	-- immediate Engine stop
	printc(clRed, "Engine cannot stop in Target_Distance");
	return 0;
	end
SetCounter(C4, Target_Distance);
--print(BlockName[StoppingBlockNo] .. " " .. direction .. ", Target Distance = " .. Target_Distance);
-- equation for Stop Delay calculation
-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
StoppingDistance = Layout4.Stopping(EngineNo, EngSpeed);
StopDelay =  (Target_Distance - StoppingDistance) * (48 / (12 * 5280 * EngSpeed)) * 3600;
SetCounter(C5, StoppingDistance);
SetCounter(C6, StopDelay);
if (StopDelay < 0) then
	Rate(8, 8, 0, EngineNo, MyTIUNo);
	SetSpeed(0, 0, EngineNo, MyTIUNo);	-- immediate Engine stop
	printc(clRed, "Engine cannot stop in time (Stopping Distance > Target_Distance)");
	return 0;
	end;
print(string.format("%s : Target Distance = %d in - Stopping Distance = %.3f in - Speed %d Smph = Stop Delay %.3f seconds",
	BlockName[StoppingBlockNo], Target_Distance, StoppingDistance, EngSpeed, StopDelay));
print("Set speed to 0 after " .. StopDelay .. " seconds")
return StopDelay;
end;
--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
print("setup() Running");

MASTER_TIU = TIU;
-- Instrumented Layout 4
Layout4 = require([[Layout4]]);		-- All of the details about the Layout
-- an undirected weighted double vertex graph
MyTIUNo = MASTER_TIU;
MyEngineNo = 12;
Title(title, MyEngineNo);
Enum = tonumber(InputBox(title, "Enter Engine Number", MyEngineNo));
if (Enum ~= nil) then	-- if the result is a number, then use it, otherwise, ignore it
	MyEngineNo = Enum;
	end
print("Engine Number = " .. MyEngineNo);
-- Show new title with Engine # and put Engine # into Engine Number Spinner
Title(title .. "-Engine # " .. MyEngineNo, MyEngineNo);	
if (Layout4.StoppingisEmpty(MyEngineNo,1) == nil) then
	return Stop("No StoppingDistance Table for Engine " .. MyEngineNo);
	end	

--print("StationName = " .. Pretty(StationName));
--print("StationName = " .. Pretty(BlockName));
--print("Switches = " ..  Pretty(Switches));
--print("NextBlock = " ..  Pretty(NextBlock));
SetFlagName(98, "Waiting for RFID Tags");
SetFlagName(99, "Setup Complete");

-- start of run
---------
-- Eastbound train
MyDirection = EB;
StartBlockNo = YOUNGSTOWN_YARD1_BLOCK;	-- can start in Block YOUNGSTOWN_YARD2_BLOCK or YOUNGSTOWN_YARD1_BLOCK
CurrentBlockNo = StartBlockNo;
EndBlockNo = MCKEES_ROCKS_YARD2_BLOCK;	-- can end in Block MCKEES_ROCKS_YARD2_BLOCK or MCKEES_ROCKS_YARD1_BLOCK
MyEngineSpeed = BlockSpeed[MyDirection][StartBlockNo];
SetCounter(C3, MyEngineSpeed);
--[[---------------------]]
if (MyDirection == EB) then
	print("Eastbound");
	else
	print("Westbound");
	end

return true;		-- false=setup failed, true=setup succeeded
end
--[[---------------------------------------------------------------------------------------]]
function createMainLine2Passing(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance;	
	local EngineStopped = false;
	local StopDelay = 0;
	local function MainLine2Passing(DetectorID, EngineNo, TagLocation, CarModel, Normal_Track, Reverse_Track, OperatingSwitch)
	-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
	if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK) then		-- if MyEngineNo in tag
		print("MainLine2Passing : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
		print("MainLine2Passing : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
		SetFlagName(99,"MainLine2Passing"); 
		LastBlockNo = CurrentBlockNo;
		CurrentBlockNo = NextBlockNo;
		-- check for occupancy clear Reverse_Track or Normal_Track - one or the other must be unoccupied otherwise fail
		-- choose based on superiority or choose empty block
		-- this code checks one track then the other track, then fails with deadlock
		if (MyDirection == SuperiorDirection) then
			if (TrySetOccupancy(Normal_Track, MyEngineNo)) then
				NextBlockNo = Normal_Track;		-- Westbound trains are superior, take the through track if possible
				EngineStopped = false;
				else
				-- check inferior direction track
				if (TrySetOccupancy(Reverse_Track, MyEngineNo)) then
					NextBlockNo = Reverse_Track;
					EngineStopped = false;
					else
					NextBlockNo = Normal_Track;
					-- if we can't get a lock, stop in CurrentBlockNo
					StopDelay = CalculateStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, MyEngineSpeed)
					Sleep(StopDelay);	-- Stop delay
					SetSpeed(0, 0, MyEngineNo, MyTIUNo);	-- stop engine
					EngineStopped = true;
					end
				end;
			else	-- MyDirection ~= SuperiorDirection
			if (TrySetOccupancy(Reverse_Track, MyEngineNo))  then
				NextBlockNo = Reverse_Track;	-- Eastbound trains are inferior, take the passing track if possible
				EngineStopped = false;
				else
				-- check superior direction track
				if (TrySetOccupancy(Normal_Track, MyEngineNo)) then
					NextBlockNo = Normal_Track;
					EngineStopped = false;
					else
					NextBlockNo = Reverse_Track;
					-- if we can't get a lock, stop in CurrentBlockNo
					StopDelay = CalculateStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, MyEngineSpeed)
					Sleep(StopDelay);	-- Stop delay
					SetSpeed(0, 0, MyEngineNo, MyTIUNo);	-- stop engine
					EngineStopped = true;
					end;
				end;
			end
		print("MainLine2Passing : " .. DetectorID .. "Next Block Selection = " .. BlockName[NextBlockNo]);
		-- set OperatingSwitch switch
		if (NextBlockNo == Normal_Track) then
			Switch(NORMAL, Switches[OperatingSwitch][CHAN_NUMBER], Switches[OperatingSwitch][AIU_NUMBER], Switches[OperatingSwitch][TIU_NUMBER], 0);
			print("MainLine2Passing : Switch " .. SwitchName[OperatingSwitch] .. " to Normal");
			end;
		if (NextBlockNo == Reverse_Track) then
			Switch(REVERSE, Switches[OperatingSwitch][CHAN_NUMBER], Switches[OperatingSwitch][AIU_NUMBER], Switches[OperatingSwitch][TIU_NUMBER], 0);
			print("MainLine2Passing : Switch " .. SwitchName[OperatingSwitch] .. " to Reverse");
			end;		
		WaitforCaboose = true;
		return true;

	elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
		print("MainLine2Passing : " .. DetectorID .. "Caboose detected");
		if (not EngineStopped and (SwitchClearingDistance ~= nil)) then
			-- equation for Delay calculation
			-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
			local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * MyEngineSpeed)) * 3600;
			print(string.format("MainLine2Passing : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
				BlockName[CurrentBlockNo], SwitchClearingDistance, MyEngineSpeed, SleepDelay));
			print("MainLine2Passing : " .. "Switch cleared after " .. SleepDelay .. " seconds")
			Sleep(SleepDelay);	-- give the caboose time to clear the switch
			end;
		if (LastBlockNo ~= 0) then
			ClearOccupancy(LastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy
			end;
		if (EngineStopped) then
			-- wait for NextBlockNo to be locked (either the thru or passing track)
			SetOccupancy(NextBlockNo, MyEngineNo);	-- set occupancy flag in NextBlockNo			
			end;		
		MyEngineSpeed = BlockSpeed[MyDirection][CurrentBlockNo];
		SetSpeed(MyEngineSpeed, 0, MyEngineNo, MyTIUNo);	-- start moving if stopped or adjust speed
		SetCounter(C3, MyEngineSpeed);		
		WaitforCaboose = false;
		print("MainLine2Passing : " .. DetectorID .. " " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
		end;
	return true;
	end;

return MainLine2Passing;
end;
--[[---------------------------------------------------------------------------------------]]
function createMainLine2MainLine(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance;		
	local EngineStopped = false;
	local StopDelay = 0;
	local function MainLine2MainLine(DetectorID, EngineNo, TagLocation, CarModel, Block)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK) then	-- if MyEngineNo in tag
			print("MainLine2MainLine : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			print("MainLine2MainLine : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			SetFlagName(99,"MainLine2MainLine");
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			NextBlockNo = Block;	-- set NextBlockNo to Block,			
			-- moving into a block requires the block to be empty (== 0) or
			-- be already reserved for this engine (== EnginNo)
			if (not TrySetOccupancy(NextBlockNo, MyEngineNo)) then
				-- if we can't get a lock, stop in CurrentBlockNo
				StopDelay = CalculateStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, MyEngineSpeed)
				Sleep(StopDelay);	-- Stop delay
				SetSpeed(0, 0, MyEngineNo, MyTIUNo);	-- stop engine
				EngineStopped = true;
				else
				EngineStopped = false;
				end
			WaitforCaboose = true;
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			print("MainLine2MainLine : " .. DetectorID .. "Caboose detected");
			if (not EngineStopped and (SwitchClearingDistance ~= nil)) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * MyEngineSpeed)) * 3600;
				print(string.format("MainLine2MainLine : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[CurrentBlockNo], SwitchClearingDistance, MyEngineSpeed, SleepDelay));
				print("MainLine2MainLine : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				end;
			if (LastBlockNo ~= 0) then
				ClearOccupancy(LastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy		
				end;
			if (EngineStopped) then
				-- wait for NextBlockNo to be locked
				SetOccupancy(NextBlockNo, MyEngineNo);	-- set occupancy flag in NextBlockNo			
				end;			
			MyEngineSpeed = BlockSpeed[MyDirection][CurrentBlockNo];
			SetSpeed(MyEngineSpeed, 0, MyEngineNo, MyTIUNo);	-- start moving if stopped or adjust speed
			SetCounter(C3, MyEngineSpeed);
			WaitforCaboose = false;
			print("MainLine2MainLine : " .. DetectorID .. " " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			end;
	return true;
	end;

return MainLine2MainLine;
end;
--[[---------------------------------------------------------------------------------------]]
function createPassing2MainLine(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance;
	local EngineStopped = false;
	local StopDelay = 0;	
	local function Passing2MainLine(DetectorID, EngineNo, TagLocation, CarModel, Normal_Track, Reverse_Track, OperatingSwitch, Block)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK) then	-- if MyEngineNo in tag
			print("Passing2MainLine : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			print("Passing2MainLine : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			SetFlagName(99,"Passing2MainLine");
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			NextBlockNo = Block;	-- set NextBlockNo to Block
			--moving into a block requires the block to be empty (== 0) or
			-- be already reserved for this engine (== EnginNo)
			if (not TrySetOccupancy(NextBlockNo, MyEngineNo)) then
				-- if we can't get a lock, stop in CurrentBlockNo
				StopDelay = CalculateStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, MyEngineSpeed)
				Sleep(StopDelay);	-- Stop delay
				SetSpeed(0, 0, MyEngineNo, MyTIUNo);
				EngineStopped = true;
				else
				EngineStopped = false;
				end
			WaitforCaboose = true;
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			print("Passing2MainLine : " .. DetectorID .. "Caboose detected");
			if (not EngineStopped and (SwitchClearingDistance ~= nil)) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * MyEngineSpeed)) * 3600;
				print(string.format("Passing2MainLine : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[CurrentBlockNo], SwitchClearingDistance, MyEngineSpeed, SleepDelay));
				print("Passing2MainLine : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				end;
			if (LastBlockNo ~= 0) then			
				ClearOccupancy(LastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy		
				end;
			if (EngineStopped) then
				-- wait for NextBlockNo to be locked
				SetOccupancy(NextBlockNo, MyEngineNo);	-- set occupancy flag in NextBlockNo			
				end;
			-- set OperatingSwitch switch
			if (CurrentBlockNo == Normal_Track) then
				Switch(NORMAL, Switches[OperatingSwitch][CHAN_NUMBER], Switches[OperatingSwitch][AIU_NUMBER], Switches[OperatingSwitch][TIU_NUMBER], 0);
				print("Passing2MainLine : Switch " .. SwitchName[OperatingSwitch] .. " to Normal");
				end;
			if (CurrentBlockNo == Reverse_Track) then
				Switch(REVERSE, Switches[OperatingSwitch][CHAN_NUMBER], Switches[OperatingSwitch][AIU_NUMBER], Switches[OperatingSwitch][TIU_NUMBER], 0);
				print("Passing2MainLine : Switch " .. SwitchName[OperatingSwitch] .. " to Reverse");
				end;		
			MyEngineSpeed = BlockSpeed[MyDirection][CurrentBlockNo];
			SetSpeed(MyEngineSpeed, 0, MyEngineNo, MyTIUNo);	-- start moving if stopped or adjust speed
			SetCounter(C3, MyEngineSpeed);
			WaitforCaboose = false;
			print("Passing2MainLine : " .. DetectorID .. " " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			end;
	return true;
	end;

return Passing2MainLine;
end;
--[[---------------------------------------------------------------------------------------]]
function createMainLine2Yard(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance;		
	local EngineStopped = false;
	local StopDelay = 0;
	local function MainLine2Yard(DetectorID, EngineNo, TagLocation, CarModel, Yard1, Yard2, OperatingSwitch, StopBlock)
		if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK) then		-- if MyEngineNo in tag
			print("MainLine2Yard : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			print("MainLine2Yard : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			SetFlagName(99,"MainLine2Yard");
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			NextBlockNo = StopBlock;	-- ending block number
			-- check for occupancy clear Yard2 or Yard1 - Ending block must be unoccupied otherwise stop engine
			if (not TrySetOccupancy(NextBlockNo, MyEngineNo)) then
				-- if we can't get a lock, stop in CurrentBlockNo
				StopDelay = CalculateStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, MyEngineSpeed)
				Sleep(StopDelay);	-- Stop delay
				SetSpeed(0, 0, MyEngineNo, MyTIUNo);
				EngineStopped = true;
				else
				EngineStopped = false;
				end
			print("MainLine2Yard : " .. BlockName[NextBlockNo] .. " occupancy flag value = ".. GetOccupancy(NextBlockNo));
			print("MainLine2Yard : " .. DetectorID .. "Next Block Selection = " .. BlockName[StopBlock]);
			-- set OperatingSwitch switch
			if (CurrentBlockNo == Yard1) then
				Switch(NORMAL, Switches[OperatingSwitch][CHAN_NUMBER], Switches[OperatingSwitch][AIU_NUMBER], Switches[OperatingSwitch][TIU_NUMBER], 0);
				print("MainLine2Yard : Switch " .. SwitchName[OperatingSwitch] .. " to Normal");
				end;
			if (CurrentBlockNo == Yard2) then
				Switch(REVERSE, Switches[OperatingSwitch][CHAN_NUMBER], Switches[OperatingSwitch][AIU_NUMBER], Switches[OperatingSwitch][TIU_NUMBER], 0);
				print("MainLine2Yard : Switch " .. SwitchName[OperatingSwitch] .. " to Reverse");
				end;		
			WaitforCaboose = true;
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			print("MainLine2Yard : " .. DetectorID .. "Caboose detected");
			--print("MainLine2Yard : " .. DetectorID .. "Caboose - Last Block = " .. BlockName[LastBlockNo]);
			if (not EngineStopped and (SwitchClearingDistance ~= nil)) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * MyEngineSpeed)) * 3600;
				print(string.format("MainLine2Yard : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[CurrentBlockNo], SwitchClearingDistance, MyEngineSpeed, SleepDelay));
				print("MainLine2Yard : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				end;
			if (LastBlockNo ~= 0) then
				ClearOccupancy(LastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy
				end;
			if (EngineStopped) then
				-- wait for NextBlockNo to be locked
				SetOccupancy(NextBlockNo, MyEngineNo);	-- set occupancy flag in NextBlockNo			
				end;			
			MyEngineSpeed = BlockSpeed[MyDirection][CurrentBlockNo];
			SetSpeed(MyEngineSpeed, 0, MyEngineNo, MyTIUNo);	-- start moving if stopped or adjust speed
			SetCounter(C3, MyEngineSpeed);		
			WaitforCaboose = false;
			print("MainLine2Yard : " .. DetectorID .. " " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			--print("MainLine2Yard : " .. DetectorID .. "Next Block = ".. BlockName[NextBlockNo] .. " when leaving"); 
			end;
	return true;
	end

return MainLine2Yard;
end;
--[[---------------------------------------------------------------------------------------]]
function createStopInYard(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance;		
	local EngineStopped = false;
	local StopDelay = 0;
	local function StopInYard(DetectorID, EngineNo, TagLocation, CarModel,  OffLayoutBlock)
		if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK) then	-- if MyEngineNo in tag
			print("StopInYard : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			print("StopInYard : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			SetFlagName(99,"StopInYard");
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			NextBlockNo = OffLayoutBlock;			
			-- schedule stop in CurrentBlockNo
			StopDelay = CalculateStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, MyEngineSpeed)
			Sleep(StopDelay);	-- Stop delay
			MyEngineSpeed = 0;
			SetSpeed(MyEngineSpeed, 0, MyEngineNo, MyTIUNo);
			SetCounter(C3, MyEngineSpeed);
			EngineStopped = true;
			WaitforCaboose = true;
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			print("StopInYard : " .. DetectorID .. "Caboose detected");
			--print("StopInYard : " .. DetectorID .. "Caboose - Last Block = " .. BlockName[LastBlockNo]);
			print("StopInYard : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			if (not EngineStopped and (SwitchClearingDistance ~= nil)) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * MyEngineSpeed)) * 3600;
				print(string.format("StopInYard : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[CurrentBlockNo], SwitchClearingDistance, MyEngineSpeed, SleepDelay));
				print("StopInYard : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				end;
			if (LastBlockNo ~= 0) then
				ClearOccupancy(LastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy
				end;
			WaitforCaboose = false;
			RFIDFlag = false;
			-- Occupancy flag should be on for EndBlockNo
			print("StopInYard : " .. DetectorID .. " " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			print("StopInYard : " .. DetectorID .. "Stopped!"); 
			end;	
	return true;
	end;

return StopInYard;
end;
--[[---------------------------------------------------------------------------------------]]
--[[---------------------------------------------------------------------------------------]]
function tag(Detector, Reader, EngineNo, TagLocation, CarModel, Railroad, CarNumber, TagPacket)
-- This function is called each time a tag programmed with 32 bytes of data is detected (TAGDATABLOCK4)
-- TagPacket contains 8 characters (4 hex digits) of UID and 32 characters
-- of block 4 data from the programmed tag.

BumpCounter(COUNTER01);	
local PacketLen = #TagPacket;

if (Debug() >= 8) then print("Packet Length = ", PacketLen); end;
if (Debug() >= 7) then print(TagPacket); end;

SetFlagName(98, string.format([[Tag Detector %d Reader %d]], Detector, Reader));

	-- index the table Tags[] by tag ID, value is the tag packet itself
	Tags[string.sub(TagPacket, 1, 8)] = string.sub(TagPacket, 9, 40);	-- remove checksum and EOP
	TagCount = TagCount + 1;	-- count the number of tags
	if (EngineNo > 0) then	-- its an engine
		if (BeepOn) then Beep(ASTERISK); end;
		-- next line shows a different way to quote a string using brackets
		printc(clGreen, string.format([[tag(%.2f) : Detector %d Reader %d Eng#%d %s %s #%s %s]],
			RunTime(), Detector, Reader, EngineNo, GetRailroadName(Railroad), GetEngineName(CarModel), CarNumber, GetTagLocation(TagLocation)));
		-- follow on to process the detected engine
		else	-- it's a car
		if (BeepOn) then Beep(EXCLAMATION); end;
		printc(clBlue, string.format([[tag(%.2f) : Detector %d Reader %d Car %s %s #%s]],
			RunTime(), Detector, Reader, GetRailroadName(Railroad), GetCarName(CarModel), CarNumber));
		-- follow on to process non-engine tags
		end

	-- The complete packet received is in TagPacket 
	-- (all of the fields have already been extracted into the first parameters to tag()
	--   First 8 digits are the 4 byte Tag UID in hexidecimal
	--   Next 32 digits are the 16 bytes of Block 4 Information read from the Tag in hexidecimal
	if (Debug() >= 6) then print(string.format("Packet  %s  %s", string.sub(TagPacket, 1, 8),
		string.sub(TagPacket,9,40) )); end
--[[--------------------------]]
-- Westbound trains are superior
-- if MyEngineNo is not in tag and Caboose number is not in tag, then ignore it

if (ResetinProgress) then
	-- run back to Tag 5-0
	if (Detector == 5 and Reader == 0) then
		local DetectorID = "@ 5-0 ";
		local StopDelay = 0;
		if (EngineNo == 0 and CarModel == CABOOSE) then		-- going in reverse, execute on caboose detection
			print(DetectorID .. "Caboose detected");
			-- schedule stop in StartBlockNo
			StopDelay = CalculateStopDelay(StartBlockNo, MyEngineNo, WB, MyEngineSpeed)
			SetSpeed(0, StopDelay, MyEngineNo, MyTIUNo);	-- Stop delay
			SetDirection(FORWARD, StopDelay + 5, MyEngineNo, MyTIUNo);
			ResetinProgress = false;
			end;
		end;
	return true;		

elseif (not RFIDFlag) then	-- ignore tags unless engine is moving
	return true;
	
-- Tag 5-0
elseif (Detector == 5 and Reader == 0) then
	if (Func50 == nil) then
		Func50 = createMainLine2Passing(false, SwitchClearingInches[50][MyDirection]);
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock
	Func50("@ 5-0 ", EngineNo, TagLocation, CarModel, STRUTHERS_THRU_TRACK, STRUTHERS_PASSING_TRACK, STRUTHERS_WEST);

-- Tag 4-0
elseif (Detector == 4 and Reader == 0) then
	if (Func40 == nil) then
		Func40 = createPassing2MainLine(false, SwitchClearingInches[40][MyDirection]);
		end;
	-- status: in block NextBlockNo, occupancy already set
	Func40("@ 4-0 ", EngineNo, TagLocation, CarModel, STRUTHERS_THRU_TRACK, STRUTHERS_PASSING_TRACK, STRUTHERS_EAST, NEW_CASTLE_BLOCK)

-- Tag 2-0
elseif (Detector == 2 and Reader == 0) then
	if (Func20 == nil) then
		Func20 = createMainLine2Passing(false, SwitchClearingInches[20][MyDirection]);
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock	
	Func20("@ 2-0 ", EngineNo, TagLocation, CarModel, COLLEGE_THRU_TRACK, COLLEGE_PASSING_TRACK, COLLEGE_WEST);

-- Tag 2-1
elseif (Detector == 2 and Reader == 1) then
	if (Func21 == nil) then
		Func21 = createPassing2MainLine(false, SwitchClearingInches[21][MyDirection]);
		end;
	-- status: in block NextBlockNo, occupancy already set
	Func21("@ 2-1 ", EngineNo, TagLocation, CarModel, COLLEGE_THRU_TRACK, COLLEGE_PASSING_TRACK, COLLEGE_EAST, ALIQUIPPA_BLOCK);

-- Tag 4-1
elseif (Detector == 4 and Reader == 1) then
	if (Func41 == nil) then
		Func41 = createMainLine2Passing(false, SwitchClearingInches[41][MyDirection]);
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock	
	Func41("@ 4-1 ", EngineNo, TagLocation, CarModel, J_L_THRU_TRACK, J_L_PASSING_TRACK, ALIQUIPPA_WEST);

-- Tag 1-0
elseif (Detector == 1 and Reader == 0) then
	if (Func10 == nil) then
		Func10 = createPassing2MainLine(false, SwitchClearingInches[10][MyDirection]);
		end;
	-- status: in block NextBlockNo, occupancy already set
	Func10("@ 1-0 ", EngineNo, TagLocation, CarModel, J_L_THRU_TRACK, J_L_PASSING_TRACK, ALIQUIPPA_EAST, MCKEES_ROCKS_BLOCK);

-- Tag 5-1
elseif (Detector == 5 and Reader == 1) then
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock	
	if (LLLoop == false) then
		if (Func51 == nil) then
			Func51 = createMainLine2Yard(false, SwitchClearingInches[51][MyDirection]);
			end;
		Func51("@ 5-1 ", EngineNo, TagLocation, CarModel, MCKEES_ROCKS_YARD1_BLOCK, MCKEES_ROCKS_YARD2_BLOCK, MCKEES_ROCKS_YARD, EndBlockNo);
		else
		if (Func51 == nil) then
			Func51 = createMainLine2MainLine(false);
			end;
		Func51("@ 5-1 ", EngineNo, TagLocation, CarModel, YOUNGSTOWN_BLOCK);
		end;

-- Tag 3-0
elseif (Detector == 3 and Reader == 0) then
	-- status: in block NextBlockNo, occupancy already set
	if (LLLoop == false) then
		if (Func30 == nil) then
			Func30 = createStopInYard(false, SwitchClearingInches[30][MyDirection]);
			end;
		Func30("@ 3-0 ", EngineNo, TagLocation, CarModel,  OFF_LAYOUT_EAST);
		end;
	end;

return true;	-- true=continue to process tags, false=stop processing tags
end	
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
-- this function is called once when the user presses the [STOP] button or the Stop() function is called
print("All Newly Detected Tags:");
local k, v;
for k,v in  pairs(Tags) do		-- dump out the Tags table
	--                           . . . . . . . . . . . . . . . .
	-- print format:  6619044A  88080101030108011B02202031353536
	print(k .. "  " .. v);
	end
SetSpeed(0, 0, MyEngineNo, MyTIUNo);
Rate(1, 1, 2, MyEngineNo, MyTIUNo);
print("Shutdown in Progress.....");
if (StartedFlag) then
	print("Shutting down the Engine");
	Sleep(10);
	EngineShutdown(MyEngineNo, MyTIUNo);
	StartedFlag = false;
	end
RFIDFlag = false;
ResetinProgress = false;
print(string.format("cleanup() Complete at %.2f", RunTime()));
return true;		-- false=cleanup failed, true=cleanup succeeded
end
--[[---------------------------------------------------------------------------------------]]
function DTOButton(Eng, TIU)
print(string.format("Engine %d Trip Odometer %.3f Smiles", Eng, DTO(Eng, MyTIUNo)));
return true;
end
Function01Name, Function01Label = DTOButton, "Show DTO";
--[[---------------------------------------------------------------------------------------]]
function DODButton(Eng, TIU)
print(string.format("Engine %d Odometer %.3f Smiles", Eng, DOD(Eng, MyTIUNo)));
return true;
end
Function02Name, Function02Label = DODButton, "Show DOD";
--[[---------------------------------------------------------------------------------------]]
function SoundOffButton(Eng, TIU)
print("Sound Off");
S1 = GetVolume(MASTER_VOLUME, MyEngineNo, MyTIUNo);	-- get Volume
print(" Sound Off - Engine " .. MyEngineNo .. " Level was " .. S1);
EngineSoundSave = S1;
SoundOff = true;
SetVolume(MASTER_VOLUME, 0, 0, MyEngineNo, MyTIUNo);
return true;
end
Function19Name, Function19Label = SoundOffButton, "Sound Off";
--[[---------------------------------------------------------------------------------------]]
function SoundOnButton(Eng, TIU)
print("Sound On");
if (SoundOff) then
	S1 = EngineSoundSave;		
	print(" Sound On - Engine " .. MyEngineNo .. " Level restored to " .. S1);
	SoundOff = false;
	SetVolume(MASTER_VOLUME, S1, 0, MyEngineNo, MyTIUNo);
	end;
return true;
end
Function20Name, Function20Label = SoundOnButton, "Sound On";
--[[---------------------------------------------------------------------------------------]]
function InitButton(Eng, TIU)
print("Initialize");
print("Set all Switches to Normal postion");
local k, v;
-- set all switches to NORMAL position
for k,v in pairs(Switches) do
	print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
	Switch(NORMAL, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
	Sleep(0.5);
	end
if (Debug() > 3) then
	-- set all switches to REVERSE
	for k,v in pairs(Switches) do
		print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
		Switch(REVERSE, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
		Sleep(0.5);
		end;
	-- set all switches to NORMAL
	for k,v in pairs(Switches) do
		print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
		Switch(NORMAL, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
		Sleep(0.5);
		end;
	end;
-- clear occupancy flags
local x;
for x = FIRST_BLOCK, LAST_BLOCK do
	OverrideOccupancy(x, 0);
	end;
--[[-----------------------------]]
-- clear all tag handling functions
local Func30 = nil;
local Func51 = nil;
local Func10 = nil;
local Func41 = nil;
local Func21 = nil;
local Func20 = nil;
local Func40 = nil;
local Func50 = nil;

print("Initialization is Complete");
return true;
end
Function06Name, Function06Label = InitButton, "Initialize Layout";
--[[---------------------------------------------------------------------------------------]]
function StartButton(Eng, TIU)
print("Start the Run");

--[[-----------------------------]]
-- clear all tag handling functions
local Func30 = nil;
local Func51 = nil;
local Func10 = nil;
local Func41 = nil;
local Func21 = nil;
local Func20 = nil;
local Func40 = nil;
local Func50 = nil;

--[[------------------------------------------]]
LastBlockNo = OFF_LAYOUT_WEST;
CurrentBlockNo = StartBlockNo;
OverrideOccupancy(StartBlockNo, MyEngineNo);	-- force set occupancy flag
-- check that YOUNGSTOWN_BLOCK is not occupied
NextBlockNo = YOUNGSTOWN_BLOCK;
SetOccupancy(NextBlockNo, MyEngineNo);	-- occupied block

--print("StationName = " .. Pretty(StationName));
--print("StationName = " .. Pretty(BlockName));
--print("Switches = " ..  Pretty(Switches));
--print("NextBlock = " ..  Pretty(NextBlock));

-- start the engine
Rate(1, 1, 0, MyEngineNo, MyTIUNo);
print("setup : Set accell/decell rates to 1");
if (StartedFlag == false) then
	EngineStartUp(MyEngineNo, MyTIUNo);
	print("Starting up the Engine");
	StartedFlag = true;
	Sleep(20);
	end
SetDirection(FORWARD, 0, MyEngineNo, MyTIUNo);
RFIDFlag = true;
Whistle(ON, 20, MyEngineNo, MyTIUNo);
Whistle(OFF, 22, MyEngineNo, MyTIUNo);
print("Trip Odometer " .. DTO(MyEngineNo, MyTIUNo)); 	
Sleep(3);

-- position YOUNGSTOWN_YARD switch as needed
-- depending on value of CurrentBlockNo
if (CurrentBlockNo == YOUNGSTOWN_YARD1_BLOCK) then
	Switch(NORMAL, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	end;
if (CurrentBlockNo == YOUNGSTOWN_YARD2_BLOCK) then
	Switch(REVERSE, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	end;
--
SetSpeed(MyEngineSpeed, 5, MyEngineNo, MyTIUNo);
--Sleep(14);
--print(string.format("Speed %d Smph",GetSpeed(MyEngineNo, MyTIUNo)));
--for x = FIRST_BLOCK, LAST_BLOCK do
--	print(BlockName[x] .. " = ", GetOccupancy(x))
--	end;	
--[[----------------------------------------------------]]
print(string.format("RunTime = %.2f seconds", RunTime()));
-- TODO once both engines exit the yard, throw the Lionel-Ives Switch
return true;
end
Function07Name, Function07Label = StartButton, "Start Run";
--[[---------------------------------------------------------------------------------------]]

function event(FromPC, Type, P1, P2, P3)
print("event() Running - From PC# ", FromPC, " EventType = ", Type, " Parameters = ", P1, " ", P2, " ", P3);
return true;		-- false=event failed, true=event succeeded	
end
--[[---------------------------------------------------------------------------------------]]

function OccButton(Eng, TIU)
printc(clGreen, "Occupancy Flags -------------------------");
local x, FColor;
for x = FIRST_BLOCK, LAST_BLOCK do
	local flag = GetGlobalInt(x);
	if (flag == 0) then
		FColor = clBlack;
		else
		FColor = clRed;
		end;
	printc(FColor, BlockName[x] .. " = ", flag)
	end;	
return true;
end
Function08Name, Function08Label = OccButton, "Occupancy Flags";
--[[---------------------------------------------------------------------------------------]]
-- Run the engine back to the start (in reverse)
function ResetButton()
if (ResetinProgress) then
	return;	-- already being reset
	end;
print("Return to Initial Position");
-- stop the engine
SetSpeed(0, 0, MyEngineNo, MyTIUNo);
RFIDFlag = false;
Sleep(5);
--[[-----------------------------]]
-- clear all tag handling functions
local Func30 = nil;
local Func51 = nil;
local Func10 = nil;
local Func41 = nil;
local Func21 = nil;
local Func20 = nil;
local Func40 = nil;
local Func50 = nil;

-- position YOUNGSTOWN_YARD switch as needed
-- depending on value of StartBlockNo
if (StartBlockNo == YOUNGSTOWN_YARD1_BLOCK) then
	Switch(NORMAL, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	end;
if (StartBlockNo == YOUNGSTOWN_YARD2_BLOCK) then
	Switch(REVERSE, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	end;
--
-- start the engine
if (not StartedFlag) then
	print("Starting up the Engine");
	EngineStartUp(MyEngineNo, MyTIUNo);
	StartedFlag = true;
	Sleep(20);
	end
-- direction to reverse
SetDirection(REVERSE, 0, MyEngineNo, MyTIUNo);
MyEngineSpeed = BlockSpeed[MyDirection][CurrentBlockNo];
SetSpeed(MyEngineSpeed, 5, MyEngineNo, MyTIUNo);
SetCounter(C3, MyEngineSpeed);
ResetinProgress = true;
return true
end;
Function09Name, Function09Label = ResetButton, "Initial Position";
--[[---------------------------------------------------------------------------------------]]

function BeepOffButton()
	print("Beep Off");
	BeepOn = false;
	return true;
end
Function14Name, Function14Label = BeepOffButton, "Beep Off";
--[[---------------------------------------------------------------------------------------]]

function BeepOnButton()
	print("Beep On");
	BeepOn = true;
	return true;
end
Function15Name, Function15Label = BeepOnButton, "Beep On";
--[[---------------------------------------------------------------------------------------]]
function LLLButton()
	print("Lionel-Ives Loop On");
	LLLoop = true;
	return true;
end
Function05Name, Function05Label = LLLButton, "Lionel-Ives Loop";
--[[---------------------------------------------------------------------------------------]]
--[[
function tick()
-- demonstration of updating a Flag Name as a comment or status message
SetFlagName(100, string.format("RunTime = %d seconds", math.floor(RunTime())));
return true;
end;]]
--[[---------------------------------------------------------------------------------------]]
