--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2021 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
title = "Signal Master Test"
require([[defines]]);
require([[functions]]);

local Count = 0;
-- These labels appear on each of the tabs for the Function Buttons
Tab01Label = "Not Used";
Tab02Label = "Not Used";
Tab03Label = "Not Used";

Tags = {};		-- table to hold all of the tags detected in sequence
TagCount = 0;
-- state machine states
IDLE = 0;
SM = IDLE;
ErrCount = 0;
--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
	print("setup() Running");
	ShowSignals();	-- not really needed since first SetSignal() will show the Signal Window
	DeleteAllSignals();	-- clear the window
	print("Creating Signals");
	CreateSignal(1, WB PERMISSIVE_IN_OPEN_COUNTRY, 0, 0, "AA", AsDark, 3, 0);	-- Set signal #1 to 3 light, dark
	CreateSignal(2, WB, PERMISSIVE_IN_OPEN_COUNTRY, 100, 0, "BB", AsDark, 3, 268);	-- Set signal #2 to 3 light, dark, Number Plate
	CreateSignal(3, WB, PERMISSIVE_IN_OPEN_COUNTRY, 200, 0, "CC", AsDark, 3, 111);	-- Set signal #3 to 3 light, dark, Number Plate
	CreateSignal(4, WB, PERMISSIVE_IN_OPEN_COUNTRY, 300, 0, "DD", AsDark, 3, 0);	-- Set signal #4 to 3 light, dark
	CreateSignal(5, EB, PERMISSIVE_IN_OPEN_COUNTRY, 300, 100, "G2", AsDark, 2, 0);	-- Set signal #5 to 1 light, dark	
	CreateSignal(6, EB, 400, 100, "G3", AsDark, 1, 0);	-- Set signal #6 to 2 light, dark
	CreateSignal(25, EB, PERMISSIVE_IN_OPEN_COUNTRY, 300, 400, "J2", AsClear, 2, 0);	-- Set signal #25 to 1 light, AsClear	
	CreateSignal(26, EB, PERMISSIVE_IN_OPEN_COUNTRY, 400, 400, "J3", AsApproachLimited, 3, 0);	-- Set signal #26 to 3 light, AsApproachLimited
	print("Signals Created");
	Sleep(6);
	SetAspect(1, AsMediumClear);	-- Set signal #1 to Medium Clear (4)
	Asp = GetAspect(1);
	if (Asp ~= AsMediumClear) then print("Signal 1 Aspect not AsMediumClear (4)"); ErrCount = ErrCount + 1; end;
	
	--Sleep(6);
	--SetAspect(2, AsClear);	-- Set signal #2 to Clear (2)
	--Asp = GetAspect(2);
	--if (Asp ~= AsClear) then print("Signal 2 Aspect not AsClear (2)"); ErrCount = ErrCount + 1; end;
	
	SetAspect(2, AsApproach);	-- Set signal #2 to AsApproach (3)
	Asp = GetAspect(2);
	if (Asp ~= AsApproach) then print("Signal 2 Aspect not AsApproach (3)"); ErrCount = ErrCount + 1; end;
	
	SetAspect(5, AsApproachLimited);	-- Set signal #5 to AsApproachLimited (12)
	Asp = GetAspect(5);
	if (Asp ~= AsApproachLimited) then print("Signal 5 Aspect not AsApproachLimited (12)"); ErrCount = ErrCount + 1; end;
	
	SetAspect(6, AsClear);	-- Set signal #6 to AsClear (2)
	Asp = GetAspect(6);
	if (Asp ~= AsClear) then print("Signal 6 Aspect not AsClear (2)"); ErrCount = ErrCount + 1; end;
	
		
	print("Reflecting signals from Block Signal window into the Cab Display");
	return true;		-- false=setup failed, true=setup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
function loop(Engine, TIU)
	-- Always call Sleep(X) at least once in this function if it returns true;
	--print("loop() Running");


	-- these signals, #1, #2, #3 and #4, can be associated to Cab Signals by "Cab Signal Test.lua".
	Sleep(2);
	SetAspect(1, AsMediumClear);	-- Set signal #1 to Medium Clear (4)
	Asp = GetAspect(1);
	if (Asp ~= AsMediumClear) then print("Signal 1 Aspect not AsMediumClear (4)"); ErrCount = ErrCount + 1; end;
	--
	Sleep(2);
	SetAspect(2, AsClear);	-- Set signal #2 to Clear (2)
	Asp = GetAspect(2);
	if (Asp ~= AsClear) then print("Signal 2 Aspect not AsClear (2)"); ErrCount = ErrCount + 1; end;
	--
	Sleep(2);
	SetAspect(1, AsMediumApproach);	-- Set signal #1 to Medium Approach (5)
	Asp = GetAspect(1);
	if (Asp ~= AsMediumApproach) then print("Signal 1 Aspect not AsMediumApproach (5)"); ErrCount = ErrCount + 1; end;
	--
	val = math.random(AsDark, AsLast - 1);
	Sleep(2);
	SetAspect(2, val);	-- Set signal #2 to Random Number (val)	
	Asp = GetAspect(2);
	if (Asp ~= val) then print("Signal 2 Aspect not ", AspectNames[val] , " (", val, ")"); end;
	--
	Sleep(2);
	SetAspect(3, AsMediumApproach);	-- Set signal #3 to Medium Approach (5)
	Asp = GetAspect(3);
	if (Asp ~= AsMediumApproach) then print("Signal 3 Aspect not AsMediumApproach (5)"); ErrCount = ErrCount + 1; end;
	--
	Sleep(2);
	SetAspect(4, AsStopSignal);	-- Set signal #4 to Stop (1)	
	Asp = GetAspect(4);
	--
	Sleep(2);
	SetAspect(3, AsMediumClear);	-- Set signal #3 to Medium Clear (4)
	Asp = GetAspect(3);
	if (Asp ~= AsMediumClear) then print("Signal 3 Aspect not AsMediumClear (4)"); ErrCount = ErrCount + 1; end;
	--
	Sleep(2);
	SetAspect(4, AsClear);	-- Set signal #4 to Clear (2)
	Asp = GetAspect(4);
	if (Asp ~= AsClear) then print("Signal 4 Aspect not AsClear (2)"); ErrCount = ErrCount + 1; end;
	--
	SetAspect(5, AsLimitedClear);	-- Set signal #5 to AsLimitedClear (13)
	Asp = GetAspect(5);
	if (Asp ~= AsLimitedClear) then print("Signal 5 Aspect not AsLimitedClear (13)"); ErrCount = ErrCount + 1; end;
		
	--print("Almost Done...");
	return true;  --Stop("DONE");
	end
--[[---------------------------------------------------------------------------------------]]
function tag(Detector, Reader, EngineNo, TagLocation, CarModel, Railroad, CarNumber, TagPacket)
	-- This function is called each time a tag programmed with 32 bytes of data is detected (TAGDATABLOCK4)
	-- TagPacket contains 8 characters (4 hex digits) of UID and 32 characters
	-- of block 4 data from the programmed tag.
	--Beep();	-- sound a tone
	BumpCounter(COUNTER01);	

	-- index the table Tags[] by tag ID, value is the tag packet itself
	Tags[string.sub(TagPacket, 1, 8)] = string.sub(TagPacket, 9, 40);	-- remove checksum and EOP
	TagCount = TagCount + 1;	-- count the number of tags
	
	if (EngineNo > 0) then	-- its an engine
		-- next line shows a different way to quote a string using brackets
		print(string.format([[tag(%.2f) : Detector %d Reader %d Eng#%d %s %s #%s %s]],
			RunTime(), Detector, Reader, EngineNo, GetRailroadName(Railroad), GetEngineName(CarModel), CarNumber, GetTagLocation(TagLocation)));
		-- follow on to process the detected engine
		else	-- it's a car
		print(string.format([[tag(%.2f) : Detector %d Reader %d Car %s %s #%s]],
			RunTime(), Detector, Reader, GetRailroadName(Railroad), GetCarName(CarModel), CarNumber));
		-- follow on to process the detected Caboose	
		if (CarModel ~= CABOOSE) then	-- ignore cars other than Caboose
			return true;	-- true=continue to process tags, false=stop processing tags
			end
		end
	
	-- 
	print("Process Signal");	
	BumpCounter(COUNTER02);	
	-- Detection of engine tag causes the trailing signal to go red

	-- Detection of caboose tag causes the trailing signal to go green
	
	
--[[---------------------------------------------------------------------------------------]]
-- STATE MACHINE
--[[---------------------------------------------------------------------------------------]]	
	SetCounter(COUNTER04, SM);
--[[---------------------------------------------------------------------------------------]]	
	if (SM == IDLE) then	-- do nothing 
		-- continue in this state
		return true;
		end
--[[---------------------------------------------------------------------------------------]]		
	if (SM == IDLE) then

		-- continue in this state
		return true;
		end
		
	return true;	-- true=continue to process tags, false=stop processing tags
	end	
	
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
	-- this function is called once when the user presses the [STOP] button or the Stop() function is called
	print("cleanup() Running");
	if (ErrCount == 0) then
		print("No errors");
		else
		print(ErrCount .. " Error(s)");
		end;	
	return true;		-- false=cleanup failed, true=cleanup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
