--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2020 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
title = "Headway EB"	-- Eastbound
require([[defines]]);
require([[functions]]);
--
--[[
When using this module, all variables (including functions!) must be declared through 
a regular assignment (even assigning nil will do) in a strict scope before being used 
anywhere or assigned to inside a nested scope.
https://github.com/lua-stdlib/strict
]]
local _ENV = require 'std.strict' (_G)	-- strict.lua will detect undeclared global variables
--
local TagCount = 0;
-- Counter Labels
Counter01Label = "tags()";
Counter03Label = "Block Speed Limit";
local C3 = COUNTER03;
Counter04Label = "Target Distance";
local C4 = COUNTER04;
Counter05Label = "Stopping Distance";
local C5 = COUNTER05;
Counter06Label = "Stop Delay";
local C6 = COUNTER06;

local Tags = {};		-- table to hold all of the tags detected in sequence
local SoundOn = true;

local MyDirection = 0;
local StartBlockNo, EndBlockNo;
local CurrentBlockNo, LastBlockNo, NextBlockNo;
local StartedFlag = false;
local RFIDFlag = false;
local ResetinProgress = false;
local Headway = false;
local LLLoop = true;	-- start in looping mode
local Layout4;
local MyTIUNo, Enum;
local MyEngineNo = 12;
Title(title, MyEngineNo);
local SoundOff = false;
local EngineSoundSave;
local BeepOn = false;
local BlockSpeedLimit;	-- Smph
MASTER_TIU = nil;	-- TIU number used by Layout4 (global)
-- clear all tag handling functions
local Func30 = nil;
local Func51 = nil;
local Func10 = nil;
local Func41 = nil;
local Func21 = nil;
local Func20 = nil;
local Func40 = nil;
local Func50 = nil;
local Func60 = nil;
local Func61 = nil;

-- These labels appear on each of the tabs for the Function Buttons
Tab01Label = "Control";
Tab02Label = "Support";
Tab03Label = "Not Used";
--[[---------------------------------------------------------------------------------------]]
-- include the Thinking Engines functions
--require([[Thinking Engines]]);
function CalculateBlockStopDelay(StoppingBlockNo, EngineNo, direction, EngSpeed)
--
-- Calculate the stop delay based on parameters
-- Returns the stop delay in seconds and the target distance in inches
local Target_Station;
local Target_Distance;
local StopDelay;
local StoppingDistance
Target_Station = BlockName[StoppingBlockNo];
--print("Block = " .. BlockName[StoppingBlockNo] .. ", Direction = " .. direction .. ", Type = " .. FREIGHT);
--Target_Detector = Layout4.STOPBlock(StoppingBlockNo, direction, DETECTOR);
--Target_Reader = Layout4.STOPBlock(StoppingBlockNo, direction, READER);
if (EngineNo == 7 or EngineNo == 13) then
	Target_Distance = Layout4.STOPBlock(StoppingBlockNo, direction, PASSENGER);	--Passenger StoppingDistance;
	else
	Target_Distance = Layout4.STOPBlock(StoppingBlockNo, direction, FREIGHT);	--Freight StoppingDistance;
	end
if (Target_Distance == nil) then
	Target_Distance = 0;
	StopDelay = 0;
	printc(clRed, "CalculateBlockStopDelay() : Target_Distance missing in Layout description");
	return StopDelay, Target_Distance;
	end
SetCounter(C4, Target_Distance);
--print(BlockName[StoppingBlockNo] .. " " .. direction .. ", Target Distance = " .. Target_Distance);
-- equation for Stop Delay calculation
-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
StoppingDistance = Layout4.Stopping(EngineNo, EngSpeed);
StopDelay =  (Target_Distance - StoppingDistance) * (48 / (12 * 5280 * EngSpeed)) * 3600;
-- if the StopDelay value comes out negative, that means we have to stop
-- very quickly. If that happens, set the Decell rate very high and then
-- set the speed to zero.
SetCounter(C5, StoppingDistance);
SetCounter(C6, StopDelay);
if (StopDelay < 0) then
	StopDelay = 0;
	printc(clRed, "CalculateBlockStopDelay() : Engine cannot stop in time (Stopping Distance > Target_Distance)");
	return StopDelay, Target_Distance;
	end;
print(string.format("%s : Target Distance = %d in - Stopping Distance = %.3f in - Speed %d Smph = Stop Delay %.3f seconds",
	BlockName[StoppingBlockNo], Target_Distance, StoppingDistance, EngSpeed, StopDelay));
print("CalculateBlockStopDelay() : Set speed to 0 after " .. StopDelay .. " seconds")
return StopDelay, Target_Distance;
end;
--[[---------------------------------------------------------------------------------------]]
function createMainLine2Passing(flag)
	local WaitforCaboose = flag;
	--local SwitchClearingDistance = SwDistance or 24;	
	local EngineStopped = false;
	local StopDelay = 0;
	local GotBlock = false;
	local StopTime = 0;
	local Target_Distance = 0;
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;	
	local function MainLine2Passing(DetectorID, EngineNo, TagLocation, CarModel, Normal_Track, Reverse_Track, ThisSwitch, NextSwitch)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		-- TODO NextSwitch appears to always be nil so it may not be needed
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			print("MainLine2Passing : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			--print("MainLine2Passing : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			--
			RegisterCabSignals(NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL], NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]);
			local Aspect = (GetCabAspect(HOMESIGNAL));
			print ("--> Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
			-- TODO check if home signal is AsStopSignal or AsStopandProceed. If it is, stop in this block. Set EngineSTopped = true.
			-- Then go on to check TrySetOccupancy and if we get it, start moving again when AsStopSignal is cleared.
			if (Aspect == AsStopSignal or Aspect == AsStopandProceed) then
				-- stop in this block
				print("--> Home Signal aspect is Stop Signal or Stop and Proceed. Stop the engine");
				end;
			--	
			-- Adjust speed for the current block
			BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
			SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
			SetCounter(C3, BlockSpeedLimit);
			print(string.format("%s : Speed %d Smph", BlockName[CurrentBlockNo], BlockSpeedLimit));

			-- check for occupancy clear Reverse_Track or Normal_Track - one or the other must be unoccupied otherwise fail
			-- choose based on superiority or choose empty block. Put choice into NextBlockNo.
			-- this code checks one track then the other track, if deadlock, it waits.
			if (MyDirection == SuperiorDirection) then
				if (TrySetOccupancy(Normal_Track, MyEngineNo)) then
					NextBlockNo = Normal_Track;		-- Westbound trains are superior, take the through track if possible
					StopDelay = 0;
					EngineStopped = false;
					GotBlock = true;
					StopTime = 0;
					else
					--[[
					-- check inferior direction track
					if (TrySetOccupancy(Reverse_Track, MyEngineNo)) then
						NextBlockNo = Reverse_Track;
						StopDelay = 0;
						EngineStopped = false;
						GotBlock = true;
						StopTime = 0;
						else
					]]
					NextBlockNo = Normal_Track;
						-- if we can't get a lock, stop in CurrentBlockNo
						StopDelay, Target_Distance = CalculateBlockStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, BlockSpeedLimit)
						-- Stop delay any positive value including 0
						if (StopDelay == 0) then
							Rate(0, 8, 0, MyEngineNo, MyTIUNo);	-- stop as quickly as possible
							end;
						SetSpeed(0, StopDelay, MyEngineNo, MyTIUNo);	-- stop engine;
						if (StopDelay == 0) then
							Rate(0, 1, 0, MyEngineNo, MyTIUNo);
							end;
						EngineStopped = true;
						GotBlock = false;
						StopTime = RunTime();
						end

				else	-- MyDirection ~= SuperiorDirection
				if (TrySetOccupancy(Reverse_Track, MyEngineNo))  then
					NextBlockNo = Reverse_Track;	-- Eastbound trains are inferior, take the passing track if possible
					StopDelay = 0;
					EngineStopped = false;
					GotBlock = true;
					StopTime = 0;
					else
					--[[
					-- check superior direction track
					if (TrySetOccupancy(Normal_Track, MyEngineNo)) then
						NextBlockNo = Normal_Track;
						StopDelay = 0;
						EngineStopped = false;
						GotBlock = true;
						StopTime = 0;
						else
					]]
					NextBlockNo = Reverse_Track;
					-- if we can't get a lock, stop in CurrentBlockNo
					StopDelay, Target_Distance = CalculateBlockStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, BlockSpeedLimit)
					-- Stop delay
					if (StopDelay == 0) then
						Rate(0, 8, 0, MyEngineNo, MyTIUNo);	-- stop as quickly as possible
						end;
					SetSpeed(0, StopDelay, MyEngineNo, MyTIUNo);	-- stop engine
					if (StopDelay == 0) then
						Rate(0, 1, 0, MyEngineNo, MyTIUNo);
						end;
					EngineStopped = true;
					GotBlock = false;
					StopTime = RunTime();
					end;
				end
			if (GotBlock) then
				DisplayBlockNames("MainLine2Passing : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
				end;

			-- engine was stopped quickly
			if (not GotBlock and Target_Distance <= 50) then
				-- Caboose may not have passed over detector yet and it may occupy two blocks
				-- wait for one block to be locked (default wait 10 minutes)
				-- wait for either Normal_Track or Reverse_Track to lock
				local result;
				if (MyDirection == SuperiorDirection) then
					result, NextBlockNo = SingleSetOccupancy({Normal_Track}, MyEngineNo);
					if (not result) then
						return Stop("MainLine2Passing : Timed out waiting for " .. BlockName[Normal_Track]);
						end;
					else
					result, NextBlockNo = SingleSetOccupancy({Reverse_Track}, MyEngineNo);
					if (not result) then
						return Stop("MainLine2Passing : Timed out waiting for " .. BlockName[Reverse_Track]);
						end;
					end;
				DisplayBlockNames("MainLine2Passing : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
				GotBlock = true;	-- indicate Block is locked
				if (EngineStopped) then
					local Aspect = (GetCabAspect(HOMESIGNAL));
					print ("--> Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
					-- start moving if stopped - TODO check that home signal is not AsStopSignal (but it can be AsStopandProceed)
					if (Aspect == AsStopSignal) then
						-- we are stopped, so wait until the Stop aspect is cleared.
						print("--> Home Signal aspect is Stop Signal. Wait for it to clear before proceeding.");
						end;
					local UsedUpTime = RunTime() - StopTime;	-- never negative
					local StartDelay;
					if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
						StartDelay = 0;
						else
						StartDelay = StopDelay - UsedUpTime + 5;	--UsedUpTime always <= StopDelay
						end;
					BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
					SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
					SetCounter(C3, BlockSpeedLimit);
					print(string.format("MainLine2Passing : %s : Speed %d Smph = Start Delay %.3f seconds",
						BlockName[CurrentBlockNo], BlockSpeedLimit, StartDelay));
					EngineStopped = false;	-- indicate that we handled the stop here
					end;
				end;
			--[[
			if (not GotBlock and Target_Distance > 50) then
				-- defer wait to Caboose detection code			
				end;
			--]]
			if (GotBlock) then		
				--print("MainLine2Passing : " .. DetectorID .. "Next Block Selection = " .. BlockName[NextBlockNo]);
				-- Next switch is not set until the engine goes over the reader just before the switch
				-- set NextSwitch switch
				if (NextSwitch ~= nil) then
					if (NextBlockNo == Normal_Track) then
						Switch(NORMAL, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("MainLine2Passing : Next Switch " .. SwitchName[NextSwitch] .. " to Normal");
						end;
					if (NextBlockNo == Reverse_Track) then
						Switch(REVERSE, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("MainLine2Passing : Next Switch " .. SwitchName[NextSwitch] .. " to Reverse");
						end;
					end;
				end;
			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;			
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			-- in the Caboose section of code, use MyCurrentBlockNo, MyLastBlockNo, and MyNextBlockNo
			print("MainLine2Passing : " .. DetectorID .. "Caboose detected");
			--
			-- ASAP Once the caboose is detected (without any clearing delay) return switch at entrance of passing track back
			-- to NORMAL - this switch was set to REVERSE as result engine entering the block
			--
			if (ThisSwitch ~= nil) then
				Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
				print("MainLine2Passing : This Switch " .. SwitchName[ThisSwitch] .. " to Normal");
				end;			
			if (MyLastBlockNo ~= 0) then
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy
				end;
			if (not GotBlock) then
				-- wait for all blocks to be locked (default wait 10 minutes)
				-- wait for NextBlockNo to be locked (either the thru or passing track)
				local result;
				-- set occupancy flag in NextBlockNo
				if (NextBlockNo == Normal_Track) then
					result, NextBlockNo = SingleSetOccupancy({Normal_Track}, MyEngineNo);
					if (not result) then
						return Stop("MainLine2Passing : Timed out waiting for " .. BlockName[Normal_Track]);
						end;
					end;
				if (NextBlockNo == Reverse_Track) then
					result, NextBlockNo = SingleSetOccupancy({Reverse_Track}, MyEngineNo);
					if (not result) then
						return Stop("MainLine2Passing : Timed out waiting for " .. BlockName[Reverse_Track]);
						end;
					end;
				DisplayBlockNames("MainLine2Passing : ", MyLastBlockNo, MyCurrentBlockNo, NextBlockNo);
				GotBlock = true;	-- indicate Block is locked
				-- 
				if (NextSwitch ~= nil) then
					-- set NextSwitch switch
					if (CurrentBlockNo == Normal_Track) then
						Switch(NORMAL, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("MainLine2Passing : Next Switch " .. SwitchName[NextSwitch] .. " to Normal");
						end;
					if (CurrentBlockNo == Reverse_Track) then
						Switch(REVERSE, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("MainLine2Passing : Next Switch " .. SwitchName[NextSwitch] .. " to Reverse");
						end;
					end;
				end;
			if (EngineStopped) then
				local Aspect = (GetCabAspect(HOMESIGNAL));
				print ("--> Home Signal in ", BlockName[MyCurrentBlockNo], " : " .. AspectNames[Aspect]);
				-- start moving if stopped - TODO check that home signal is not AsStopSignal (but it can be AsStopandProceed)
				if (Aspect == AsStopSignal) then
					-- we are stopped, so wait until the Stop aspect is cleared.
					print("--> Home Signal aspect is Stop Signal. Wait for it to clear before proceeding.");
					end;
				local UsedUpTime = RunTime() - StopTime;	-- never negative
				local StartDelay;
				if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
					StartDelay = 0;
					else
					StartDelay = StopDelay - UsedUpTime + 5;	--UsedUpTime always <= StopDelay
					end;			
				BlockSpeedLimit = BlockSpeed[MyDirection][MyCurrentBlockNo];
				SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
				SetCounter(C3, BlockSpeedLimit);
				EngineStopped = false;	-- indicate that we handled the stop here
				print(string.format("MainLine2Passing : %s : Speed %d Smph = Start Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], BlockSpeedLimit, StartDelay));		
				end;
				
			WaitforCaboose = false;
			--print("MainLine2Passing : " .. DetectorID .. " " .. BlockName[MyLastBlockNo] .. " > " .. BlockName[MyCurrentBlockNo] .. " > " .. BlockName[MyNextBlockNo]);
			end;
	return true;
	end;

return MainLine2Passing;
end;
--[[---------------------------------------------------------------------------------------]]
function createMainLine2MainLine(flag)
	local WaitforCaboose = flag;
	local EngineStopped = false;
	local StopDelay = 0
	local GotBlock = false;
	local StopTime = 0;
	local Target_Distance = 0;
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;
	local function MainLine2MainLine(DetectorID, EngineNo, TagLocation, CarModel, ThisSwitch, Block)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		if (type(Block) ~= "table") then return Stop("MainLine2MainLine : Block not a table"); end
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			print("MainLine2MainLine : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			--print("MainLine2MainLine : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			--
			RegisterCabSignals(NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL], NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]);
			local Aspect = (GetCabAspect(HOMESIGNAL));
			print ("--> Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
			-- TODO check if home signal is AsStopSignal or AsStopandProceed. If it is, stop in this block. Set EngineSTopped = true.
			-- Then go on to check TryMultiSetOccupancy and if we get it, start moving again.
			if (Aspect == AsStopSignal or Aspect == AsStopandProceed) then
				-- stop in this block
				print("--> Home Signal aspect is Stop Signal or Stop and Proceed. Stop the engine");
				end;
			--			
			-- Adjust speed for the current block
			BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
			SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
			SetCounter(C3, BlockSpeedLimit);
			print(string.format("%s : Speed %d Smph", BlockName[CurrentBlockNo], BlockSpeedLimit));

			-- no engine entrance switch settings
				
			NextBlockNo = Block[1];	-- set NextBlockNo to Block[1]
			DisplayBlockNames("MainLine2MainLine : ", LastBlockNo, CurrentBlockNo, NextBlockNo);			
			-- moving into a block requires the block to be empty (== 0) or
			-- be already reserved for this engine (== EnginNo)
			if (not TryMultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then
				-- if we can't get a lock, stop in CurrentBlockNo
				StopDelay, Target_Distance = CalculateBlockStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, BlockSpeedLimit)
				-- Stop delay may be zero for immediate stop
				if (StopDelay == 0) then
					Rate(0, 8, 0, MyEngineNo, MyTIUNo);	-- stop as quickly as possible
					end;				
				SetSpeed(0, StopDelay, MyEngineNo, MyTIUNo);	-- stop engine
				if (StopDelay == 0) then
					Rate(0, 1, 0, MyEngineNo, MyTIUNo);
					end;
				EngineStopped = true;	-- engine was stopped
				GotBlock = false;
				StopTime = RunTime();
				else
				-- got a lock on the needed block(s), continue on
				StopDelay = 0;
				EngineStopped = false;	-- engine was not stopped
				GotBlock = true;
				StopTime = 0;
				end
			--if (not GotBlock and StopDelay == 0) then
			if (not GotBlock and Target_Distance <= 50) then	-- Engine was stopped
				-- Caboose may not have passed over detector yet and it may occupy two blocks
				-- wait for all blocks to be locked (default wait 10 minutes)
				if (not MultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then	-- set occupancy flag in all blocks						
					return Stop("MainLine2MainLine : Timed out waiting for " .. BlockName[NextBlockNo]);
					end
				GotBlock = true;	-- indicate Block is locked
				-- start moving if stopped
				if (EngineStopped) then
					local Aspect = (GetCabAspect(HOMESIGNAL));
					print ("--> Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
					-- start moving if stopped - TODO check that home signal is not AsStopSignal (but it can be AsStopandProceed)
					if (Aspect == AsStopSignal) then
						-- we are stopped, so wait until the Stop aspect is cleared.
						print("--> Home Signal aspect is Stop Signal. Wait for it to clear before proceeding.");
						end;
					local UsedUpTime = RunTime() - StopTime;	-- never negative
					local StartDelay;
					if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
						StartDelay = 0;
						else
						StartDelay = StopDelay - UsedUpTime + 5;	--UsedUpTime always <= StopDelay
						end;				
					BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
					SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
					SetCounter(C3, BlockSpeedLimit);
					print(string.format("MainLine2MainLine : %s : Speed %d Smph = Start Delay %.3f seconds",
						BlockName[CurrentBlockNo], BlockSpeedLimit, StartDelay));
					EngineStopped = false;	-- indicate that we handled the stop here
					end;
				end;
			--[[	
			if (not GotBlock and Target_Distance > 50) then
				-- defer wait to Caboose detection code			
				end;
			--]]	
			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			-- in the Caboose section of code, use MyCurrentBlockNo, MyLastBlockNo, and MyNextBlockNo
			print("MainLine2MainLine : " .. DetectorID .. "Caboose detected");
			-- on any MainLine exit readers, the switch is before the tag reader so 
			-- the SwitchClearingDistance is zero
			--
			-- ASAP Once the caboose is detected (without any clearing delay) return switch at exit of passing track back
			-- to NORMAL - this switch was set as result of the engine entering that last block
			--
			if (ThisSwitch ~= nil) then
				Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
				print("MainLine2MainLine : This Switch " .. SwitchName[ThisSwitch] .. " to Normal");
				end;
				
			if (MyLastBlockNo ~= 0) then
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy		
				end;
			if (not GotBlock) then
				-- wait for all blocks to be locked (default wait 10 minutes)
				if (not MultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then	-- set occupancy flag in all blocks			
					return Stop("MainLine2MainLine : Timed out waiting for " .. BlockName[MyNextBlockNo]);
					end;
				end;
			if (EngineStopped) then
				local Aspect = (GetCabAspect(HOMESIGNAL));
				print ("--> Home Signal in ", BlockName[MyCurrentBlockNo], " : " .. AspectNames[Aspect]);
				-- start moving if stopped - TODO check that home signal is not AsStopSignal (but it can be AsStopandProceed)
				if (Aspect == AsStopSignal) then
					-- we are stopped, so wait until the Stop aspect is cleared.
					print("--> Home Signal aspect is Stop Signal. Wait for it to clear before proceeding.");
					end;
				local UsedUpTime = RunTime() - StopTime;	-- never negative
				local StartDelay;
				if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
					StartDelay = 0;
					else
					StartDelay = StopDelay - UsedUpTime + 5;	--UsedUpTime always <= StopDelay
					end;				
				BlockSpeedLimit = BlockSpeed[MyDirection][MyCurrentBlockNo];
				SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
				SetCounter(C3, BlockSpeedLimit);
				print(string.format("MainLine2MainLine : %s : Speed %d Smph = Start Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], BlockSpeedLimit, StartDelay));
				EngineStopped = false;	-- indicate that we handled the stop here
				end;
			WaitforCaboose = false;
			--print("MainLine2MainLine : " .. DetectorID .. " " .. BlockName[MyLastBlockNo] .. " > " .. BlockName[MyCurrentBlockNo] .. " > " .. BlockName[MyNextBlockNo]);
			end;
	return true;
	end;

return MainLine2MainLine;
end;
--[[---------------------------------------------------------------------------------------]]
function createPassing2MainLine(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance or 0;
	local EngineStopped = false;
	local StopDelay = 0;
	local GotBlock = false;
	local StopTime = 0;
	local Target_Distance = 0;
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;	
	local function Passing2MainLine(DetectorID, EngineNo, TagLocation, CarModel, Normal_Track, Reverse_Track, ThisSwitch, NextSwitch, Block)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		if (type(Block) ~= "table") then return Stop("Passing2MainLine : Block not a table"); end
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			print("Passing2MainLine : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			--print("Passing2MainLine : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			--
			RegisterCabSignals(NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL], NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]);
			local Aspect = (GetCabAspect(HOMESIGNAL));
			print ("--> Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
			-- TODO check if home signal is AsStopSignal or AsStopandProceed. If it is, stop in this block. Set EngineSTopped = true.
			-- Then go on to check TryMultiSetOccupancy and if we get it, start moving again.
			if (Aspect == AsStopSignal or Aspect == AsStopandProceed) then
				-- stop in this block
				print("--> Home Signal aspect is Stop Signal or Stop and Proceed. Stop the engine");
				end;
			--				
			-- Adjust speed for the current block
			BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
			SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
			SetCounter(C3, BlockSpeedLimit);
			print(string.format("%s : Speed %d Smph", BlockName[CurrentBlockNo], BlockSpeedLimit));
			--
			-- ASAP, set the switch at entrance to passing track depending on block selected by MainLine2Passing
			--
			if (ThisSwitch ~= nil) then
				if (CurrentBlockNo == Normal_Track) then
					Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
					print("Passing2MainLine : This Switch " .. SwitchName[ThisSwitch] .. " to Normal");
					else
					Switch(REVERSE, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
					print("Passing2MainLine : This Switch " .. SwitchName[ThisSwitch] .. " to Reverse");
					end;
				end;
				
			NextBlockNo = Block[1];	-- set NextBlockNo to Block[1]
			DisplayBlockNames("Passing2MainLine : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
			-- moving into a block requires the block to be empty (== 0) or
			-- be already reserved for this engine (== MyEngineNo)
			if (not TryMultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then
				-- if we can't get a lock, stop in CurrentBlockNo
				StopDelay, Target_Distance = CalculateBlockStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, BlockSpeedLimit)
				-- Stop delay may be zero for immediate stop
				if (StopDelay == 0) then
					Rate(0, 8, 0, MyEngineNo, MyTIUNo);	-- stop as quickly as possible
					end;				
				SetSpeed(0, StopDelay, MyEngineNo, MyTIUNo);	-- stop engine
				if (StopDelay == 0) then
					Rate(0, 1, 0, MyEngineNo, MyTIUNo);
					end;
				EngineStopped = true;	-- engine was stopped
				GotBlock = false;
				StopTime = RunTime();
				else
				-- got a lock on the needed block(s), continue on
				StopDelay = 0;
				EngineStopped = false;	-- engine was not stopped
				GotBlock = true;
				StopTime = 0;
				end

			--if (not GotBlock and StopDelay == 0) then
			if (not GotBlock and Target_Distance <= 50) then
				-- Engine was stopped
				-- Caboose may not have passed over detector yet and it may occupy two blocks
				-- it may occupy two blocks
				-- wait for all blocks to be locked (default wait 10 minutes)
				if (not MultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then	-- set occupancy flag in all blocks			
					return Stop("Passing2MainLine : Timed out waiting for " .. BlockName[NextBlockNo]);
					end
				GotBlock = true;	-- indicate Block is locked	
				if (EngineStopped) then
					local Aspect = (GetCabAspect(HOMESIGNAL));
					print ("--> Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
					-- start moving if stopped - TODO check that home signal is not AsStopSignal (but it can be AsStopandProceed)
					if (Aspect == AsStopSignal) then
						-- we are stopped, so wait until the Stop aspect is cleared.
						print("--> Home Signal aspect is Stop Signal. Wait for it to clear before proceeding.");
						end;
					local UsedUpTime = RunTime() - StopTime;	-- never negative
					local StartDelay;
					if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
						StartDelay = 0;
						else
						StartDelay = StopDelay - UsedUpTime + 5;	--UsedUpTime always <= StopDelay
						end;
					BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
					SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
					SetCounter(C3, BlockSpeedLimit);	
					print(string.format("Passing2MainLine : %s : Speed %d Smph = Start Delay %.3f seconds",
						BlockName[CurrentBlockNo], BlockSpeedLimit, StartDelay));
					EngineStopped = false;	-- indicate that we handled the stop here	
					end;
				end;
			--[[	
			if (not GotBlock and Target_Distance > 50) then
				-- defer wait to Caboose detection code			
				end;
			--]]					
			if (GotBlock) then
				-- set NextSwitch switch
				if (CurrentBlockNo == Normal_Track) then
					Switch(NORMAL, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
					print("Passing2MainLine : Next Switch " .. SwitchName[NextSwitch] .. " to Normal");
					end;
				if (CurrentBlockNo == Reverse_Track) then
					Switch(REVERSE, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
					print("Passing2MainLine : NextSwitch " .. SwitchName[NextSwitch] .. " to Reverse");
					end;
				end;

			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;			
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			-- in the Caboose section of code, use MyCurrentBlockNo, MyLastBlockNo, and MyNextBlockNo
			print("Passing2MainLine : " .. DetectorID .. "Caboose detected");

			if (SwitchClearingDistance > 0) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * BlockSpeedLimit)) * 3600;
				print(string.format("Passing2MainLine : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], SwitchClearingDistance, BlockSpeedLimit, SleepDelay));
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				print("Passing2MainLine : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				end;
			--
			-- Once we reach the switch clearing distance, return switch at entrance to passing track back
			-- to NORMAL - this switch was set as result of the engine entering the block
			--
			if (ThisSwitch ~= nil) then
				Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
				print("Passing2MainLine : This Switch " .. SwitchName[ThisSwitch] .. " to Normal");
				end;
			if (MyLastBlockNo ~= 0) then			
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy		
				end;
			if (not GotBlock) then
				-- wait for all blocks to be locked (default wait 10 minutes)
				if (not MultiSetOccupancy(Headway and {Block[1]} or Block, MyEngineNo)) then	-- set occupancy flag in all blocks			
					return Stop("Passing2MainLine : Timed out waiting for " .. BlockName[MyNextBlockNo]);
					end;
				GotBlock = true;	-- indicate Block is locked
				-- set next switch (passing track exit) based on current block
				if (NextSwitch ~= nil and GotBlock) then
					-- set NextSwitch switch
					if (MyCurrentBlockNo == Normal_Track) then
						Switch(NORMAL, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("Passing2MainLine : Next Switch " .. SwitchName[NextSwitch] .. " to Normal");
						end;
					if (MyCurrentBlockNo == Reverse_Track) then
						Switch(REVERSE, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
						print("Passing2MainLine : Next Switch " .. SwitchName[NextSwitch] .. " to Reverse");
						end;			
					end;
				end;

			if (EngineStopped) then
				local Aspect = (GetCabAspect(HOMESIGNAL));
				print ("--> Home Signal in ", BlockName[MyCurrentBlockNo], " : " .. AspectNames[Aspect]);
				-- start moving if stopped - TODO check that home signal is not AsStopSignal (but it can be AsStopandProceed)
				if (Aspect == AsStopSignal) then
					-- we are stopped, so wait until the Stop aspect is cleared.
					print("--> Home Signal aspect is Stop Signal. Wait for it to clear before proceeding.");
					end;
				local UsedUpTime = RunTime() - StopTime;	-- never negative
				local StartDelay;
				if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
					StartDelay = 0;
					else
					StartDelay = StopDelay - UsedUpTime + 5;	--UsedUpTime always <= StopDelay
					end;			
				BlockSpeedLimit = BlockSpeed[MyDirection][MyCurrentBlockNo];
				SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
				SetCounter(C3, BlockSpeedLimit);
				print(string.format("Passing2MainLine : %s : Speed %d Smph = Start Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], BlockSpeedLimit, StartDelay));
				EngineStopped = false;	-- indicate that we handled the stop here
				end;

			WaitforCaboose = false;				
			--print("Passing2MainLine : " .. DetectorID .. " " .. BlockName[MyLastBlockNo] .. " > " .. BlockName[MyCurrentBlockNo] .. " > " .. BlockName[MyNextBlockNo]);
			end;
	return true;
	end;

return Passing2MainLine;
end;
--[[---------------------------------------------------------------------------------------]]

function createMainLine2Yard(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance or 24;		
	local EngineStopped = false;
	local StopDelay = 0;
	local GotBlock = false;
	local StopTime = 0;
	local Target_Distance;
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;
	local function MainLine2Yard(DetectorID, EngineNo, TagLocation, CarModel, Yard1, Yard2, ThisSwitch, Block)
		if (type(Block) ~= "table") then return Stop("MainLine2Yard : Block not a table"); end
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			print("MainLine2Yard : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			--print("MainLine2Yard : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			--
			RegisterCabSignals(NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL], NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]);
			local Aspect = (GetCabAspect(HOMESIGNAL));
			print ("--> Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
			-- TODO check if home signal is AsStopSignal or AsStopandProceed. If it is, stop in this block. Set EngineSTopped = true.
			-- Then go on to check TrySetOccupancy and if we get it, start moving again.
			if (Aspect == AsStopSignal or Aspect == AsStopandProceed) then
				-- stop in this block
				print("--> Home Signal aspect is Stop Signal or Stop and Proceed. Stop the engine");
				end;
			--	
			-- Adjust speed for the current block
			BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
			SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);
			SetCounter(C3, BlockSpeedLimit);
			print(string.format("%s : Speed %d Smph",
				BlockName[CurrentBlockNo], BlockSpeedLimit));	
				
			NextBlockNo = Block[1];	-- ending block number
			-- check for occupancy clear Yard2 or Yard1 - Ending block must be unoccupied otherwise stop engine
			if (TrySetOccupancy(NextBlockNo, MyEngineNo)) then
				StopDelay = 0;
				EngineStopped = false;
				GotBlock = true;
				StopTime = 0;
				else
				-- if we can't get a lock, stop in CurrentBlockNo
				StopDelay, Target_Distance = CalculateBlockStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, BlockSpeedLimit)
				-- Stop delay
				if (StopDelay == 0) then
					Rate(0, 8, 0, MyEngineNo, MyTIUNo);	-- stop as quickly as possible
					end;
				SetSpeed(0, StopDelay, MyEngineNo, MyTIUNo);	-- stop engine
				if (StopDelay == 0) then
					Rate(0, 1, 0, MyEngineNo, MyTIUNo);
					end;
				EngineStopped = true;
				GotBlock = false;
				StopTime = RunTime();
				end
			if (GotBlock) then	
				DisplayBlockNames("MainLine2Yard : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
				end;
				
			-- engine was stopped quickly
			--if (not GotBlock and StopDelay == 0) then
			if (not GotBlock and Target_Distance <= 50) then
				-- Caboose may not have passed over detector yet and it may occupy two blocks
				-- wait for all blocks to be locked (default wait 10 minutes)
				-- wait for either Normal_Track or Reverse_Track to clear
				SetOccupancy(NextBlockNo, MyEngineNo);	-- set occupancy flag in NextBlockNo
				DisplayBlockNames("MainLine2Yard : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
				GotBlock = true;	-- indicate Block is locked
				if (EngineStopped) then
					local Aspect = (GetCabAspect(HOMESIGNAL));
					print ("--> Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[Aspect]);
					-- start moving if stopped - TODO check that home signal is not AsStopSignal (but it can be AsStopandProceed)
					if (Aspect == AsStopSignal) then
						-- we are stopped, so wait until the Stop aspect is cleared.
						print("--> Home Signal aspect is Stop Signal. Wait for it to clear before proceeding.");
						end;
					local UsedUpTime = RunTime() - StopTime;	-- never negative
					local StartDelay;
					if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
						StartDelay = 0;
						else
						StartDelay = StopDelay - UsedUpTime + 5;	--UsedUpTime always <= StopDelay
						end;
					BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
					SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
					SetCounter(C3, BlockSpeedLimit);
					print(string.format("MainLine2Yard : %s : Speed %d Smph = Start Delay %.3f seconds",
						BlockName[CurrentBlockNo], BlockSpeedLimit, StartDelay));
					EngineStopped = false;	-- indicate that we handled the stop here
					end;
				end;
					
			--[[		
			if (not GotBlock and Target_Distance > 50) then
				-- defer wait to Caboose detection code			
				end;
			--]]
			if (GotBlock) then	
				-- set ThisSwitch switch
				if (NextBlockNo == Yard1) then
					Switch(NORMAL, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
					print("MainLine2Yard : Switch " .. SwitchName[ThisSwitch] .. " to Normal");
					end;
				if (NextBlockNo == Yard2) then
					Switch(REVERSE, Switches[ThisSwitch][CHAN_NUMBER], Switches[ThisSwitch][AIU_NUMBER], Switches[ThisSwitch][TIU_NUMBER], 0);
					print("MainLine2Yard : Switch " .. SwitchName[ThisSwitch] .. " to Reverse");
					end;
				end;
			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;			
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			-- in the Caboose section of code, use MyCurrentBlockNo, MyLastBlockNo, and MyNextBlockNo
			print("MainLine2Yard : " .. DetectorID .. "Caboose detected");
			--print("MainLine2Yard : " .. DetectorID .. "Caboose - Last Block = " .. BlockName[MyLastBlockNo]);
			if (SwitchClearingDistance > 0) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * BlockSpeedLimit)) * 3600;
				print(string.format("MainLine2Yard : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], SwitchClearingDistance, BlockSpeedLimit, SleepDelay));
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				print("MainLine2Yard : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				end;
			if (MyLastBlockNo ~= 0) then
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy
				end;
			if (not GotBlock) then
				-- wait for NextBlockNo to be locked (either Yard track)
				local result;
				-- set occupancy flag in NextBlockNo
				SetOccupancy(MyNextBlockNo, MyEngineNo);	-- set occupancy flag in NextBlockNo
				DisplayBlockNames("MainLine2Yard : ", MyLastBlockNo, MyCurrentBlockNo, MyNextBlockNo);
				GotBlock = true;	-- indicate Block is locked
				-- set NextSwitch switch
				if (MyNextBlockNo == Yard1) then
					Switch(NORMAL, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
					print("MainLine2Yard : Switch " .. SwitchName[NextSwitch] .. " to Normal");
					end;
				if (MyNextBlockNo == Yard2) then
					Switch(REVERSE, Switches[NextSwitch][CHAN_NUMBER], Switches[NextSwitch][AIU_NUMBER], Switches[NextSwitch][TIU_NUMBER], 0);
					print("MainLine2Yard : Switch " .. SwitchName[NextSwitch] .. " to Reverse");
					end;
				end;
			if (EngineStopped) then
				local Aspect = (GetCabAspect(HOMESIGNAL));
				print ("--> Home Signal in ", BlockName[MyCurrentBlockNo], " : " .. AspectNames[Aspect]);
				-- start moving if stopped - TODO check that home signal is not AsStopSignal (but it can be AsStopandProceed)
				if (Aspect == AsStopSignal) then
					-- we are stopped, so wait until the Stop aspect is cleared.
					print("--> Home Signal aspect is Stop Signal. Wait for it to clear before proceeding.");
					end;
				local UsedUpTime = RunTime() - StopTime;	-- never negative
				local StartDelay;
				if (UsedUpTime > StopDelay) then	-- if StopDelay == 0 then always true
					StartDelay = 0;
					else
					StartDelay = StopDelay - UsedUpTime + 5;	--UsedUpTime always <= StopDelay
					end;
				BlockSpeedLimit = BlockSpeed[MyDirection][MyCurrentBlockNo];
				SetSpeed(BlockSpeedLimit, StartDelay, MyEngineNo, MyTIUNo);
				SetCounter(C3, BlockSpeedLimit);
				EngineStopped = false;	-- indicate that we handled the stop here
				print(string.format("MainLine2Yard : %s : Speed %d Smph = Start Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], BlockSpeedLimit, StartDelay));		
				end;
			WaitforCaboose = false;
			--print("MainLine2Yard : " .. DetectorID .. " " .. BlockName[MyLastBlockNo] .. " > " .. BlockName[MyCurrentBlockNo] .. " > " .. BlockName[MyNextBlockNo]);
			end;
	return true;
	end

return MainLine2Yard;
end;

--[[---------------------------------------------------------------------------------------]]
function createStopInYard(flag, SwDistance)
	local WaitforCaboose = flag;
	local SwitchClearingDistance = SwDistance or 24;		
	local EngineStopped = false;
	local StopDelay = 0;
	local StopTime = 0;	
	local Target_Distance;	
	local MyCurrentBlockNo, MyLastBlockNo, MyNextBlockNo;	
	local function StopInYard(DetectorID, EngineNo, TagLocation, CarModel,  OffLayoutBlock)
		-- uses globals: MyEngineNo, LastBlockNo, CurrentBlockNo, NextBlockNo
		if (type(OffLayoutBlock) ~= "table") then return Stop("StopInYard OffLayoutBlock not a table"); end
		if (EngineNo == MyEngineNo and ((TagLocation == TAG_ON_FRONT_TRUCK) or
			(not WaitforCaboose and (TagLocation == TAG_ON_REAR_TRUCK)))) then		-- if MyEngineNo in tag
			print("StopInYard : " .. DetectorID .. "Engine " .. MyEngineNo .. " detected");
			--print("StopInYard : " .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
			LastBlockNo = CurrentBlockNo;
			CurrentBlockNo = NextBlockNo;
			--
			NextBlockNo = OffLayoutBlock[1];	
			DisplayBlockNames("StopInYard : ", LastBlockNo, CurrentBlockNo, NextBlockNo);			
			-- schedule stop in CurrentBlockNo
			StopDelay, Target_Distance = CalculateBlockStopDelay(CurrentBlockNo, MyEngineNo, MyDirection, BlockSpeedLimit)
			-- Stop delay
			if (StopDelay == 0) then
				Rate(0, 8, 0, MyEngineNo, MyTIUNo);	-- stop as quickly as possible
				end;
			SetSpeed(0, StopDelay, MyEngineNo, MyTIUNo);	-- stop engine
			if (StopDelay == 0) then
				Rate(0, 1, 0, MyEngineNo, MyTIUNo);
				end;
			EngineStopped = true;
			StopTime = RunTime();
			WaitforCaboose = true;
			-- save current state for the Caboose detection call of this function
			MyCurrentBlockNo = CurrentBlockNo;
			MyLastBlockNo = LastBlockNo;
			MyNextBlockNo = NextBlockNo;			
			return true;

		elseif (EngineNo == 0 and CarModel == CABOOSE and WaitforCaboose) then
			print("StopInYard : " .. DetectorID .. "Caboose detected");
			if (SwitchClearingDistance > 0) then
				-- equation for Delay calculation
				-- time(seconds) = d(in) * 48 Smiles/mile * 3600 sec/hr * / (12 in/ft * 5280 ft/mile * X Smiles/hr)
				local SleepDelay =  (SwitchClearingDistance) * (48 / (12 * 5280 * BlockSpeedLimit)) * 3600;
				print(string.format("StopInYard : %s : Switch Clearing Distance = %.3f in - Speed %d Smph = Sleep Delay %.3f seconds",
					BlockName[MyCurrentBlockNo], SwitchClearingDistance, BlockSpeedLimit, SleepDelay));
				Sleep(SleepDelay);	-- give the caboose time to clear the switch
				print("StopInYard : " .. "Switch cleared after " .. SleepDelay .. " seconds")
				end;
			if (MyLastBlockNo ~= 0) then
				ClearOccupancy(MyLastBlockNo, MyEngineNo);	-- clear LastBlockNo occupancy
				end;
			if (EngineStopped) then
				print("StopInYard : " .. DetectorID .. "Stopped!");
				RFIDFlag = false;
				end
			WaitforCaboose = false;
			--print("StopInYard : " .. DetectorID .. " " .. BlockName[MyLastBlockNo] .. " > " .. BlockName[MyCurrentBlockNo] .. " > " .. BlockName[MyNextBlockNo]);
			end;	
	return true;
	end;

return StopInYard;
end;

--[[---------------------------------------------------------------------------------------]]

function setup(Engine, TIU)
print("setup() Running");

MASTER_TIU = TIU;
-- Instrumented Layout 4
Layout4 = require([[Layout4]]);		-- All of the details about the Layout
-- an undirected weighted double vertex graph
MyTIUNo = MASTER_TIU;
Title(title, MyEngineNo);

for iEng = 1, 99 do	-- check all possible engines on one TIU
	if (Exists(iEng)) then
		local Name, PS;
		Name, PS = GetName(iEng);
		print(iEng .. " " .. Name .. " PS" .. PS);
		end
	end
--
ShowLayout();
--	
Enum = tonumber(InputBox(title, "Enter Engine Number", MyEngineNo));
if (Enum ~= nil) then	-- if the result is a number, then use it, otherwise, ignore it
	MyEngineNo = Enum;
	end
print("Engine Number = " .. MyEngineNo);
-- Show new title with Engine # and Name and put Engine # into Engine Number Spinner
local Name = GetName(MyEngineNo);
Title(title .. "#" .. MyEngineNo .. " " .. Name, MyEngineNo);	
if (Layout4.StoppingisEmpty(MyEngineNo,1) == nil) then
	return Stop("No StoppingDistance Table for Engine " .. MyEngineNo);
	end	
--print("StationName = " .. Pretty(StationName));
--print("StationName = " .. Pretty(BlockName));
--print("Switches = " ..  Pretty(Switches));
--print("NextBlock = " ..  Pretty(NextBlock));
--
-- start of run
---------
-- Eastbound train
MyDirection = EB;
-- must ask the dispatcher
print("Select Starting Block Number");
for k, v in pairs(BlockName) do
	if (k <= 100) then
		print(v .. "(" .. k .. ")");
		end;
	end;
--
-- read Starting Block Number
--
local response = InputQuery(title, "Enter Starting Block Number (0 for none)", ALIQUIPPA_BLOCK);
if (response == false) then
	return Stop("Canceled");
	end;
local Enum = tonumber(response);
if (Enum ~= nil and Enum ~= 0) then	-- if the result is a block number, then use it, otherwise, ignore it
	StartBlockNo = Enum;
	print("Starting Block = " .. BlockName[Enum]);
	end

CurrentBlockNo = StartBlockNo;
EndBlockNo = MCKEES_ROCKS_YARD1_BLOCK;	-- EndBlockNo is not used because this sketch loops forever
--
-- create and register the cab signals
	print("Creating Cab Signals");
	CreateCab(Hd3Light, Hd3Light);	-- create light signals - (home, distant) -- create as 3 head signals so that they accept all possible aspects
	print("Registering the Cab Signals #", NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL], " and #", NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]);
	RegisterCabSignals(NextSignal[MyDirection][CurrentBlockNo][HOMESIGNAL], NextSignal[MyDirection][CurrentBlockNo][DISTANTSIGNAL]);
									-- associate the Home cab signal with next signal and
									-- the Distant cab signal with second signal.
									-- Signals must be created by the "Layout4 Signal Master.lua"
									-- script which should be run in a different PC window
	print("Reflecting signals from Layout window into the Cab Display");
	print ("--> Home Signal in ", BlockName[CurrentBlockNo], " : " .. AspectNames[(GetCabAspect(HOMESIGNAL))]);
	
BlockSpeedLimit = BlockSpeed[MyDirection][StartBlockNo];
SetCounter(C3, BlockSpeedLimit);
--[[---------------------]]
if (MyDirection == EB) then
	print("Eastbound");
	else
	print("Westbound");
	end
-- read up current sound volume
EngineSoundSave = GetVolume(MASTER_VOLUME, MyEngineNo, MyTIUNo);	-- get Volume
--
OverrideOccupancy(StartBlockNo, MyEngineNo);	-- force set occupancy flag
--print(CurrentBlockNo .. " - " .. StartBlockNo);
return true;		-- false=setup failed, true=setup succeeded
end
--[[---------------------------------------------------------------------------------------]]

function DisplayBlockNames(Title, LastBlockNo, CurrentBlockNo, NextBlockNo)
print(Title .. BlockName[LastBlockNo] .. " > " .. BlockName[CurrentBlockNo] .. " > " .. BlockName[NextBlockNo]);
end;
--[[---------------------------------------------------------------------------------------]]
function tag(Detector, Reader, EngineNo, TagLocation, CarModel, Railroad, CarNumber, TagPacket)
-- This function is called each time a tag programmed with 32 bytes of data is detected (TAGDATABLOCK4)
-- TagPacket contains 8 characters (4 hex digits) of UID and 32 characters
-- of block 4 data from the programmed tag.

BumpCounter(COUNTER01);	
local PacketLen = #TagPacket;

if (Debug() >= 8) then print("Packet Length = ", PacketLen); end;
if (Debug() >= 7) then print(TagPacket); end;

-- index the table Tags[] by tag ID, value is the tag packet itself
Tags[string.sub(TagPacket, 1, 8)] = string.sub(TagPacket, 9, 40);	-- remove checksum and EOP
TagCount = TagCount + 1;	-- count the number of tags
if (EngineNo > 0) then	-- it's an engine
	if (EngineNo == MyEngineNo) then
		if (BeepOn) then Beep(ASTERISK); end;
		-- next line shows a different way to quote a string using brackets
		printc(clGreen, string.format([[tag(%.2f) : Detector %d Reader %d Eng#%d %s %s #%s %s]],
			RunTime(), Detector, Reader, EngineNo, GetRailroadName(Railroad), GetEngineName(CarModel), CarNumber, GetTagLocation(TagLocation)));
		else
		-- ignore engines that are not me
		return true;	-- true=continue to process tags, false=stop processing tags
		end;
	-- follow on to process the detected engine
	else	-- it's a car
	if (CarModel == CABOOSE) then
		if (BeepOn) then Beep(EXCLAMATION); end;
		printc(clBlue, string.format([[tag(%.2f) : Detector %d Reader %d Car %s %s #%s]],
			RunTime(), Detector, Reader, GetRailroadName(Railroad), GetCarName(CarModel), CarNumber));
		else
		-- ignore all non-Caboose cars
		return true;	-- true=continue to process tags, false=stop processing tags
		end;		
	-- follow on to process non-engine tags
	end

-- The complete packet received is in TagPacket 
-- (all of the fields have already been extracted into the first parameters to tag()
--   First 8 digits are the 4 byte Tag UID in hexidecimal
--   Next 32 digits are the 16 bytes of Block 4 Information read from the Tag in hexidecimal
if (Debug() >= 6) then print(string.format("Packet  %s  %s", string.sub(TagPacket, 1, 8),
	string.sub(TagPacket,9,40) )); end
--[[--------------------------]]
-- Westbound trains are superior
-- if MyEngineNo is not in tag and Caboose number is not in tag, then ignore it

if (ResetinProgress) then
	-- run back to Tag 5-0
	if (Detector == 5 and Reader == 0) then
		local DetectorID = "@ 5-0 ";
		local StopDelay = 0;
		if (EngineNo == 0 and CarModel == CABOOSE) then		-- going in reverse, execute on caboose detection
			print(DetectorID .. "Caboose detected");
			-- schedule stop in StartBlockNo
			StopDelay = CalculateBlockStopDelay(StartBlockNo, MyEngineNo, WB, BlockSpeedLimit)
			SetSpeed(0, StopDelay, MyEngineNo, MyTIUNo);	-- Stop delay
			SetDirection(FORWARD, StopDelay + 5, MyEngineNo, MyTIUNo);
			ResetinProgress = false;
			end;
		end;
	return true;		

elseif (not RFIDFlag) then	-- ignore tags unless engine is moving
	return true;
	
-- Tag 5-0
elseif (Detector == 5 and Reader == 0) then
	if (Func50 == nil) then
		Func50 = createMainLine2Passing(false);
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock
	Func50("@ 5-0 ", EngineNo, TagLocation, CarModel, STRUTHERS_THRU_TRACK, STRUTHERS_STATION_TRACK, nil, nil);

-- Tag 4-0
elseif (Detector == 4 and Reader == 0) then
	if (Func40 == nil) then
		Func40 = createPassing2MainLine(false, SwitchClearingInches[40][MyDirection]);
		end;
	-- status: in block NextBlockNo, occupancy already set. Set STRUTHERS_WEST to NORMAL on Caboose detection.
	Func40("@ 4-0 ", EngineNo, TagLocation, CarModel, STRUTHERS_THRU_TRACK, STRUTHERS_STATION_TRACK, STRUTHERS_WEST, STRUTHERS_EAST, 
		{NEW_CASTLE_BLOCK, COLLEGE_STATION_TRACK});

-- Tag 2-0
elseif (Detector == 2 and Reader == 0) then
	if (Func20 == nil) then
		Func20 = createMainLine2Passing(false);
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock	
	Func20("@ 2-0 ", EngineNo, TagLocation, CarModel, COLLEGE_THRU_TRACK, COLLEGE_STATION_TRACK, STRUTHERS_EAST, nil);

-- Tag 2-1
elseif (Detector == 2 and Reader == 1) then
	if (Func21 == nil) then
		Func21 = createPassing2MainLine(false, SwitchClearingInches[21][MyDirection]);
		end;
	-- status: in block NextBlockNo, occupancy already set. Set COLLEGE_WEST to NORMAL on Caboose detection.
	Func21("@ 2-1 ", EngineNo, TagLocation, CarModel, COLLEGE_THRU_TRACK, COLLEGE_STATION_TRACK, COLLEGE_WEST, COLLEGE_EAST, 
		{BEAVER_FALLS_BLOCK, MONACA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK});

-- Tag 4-1
elseif (Detector == 4 and Reader == 1) then
	if (Func41 == nil) then
		Func41 = createMainLine2MainLine(false); 
		end;
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock	.  Set COLLEGE_EAST to NORMAL on Caboose detection.
	Func41("@ 4-1 ", EngineNo, TagLocation, CarModel, COLLEGE_EAST,
		{MONACA_BLOCK, ALIQUIPPA_BLOCK,	J_L_PASSING_TRACK});

-- Tag 6-1
elseif (Detector == 6 and Reader == 1) then
	-- entering tunnel, reduce volume
	if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK and SoundOff == false) then
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 5/6), 0, MyEngineNo, MyTIUNo);
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 4/6), 1, MyEngineNo, MyTIUNo);
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 1/2), 2, MyEngineNo, MyTIUNo);
		end;
	if (Func61 == nil) then
		Func61 = createMainLine2MainLine(false);
		end;
	Func61("@ 6-1 ", EngineNo, TagLocation, CarModel, nil,
		{ALIQUIPPA_BLOCK, J_L_PASSING_TRACK});
	
-- Tag 6-0
elseif (Detector == 6 and Reader == 0) then
	-- leaving tunnel, increase volume
	if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK and SoundOff == false) then
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 4/6), 0, MyEngineNo, MyTIUNo);
		SetVolume(MASTER_VOLUME, math.floor(EngineSoundSave * 5/6), 1, MyEngineNo, MyTIUNo);
		SetVolume(MASTER_VOLUME, EngineSoundSave, 2, MyEngineNo, MyTIUNo);
		end;
	if (Func60 == nil) then
		Func60 = createMainLine2Passing(false);
		end;
	Func60("@ 6-0 ", EngineNo, TagLocation, CarModel, J_L_THRU_TRACK, J_L_PASSING_TRACK, nil, nil);
	
-- Tag 1-0
elseif (Detector == 1 and Reader == 0) then
	if (LLLoop == false) then
		if (Func10 == nil) then
			Func10 = createPassing2MainLine(false, SwitchClearingInches[10][MyDirection]);
			end;
		Func10("@ 1-0 ", EngineNo, TagLocation, CarModel, J_L_THRU_TRACK, J_L_PASSING_TRACK, ALIQUIPPA_WEST, ALIQUIPPA_EAST,
			{MCKEES_ROCKS_BLOCK, EndBlockNo});
		else
		if (Func10 == nil) then
			Func10 = createPassing2MainLine(false, SwitchClearingInches[10][MyDirection]);
			end;
		-- status: in block NextBlockNo, occupancy already set. Set ALIQUIPPA_WEST to NORMAL on Caboose detection.
		Func10("@ 1-0 ", EngineNo, TagLocation, CarModel, J_L_THRU_TRACK, J_L_PASSING_TRACK, ALIQUIPPA_WEST, ALIQUIPPA_EAST,
			{MCKEES_ROCKS_BLOCK, LIONEL_IVES_BLOCK, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK});
		end;

-- Tag 5-1
elseif (Detector == 5 and Reader == 1) then
	-- status: just entered NextBlockNo, occupancy already set
	-- engine never stops in this block except for deadlock	
	if (LLLoop == false) then
		if (Func51 == nil) then
			Func51 = createMainLine2Yard(false, SwitchClearingInches[51][MyDirection]);
			end;
		Func51("@ 5-1 ", EngineNo, TagLocation, CarModel, MCKEES_ROCKS_YARD1_BLOCK, MCKEES_ROCKS_YARD2_BLOCK, MCKEES_ROCKS_YARD,
			{EndBlockNo});
		else
		if (Func51 == nil) then
			Func51 = createMainLine2MainLine(false);
			end;
		if (EngineNo == MyEngineNo and TagLocation == TAG_ON_FRONT_TRUCK) then
			Switch(REVERSE, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0);
			end;
		-- engine never stops in this block except for deadlock.  Set ALIQUIPPA_EAST to NORMAL on Caboose detection.
		Func51("@ 5-1 ", EngineNo, TagLocation, CarModel, ALIQUIPPA_EAST,
			{LIONEL_IVES_BLOCK, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK});
		end;

-- Tag 3-0
elseif (Detector == 3 and Reader == 0) then
	-- status: in block NextBlockNo, occupancy already set
	if (LLLoop == false) then
		if (Func30 == nil) then
			Func30 = createStopInYard(false, SwitchClearingInches[30][MyDirection]);	-- clear yard switch
			end;
		Func30("@ 3-0 ", EngineNo, TagLocation, CarModel, {POINTS_SOUTH_N_W});
		else	
		if (Func30 == nil) then
			Func30 = createMainLine2MainLine(false);	-- no switches if staying on Mainline
			end;
		-- engine never stops in this block except for deadlock
		Func30("@ 3-0 ", EngineNo, TagLocation, CarModel, nil,
			{YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK});
		end;
	end;

return true;	-- true=continue to process tags, false=stop processing tags
end	
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
-- this function is called once when the user presses the [STOP] button or the Stop() function is called
print("All Newly Detected Tags:");
local k, v;
for k,v in  pairs(Tags) do		-- dump out the Tags table
	--                           . . . . . . . . . . . . . . . .
	-- print format:  6619044A  88080101030108011B02202031353536
	print(k .. "  " .. v);
	end
SetSpeed(0, 0, MyEngineNo, MyTIUNo);
Rate(1, 1, 2, MyEngineNo, MyTIUNo);
print("Shutdown in Progress.....");
--
if (StartedFlag) then
	print("Shutting down the Engine");
	Sleep(10);
	EngineShutdown(MyEngineNo, MyTIUNo);
	StartedFlag = false;
	end
RFIDFlag = false;
ResetinProgress = false;
print(string.format("cleanup() Complete at %.2f", RunTime()));
return true;		-- false=cleanup failed, true=cleanup succeeded
end
--[[---------------------------------------------------------------------------------------]]
function DTOButton(Eng, TIU)
print(string.format("Engine %d Trip Odometer %.3f Smiles", Eng, DTO(Eng, MyTIUNo)));
return true;
end
Function01Name, Function01Label = DTOButton, "Show DTO";
--[[---------------------------------------------------------------------------------------]]
function DODButton(Eng, TIU)
print(string.format("Engine %d Odometer %.3f Smiles", Eng, DOD(Eng, MyTIUNo)));
return true;
end
Function02Name, Function02Label = DODButton, "Show DOD";
--[[---------------------------------------------------------------------------------------]]
function ClearButton(Eng, TIU)
print("Clear Occupancy Flags");
-- clear occupancy flags
local x;
for x = FIRST_BLOCK, LAST_BLOCK do
	OverrideOccupancy(x, 0);
	end;
print("Occupancy Flags cleared");
return true;
end
Function03Name, Function03Label = ClearButton, "Clear Flags";
--[[---------------------------------------------------------------------------------------]]
function HeadwayButton(Eng, TIU)
print("Toggle Headway");
-- toggle Headway flag
Headway =  not Headway;
print("Headway set to " .. tostring(Headway));
return true;
end
Function04Name, Function04Label = HeadwayButton, "Headway";
--[[---------------------------------------------------------------------------------------]]
function SoundOffButton(Eng, TIU)
print("Sound Off");
if (SoundOff == false) then
	-- read up current sound volume
	EngineSoundSave = GetVolume(MASTER_VOLUME, MyEngineNo, MyTIUNo);	-- get Volume
	print(" Sound Off - Engine " .. MyEngineNo .. " Level was " .. EngineSoundSave);
	SoundOff = true;
	SetVolume(MASTER_VOLUME, 0, 3, MyEngineNo, MyTIUNo);
	end;
return true;
end
Function19Name, Function19Label = SoundOffButton, "Sound Off";
--[[---------------------------------------------------------------------------------------]]
function SoundOnButton(Eng, TIU)
print("Sound On");
if (SoundOff == true) then
	print(" Sound On - Engine " .. MyEngineNo .. " Level restored to " .. EngineSoundSave);
	SoundOff = false;
	SetVolume(MASTER_VOLUME, EngineSoundSave, 3, MyEngineNo, MyTIUNo);
	end;
return true;
end
Function20Name, Function20Label = SoundOnButton, "Sound On";
--[[---------------------------------------------------------------------------------------]]
function InitButton(Eng, TIU)
print("Initialize");
print("Set all Switches to Normal postion");
local k, v;
-- set all switches to NORMAL position
for k,v in pairs(Switches) do
	print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
	Switch(NORMAL, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
	Sleep(0.5);
	end
if (Debug() > 3) then
	-- set all switches to REVERSE
	for k,v in pairs(Switches) do
		print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
		Switch(REVERSE, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
		Sleep(0.5);
		end;
	-- set all switches to NORMAL
	for k,v in pairs(Switches) do
		print("k = " .. k .. " TIU = " .. v[TIU_NUMBER] .. " AIU = " .. v[AIU_NUMBER] .. " Channel = " .. v[CHAN_NUMBER]);
		Switch(NORMAL, v[CHAN_NUMBER], v[AIU_NUMBER], v[TIU_NUMBER], 0);
		Sleep(0.5);
		end;
	end;
-- clear occupancy flags
local x;
for x = FIRST_BLOCK, LAST_BLOCK do
	OverrideOccupancy(x, 0);
	end;
--[[-----------------------------]]
-- clear all tag handling functions
Func30 = nil;
Func51 = nil;
Func10 = nil;
Func41 = nil;
Func21 = nil;
Func20 = nil;
Func40 = nil;
Func50 = nil;
Func60 = nil;
Func61 = nil;

print("Initialization is Complete");
return true;
end
Function06Name, Function06Label = InitButton, "Initialize Layout";
--[[---------------------------------------------------------------------------------------]]
function StartButton(Eng, TIU)
print("Start the Run");

--[[-----------------------------]]
-- clear all tag handling functions
Func30 = nil;
Func51 = nil;
Func10 = nil;
Func41 = nil;
Func21 = nil;
Func20 = nil;
Func40 = nil;
Func50 = nil;
Func60 = nil;
Func61 = nil;

--[[------------------------------------------]]
--print("StationName = " .. Pretty(StationName));
--print("StationName = " .. Pretty(BlockName));
--print("Switches = " ..  Pretty(Switches));
--print("NextBlock = " ..  Pretty(NextBlock));

-- start the engine
Rate(1, 1, 0, MyEngineNo, MyTIUNo);
print("setup : Set accell/decell rates to 1");
if (StartedFlag == false) then
	EngineStartUp(MyEngineNo, MyTIUNo);
	print("Starting up the Engine");
	StartedFlag = true;
	Sleep(20);
	end
SetDirection(FORWARD, 0, MyEngineNo, MyTIUNo);
--
-- Engine can start in any block
--
if (CurrentBlockNo == YOUNGSTOWN_YARD1_BLOCK) then
	LastBlockNo = ASHTABULA_NYC;
	NextBlockNo = YOUNGSTOWN_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, STRUTHERS_STATION_TRACK}, MyEngineNo);
	-- position YOUNGSTOWN_YARD switch as needed depending on value of CurrentBlockNo
	Switch(NORMAL, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	-- position BALLOON switch as needed after locking next block
	Switch(NORMAL, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0);	
	end;
	
if (CurrentBlockNo == YOUNGSTOWN_YARD2_BLOCK) then
	LastBlockNo = CLEVELAND_EL;
	NextBlockNo = YOUNGSTOWN_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, STRUTHERS_STATION_TRACK}, MyEngineNo);		
	-- position YOUNGSTOWN_YARD switch as needed depending on value of CurrentBlockNo
	Switch(REVERSE, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	-- position BALLOON switch as needed after locking next block
	Switch(NORMAL, Switches[BALLOON][CHAN_NUMBER], Switches[BALLOON][AIU_NUMBER], Switches[BALLOON][TIU_NUMBER], 0);	
	end;
	
if (CurrentBlockNo == YOUNGSTOWN_BLOCK) then -- "Youngstown Block"
	LastBlockNo = YOUNGSTOWN_YARD1_BLOCK;
	NextBlockNo = STRUTHERS_STATION_TRACK;
	-- check that the Next Block is not occupied
	SingleSetOccupancy({NextBlockNo}, MyEngineNo);
	end;

if (CurrentBlockNo == STRUTHERS_STATION_TRACK) then -- "Struthers Station Track"
	LastBlockNo = YOUNGSTOWN_BLOCK;
	NextBlockNo = NEW_CASTLE_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, COLLEGE_STATION_TRACK}, MyEngineNo);
	-- Position STRUTHERS_EAST after locking next block
	Switch(REVERSE, Switches[STRUTHERS_EAST][CHAN_NUMBER], Switches[STRUTHERS_EAST][AIU_NUMBER], Switches[STRUTHERS_EAST][TIU_NUMBER], 0);	
	end;

if (CurrentBlockNo == STRUTHERS_THRU_TRACK) then -- "Struthers Thru Track"
	LastBlockNo = YOUNGSTOWN_BLOCK;
	NextBlockNo = NEW_CASTLE_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, COLLEGE_STATION_TRACK}, MyEngineNo);			
	-- Position STRUTHERS_EAST after locking next block
	Switch(NORMAL, Switches[STRUTHERS_EAST][CHAN_NUMBER], Switches[STRUTHERS_EAST][AIU_NUMBER], Switches[STRUTHERS_EAST][TIU_NUMBER], 0);	
	end;

if (CurrentBlockNo == NEW_CASTLE_BLOCK) then -- "New Castle Block"
	LastBlockNo = STRUTHERS_STATION_TRACK;
	NextBlockNo = COLLEGE_STATION_TRACK;
	-- check that the Next Block is not occupied
	SingleSetOccupancy({NextBlockNo}, MyEngineNo);				
	end;
		
if (CurrentBlockNo == COLLEGE_STATION_TRACK) then -- "College Station Track"
	LastBlockNo = NEW_CASTLE_BLOCK;
	NextBlockNo = BEAVER_FALLS_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, MONACA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK}, MyEngineNo);				
	-- Position COLLEGE_EAST after locking next block
	Switch(REVERSE, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0);
	end;
	
if (CurrentBlockNo == COLLEGE_THRU_TRACK) then -- "College Thru Track"
	LastBlockNo = NEW_CASTLE_BLOCK;
	NextBlockNo = BEAVER_FALLS_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, MONACA_BLOCK, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK}, MyEngineNo);				
	-- Position COLLEGE_EAST after locking next block
	Switch(NORMAL, Switches[COLLEGE_EAST][CHAN_NUMBER], Switches[COLLEGE_EAST][AIU_NUMBER], Switches[COLLEGE_EAST][TIU_NUMBER], 0);
	end;
	
if (CurrentBlockNo == BEAVER_FALLS_BLOCK) then -- "Beaver Falls Block"
	LastBlockNo = COLLEGE_THRU_TRACK;
	NextBlockNo = MONACA_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, ALIQUIPPA_BLOCK, J_L_PASSING_TRACK}, MyEngineNo);				
	end;
	
if (CurrentBlockNo == MONACA_BLOCK) then -- "Monaca Block"
	LastBlockNo = BEAVER_FALLS_BLOCK;
	NextBlockNo = ALIQUIPPA_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, J_L_PASSING_TRACK}, MyEngineNo);				
	end;
	
if (CurrentBlockNo == ALIQUIPPA_BLOCK) then -- "Aliquippa Block"
	LastBlockNo = MONACA_BLOCK;
	NextBlockNo = J_L_PASSING_TRACK;
	-- check that the Next Block is not occupied
	SingleSetOccupancy({NextBlockNo}, MyEngineNo);
	end;
	
if (CurrentBlockNo == J_L_PASSING_TRACK) then -- "J&L Passing Track"
	LastBlockNo = ALIQUIPPA_BLOCK;
	NextBlockNo = MCKEES_ROCKS_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, LIONEL_IVES_BLOCK, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK}, MyEngineNo);
	-- Position ALIQUIPPA_EAST after locking next block
	Switch(REVERSE, Switches[ALIQUIPPA_EAST][CHAN_NUMBER], Switches[ALIQUIPPA_EAST][AIU_NUMBER], Switches[ALIQUIPPA_EAST][TIU_NUMBER], 0);
	end;

if (CurrentBlockNo == J_L_THRU_TRACK) then -- "J&L Thru Track"
	LastBlockNo = ALIQUIPPA_BLOCK;
	NextBlockNo = MCKEES_ROCKS_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, LIONEL_IVES_BLOCK, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK}, MyEngineNo);				
	-- Position ALIQUIPPA_EAST after locking next block
	Switch(NORMAL, Switches[ALIQUIPPA_EAST][CHAN_NUMBER], Switches[ALIQUIPPA_EAST][AIU_NUMBER], Switches[ALIQUIPPA_EAST][TIU_NUMBER], 0);
	end;
	
if (CurrentBlockNo == MCKEES_ROCKS_BLOCK) then -- "McKees Rocks Block"
	LastBlockNo = J_L_PASSING_TRACK;
	NextBlockNo = LIONEL_IVES_BLOCK;
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, YOUNGSTOWN_BLOCK, STRUTHERS_STATION_TRACK}, MyEngineNo);				
	end;

if (CurrentBlockNo == LIONEL_IVES_BLOCK) then -- "Lionel-Ives Block" -- AKA Balloon Track
	LastBlockNo = MCKEES_ROCKS_BLOCK;
	NextBlockNo = YOUNGSTOWN_BLOCK;	-- check that the Next Block is not occupied
	MultiSetOccupancy(Headway and {NextBlockNo} or {NextBlockNo, STRUTHERS_STATION_TRACK}, MyEngineNo);				
	end;
--
DisplayBlockNames("Start : ", LastBlockNo, CurrentBlockNo, NextBlockNo);
--


RFIDFlag = true;
Whistle(ON, 20, MyEngineNo, MyTIUNo);
Whistle(OFF, 22, MyEngineNo, MyTIUNo);
print("Trip Odometer " .. DTO(MyEngineNo, MyTIUNo)); 	
Sleep(30);

SetSpeed(BlockSpeedLimit, 5, MyEngineNo, MyTIUNo);
SetCounter(C3, BlockSpeedLimit)
	
--[[----------------------------------------------------]]
print(string.format("RunTime = %.2f seconds", RunTime()));
-- TODO once both engines exit the yard, throw the Lionel-Ives Switch
return true;
end
Function07Name, Function07Label = StartButton, "Start Run";
--[[---------------------------------------------------------------------------------------]]

function event(FromPC, Type, P1, P2, P3)
print("event() Running - From PC# ", FromPC, " EventType = ", Type, " Parameters = ", P1, " ", P2, " ", P3);
return true;		-- false=event failed, true=event succeeded	
end
--[[---------------------------------------------------------------------------------------]]

function OccButton(Eng, TIU)
printc(clGreen, "Occupancy Flags -------------------------");
local x, FColor;
for x = FIRST_BLOCK, LAST_BLOCK do
	local flag = GetFlag(x);
	if (flag == 0) then
		FColor = clBlack;
		else
		FColor = clRed;
		end;
	printc(FColor, BlockName[x] .. "(" .. x .. ") = ", flag)
	end;	
return true;
end
Function08Name, Function08Label = OccButton, "Occupancy Flags";
--[[---------------------------------------------------------------------------------------]]
-- Run the engine back to the start (in reverse)
function ResetButton()
if (ResetinProgress) then
	return;	-- already being reset
	end;
print("Return to Initial Position");
-- stop the engine
SetSpeed(0, 0, MyEngineNo, MyTIUNo);
RFIDFlag = false;
Sleep(5);
--[[-----------------------------]]
-- clear all tag handling functions
Func30 = nil;
Func51 = nil;
Func10 = nil;
Func41 = nil;
Func21 = nil;
Func20 = nil;
Func40 = nil;
Func50 = nil;
Func60 = nil;
Func61 = nil;

-- position YOUNGSTOWN_YARD switch as needed
-- depending on value of StartBlockNo
if (StartBlockNo == YOUNGSTOWN_YARD1_BLOCK) then
	Switch(NORMAL, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	end;
if (StartBlockNo == YOUNGSTOWN_YARD2_BLOCK) then
	Switch(REVERSE, Switches[YOUNGSTOWN_YARD][CHAN_NUMBER], Switches[YOUNGSTOWN_YARD][AIU_NUMBER], Switches[YOUNGSTOWN_YARD][TIU_NUMBER], 0);
	end;
--
-- start the engine
if (not StartedFlag) then
	print("Starting up the Engine");
	EngineStartUp(MyEngineNo, MyTIUNo);
	StartedFlag = true;
	Sleep(20);
	end
-- direction to reverse
SetDirection(REVERSE, 0, MyEngineNo, MyTIUNo);
BlockSpeedLimit = BlockSpeed[MyDirection][CurrentBlockNo];
SetSpeed(BlockSpeedLimit, 5, MyEngineNo, MyTIUNo);
SetCounter(C3, BlockSpeedLimit);
ResetinProgress = true;
return true
end;
Function12Name, Function12Label = ResetButton, "Initial Position";
--[[---------------------------------------------------------------------------------------]]

function ResumeSpeedButton()
	print("Resume Speed " .. BlockSpeedLimit .. " Smph");
	SetSpeed(BlockSpeedLimit, 0, MyEngineNo, MyTIUNo);	
	return true;
end
Function10Name, Function10Label = ResumeSpeedButton, "Resume Speed";
--[[---------------------------------------------------------------------------------------]]

function ReloadButton()
	print("Reloading layout4.lua");
	package.loaded["layout4"] = nil;	-- unload
	Layout4 = require([[layout4]]);	-- All of the details about the Layout
	return true;
end
Function11Name, Function11Label = ReloadButton, "Reload";
--[[---------------------------------------------------------------------------------------]]

function BeepOffButton()
	print("Beep Off");
	BeepOn = false;
	return true;
end
Function14Name, Function14Label = BeepOffButton, "Beep Off";
--[[---------------------------------------------------------------------------------------]]

function BeepOnButton()
	print("Beep On");
	BeepOn = true;
	return true;
end
Function15Name, Function15Label = BeepOnButton, "Beep On";
--[[---------------------------------------------------------------------------------------]]
function LLLButton()
	if (LLLoop) then
		print("Lionel-Ives Loop Off");
		LLLoop = false;
		else
		print("Lionel-Ives Loop On");
		LLLoop = true;
		end;
	return true;
end
Function05Name, Function05Label = LLLButton, "Lionel-Ives Loop";
--[[---------------------------------------------------------------------------------------]]