--[[
---------------------------------------------------------------------------
Remote Train Control Program for Windows

© Copyright 2020 by Mark DiVecchio

	This file is part of Remote Train Control.

	Remote Train Control is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Remote Train Control is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Remote Train Control.  If not, see <http://www.gnu.org/licenses/>.

 Home Page : http://www.silogic.com/trains/RTC_Running.html
---------------------------------------------------------------------------]]
-- Semicolons are not required in Lua code
title = "TryMultiSet Test"
require([[defines]]);
require([[functions]]);

--[[---------------------------------------------------------------------------------------]]
function setup(Engine, TIU)
	print("setup() Running");
	print("[1] == 0? " .. tostring(GetGlobalInt(1)));
	SetGlobalInt(1,55);
	print("[1] == 55? " .. tostring(GetGlobalInt(1)));
	print("[2] == 0? " .. tostring(GetGlobalInt(2)));
	result = TrySetGlobalInt(2, 33);
	print("[2] == 33? " .. tostring(GetGlobalInt(2)) .. " " .. tostring(result));
	result = TrySetGlobalInt(2, 33);
	print("[2] == 33? " .. tostring(GetGlobalInt(2)) .. " " .. tostring(result));	
	result = TrySetGlobalInt(2, 44);
	print("[2] ~= 44? " .. tostring(GetGlobalInt(2)) .. " " .. tostring(result));	
	SetGlobalInt(1,66);
	print("[1] == 66? " .. tostring(GetGlobalInt(1)));
	result = TryMultiSetGlobalInt({1, 2, 3}, 22);
	print("[1] ~= 22? " .. tostring(GetGlobalInt(1)) .. " " .. tostring(result));
	print("[2] ~= 22? " .. tostring(GetGlobalInt(2)) .. " " .. tostring(result));
	print("[3] ~= 22? " .. tostring(GetGlobalInt(3)) .. " " .. tostring(result));
	SetGlobalInt(1,0);
	SetGlobalInt(2,0);
	SetGlobalInt(3,0);	
	result = TryMultiSetGlobalInt({1, 2, 3}, 22);
	print("[1] == 22? " .. tostring(GetGlobalInt(1)) .. " " .. tostring(result));
	print("[2] == 22? " .. tostring(GetGlobalInt(2)) .. " " .. tostring(result));
	print("[3] == 22? " .. tostring(GetGlobalInt(3)) .. " " .. tostring(result));
	
	SetGlobalInt(1,0);
	SetGlobalInt(2,22);
	SetGlobalInt(3,0);	
	result = TryMultiSetGlobalInt({1, 2, 3}, 22);
	print("[1] == 22? " .. tostring(GetGlobalInt(1)) .. " " .. tostring(result));
	print("[2] == 22? " .. tostring(GetGlobalInt(2)) .. " " .. tostring(result));
	print("[3] == 22? " .. tostring(GetGlobalInt(3)) .. " " .. tostring(result));
	
	SetGlobalInt(1,0);
	SetGlobalInt(2,22);
	SetGlobalInt(3,44);	
	result = TryMultiSetGlobalInt({1, 2, 3}, 22);
	print("[1] ~= 22? " .. tostring(GetGlobalInt(1)) .. " " .. tostring(result));
	print("[2] == 22? " .. tostring(GetGlobalInt(2)) .. " " .. tostring(result));
	print("[3] ~= 22? " .. tostring(GetGlobalInt(3)) .. " " .. tostring(result));
		
	return true;		-- false=setup failed, true=setup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
function loop(Engine, TIU)
	-- Always call Sleep(X) at least once in this function if it returns true;
	print("loop() Running");
	Sleep(1);
	print("Almost Done...");
	return Stop("DONE");
	end
--[[---------------------------------------------------------------------------------------]]
function cleanup(Engine, TIU)
	-- this function is called once when the user presses the [STOP] button or the Stop() function is called
	print("cleanup() Running");
	SetGlobalInt(1,0);
	SetGlobalInt(2,0);
	SetGlobalInt(3,0);
	return true;		-- false=cleanup failed, true=cleanup succeeded
	end
--[[---------------------------------------------------------------------------------------]]
--[[---------------------------------------------------------------------------------------]]
-- set a value into the block occupancy flag
-- waits forever for the block to show as unoccupied before it sets occupancy
function SetOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return Stop("Invalid Block Number in SetOccupancy()");
	end;
if (value == 0) then
	return Stop("Value in SetOccupancy() is zero, use ClearOccupancy() instead");
	end;
local WaitTime = 120;	-- don't wait for more than this
-- set the occupance flag is it is zero or we already have ownership - use lock
while (not TrySetGlobalInt(block, value)) do
	-- wait forever
	--if (WaitTime == 0) then
	--	return Stop("Timed out wait for " .. BlockName[block] .. " to clear", true);
	--	end;
	if ((WaitTime % 30) == 0) then
		print("Waiting for " .. BlockName[block] .. " to clear");
		end;
	Sleep(2);
	WaitTime = WaitTime - 2;
	end;
print("Set occupancy flag on " .. BlockName[block] .. " to " .. value);
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- clear a block occupancy flag to zero if we have ownership
function ClearOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return Stop("Invalid Block Number in ClearOccupancy()");
	end;
if (value ~= nil and GetGlobalInt(block) == value) then
	print("Clear occupancy flag on " .. BlockName[block]);
	SetGlobalInt(block, 0);
	end;
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- set a block occupancy flag without waiting for lock or checking ownership
function OverrideOccupancy(block, value)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return Stop("Invalid Block Number in ClearOccupancy()");
	end;
if (value == nil) then
	value = 0;
	end;
print("Override occupancy flag on " .. BlockName[block]);
SetGlobalInt(block, value);
return true;
end;
--[[---------------------------------------------------------------------------------------]]
-- fetch a block occupany flag
function GetOccupancy(block)
if (block < FIRST_BLOCK or block > LAST_BLOCK) then
	return Stop("Invalid Block Number in GetOccupancy()");
	end;
return GetGlobalInt(block);
end;
--[[---------------------------------------------------------------------------------------]]